<%@ include file="/WEB-INF/jspf/page-non-authenticated.jspf" %>

<%@ page import="com.quartzdesk.common.CommonUtils" %>
<%@ page import="org.apache.commons.text.StringEscapeUtils" %>
<%@ page import="javax.servlet.http.Cookie" %>
<%@ page import="java.util.Date" %>
<%@ page import="java.util.Enumeration" %>
<%@ page import="java.util.Map" %>

<%--
  ~ Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
  ~ QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  --%>
<html>

  <body bgcolor="white">

    <style type="text/css">
      body {
        font-family: helvetica, sans-serif;
        font-size: 10pt;
        background-color: white
      }
    </style>

    <h1>Snoop page</h1>

    <b>Request Information</b>
    <pre>
  Method: ${request.method} <%= request.getMethod() %>
  Scheme: <%= request.getScheme() %>
  Character encoding: <%= request.getCharacterEncoding() %>

  Request URI: <%= StringEscapeUtils.escapeHtml4( request.getRequestURI() ) %>
  Request URL: <%= StringEscapeUtils.escapeHtml4( request.getRequestURL().toString() ) %>
  Servlet path: <%= StringEscapeUtils.escapeHtml4( request.getServletPath() ) %>
  Path info: <%= StringEscapeUtils.escapeHtml4( request.getPathInfo() ) %>
  Path translated: <%= StringEscapeUtils.escapeHtml4( request.getPathTranslated() ) %>
  Query string: <%= StringEscapeUtils.escapeHtml4( request.getQueryString() ) %>
  Request Protocol: <%= StringEscapeUtils.escapeHtml4( request.getProtocol() ) %>
  Content length: <%= request.getContentLength() %>
  Content type: <%= StringEscapeUtils.escapeHtml4( request.getContentType() ) %>

  Server name: <%= StringEscapeUtils.escapeHtml4( request.getServerName() ) %>
  Server port: <%= request.getServerPort() %>

  Remote user: <%= StringEscapeUtils.escapeHtml4( request.getRemoteUser() ) %>
  Remote address: <%= StringEscapeUtils.escapeHtml4( request.getRemoteAddr() ) %>
  Remote host: <%= StringEscapeUtils.escapeHtml4( request.getRemoteHost() ) %>

  Local address: <%= request.getLocalAddr() %>
  Local port: <%= request.getLocalPort() %>
  Local name: <%= request.getLocalName() %>

  Is secure: <%= request.isSecure() %>
  Authorization scheme: <%= request.getAuthType() %>

  Requested session ID: <%= StringEscapeUtils.escapeHtml4( request.getRequestedSessionId() ) %>
  Is requested session ID valid: <%= request.isRequestedSessionIdValid() %>
  Session ID: <%= session != null ? StringEscapeUtils.escapeHtml4( session.getId() ) : "N/A" %>
  Session created at: <%= session != null ? new Date( session.getCreationTime() ).toString() : "N/A" %>
  Session last accessed at: <%= session != null ? new Date( session.getLastAccessedTime() ).toString() : "N/A" %>
  Session max inactive interval: <%= session != null ? Integer.toString( session.getMaxInactiveInterval() ) :
        "N/A" %>
  Session is new: <%= session != null ? Boolean.toString( session.isNew() ) : "N/A"%>

  Locale: <%= request.getLocale() %>
  User agent: <%= StringEscapeUtils.escapeHtml4( request.getHeader( "User-Agent" ) ) %>
</pre>
    <hr>

    <b>Request headers:</b>

    <p>
      <code>
        <%
          for ( Enumeration<String> headerNames =
              (Enumeration<String>) request.getHeaderNames(); headerNames.hasMoreElements(); )
          {
            String headerName = headerNames.nextElement();
            Object headerValue = request.getHeader( headerName );
        %>
        <%= StringEscapeUtils.escapeHtml4( headerName ) %> = <%= StringEscapeUtils.escapeHtml4(
          CommonUtils.safeToString( headerValue ) ) %><br/>
        <%
          }
        %>
      </code>
    </p>
    <hr>


    <b>Cookies:</b>

    <p>
      <code>
        <%
          if ( request.getCookies() != null )
          {
            for ( Cookie cookie : request.getCookies() )
            {
        %>
        name=<%= StringEscapeUtils.escapeHtml4( cookie.getName() ) %>, value=<%= StringEscapeUtils.escapeHtml4(
          cookie.getValue() ) %>, path=<%= StringEscapeUtils.escapeHtml4( cookie.getPath() ) %>,
        domain=<%= StringEscapeUtils.escapeHtml4( cookie.getDomain() )%>, maxAge=<%= cookie.getMaxAge() %>,
        version=<%= cookie.getVersion()%>,
        secure=<%= cookie.getSecure() %>, comment=<%= StringEscapeUtils.escapeHtml4( cookie.getComment() ) %><br/>
        <%
            }
          }
        %>
      </code>
    </p>
    <hr>


    <b>Request parameters:</b>

    <p>
        <%
    for ( Map.Entry<String, String[]> entry : ( (Map<String, String[]>) request.getParameterMap() ).entrySet() )
    {
      String paramName = entry.getKey();
      String[] paramValues = entry.getValue();

      StringBuilder paramValuesStr = new StringBuilder();
      for ( int i = 0; i < paramValues.length; i++ )
      {
        paramValuesStr.append( paramValues[i] );
        if ( i < paramValues.length - 1 ) paramValuesStr.append( ", " );
      }
  %>
    <pre><%= StringEscapeUtils.escapeHtml4( paramName ) %> = <%= StringEscapeUtils.escapeHtml4(
        CommonUtils.safeToString( paramValuesStr ) ) %></pre>
    <%
      }
    %>
    </p>
    <hr>


    <b>Request attributes:</b>

    <p>
        <%
    for ( Enumeration<?> reqAttrNames = request.getAttributeNames(); reqAttrNames.hasMoreElements(); )
    {
      String reqAttrName = (String) reqAttrNames.nextElement();
      Object reqAttrValue = request.getAttribute( reqAttrName );
  %>
    <pre><%= StringEscapeUtils.escapeHtml4(
        reqAttrName ) %> (<%= reqAttrValue.getClass().getName() %>) = <%= StringEscapeUtils.escapeHtml4(
        CommonUtils.safeToString( reqAttrValue ) ) %></pre>
    <%
      }
    %>
    </p>
    <hr>


    <b>Session attributes:</b>

    <p>
        <%
    if ( session != null )
    {
      for ( Enumeration<?> sesAttrNames = session.getAttributeNames(); sesAttrNames.hasMoreElements(); )
      {
        String sesAttrName = (String) sesAttrNames.nextElement();
        Object sesAttrValue = session.getAttribute( sesAttrName );
  %>
    <pre><%= StringEscapeUtils.escapeHtml4(
        sesAttrName ) %> (<%= sesAttrValue.getClass().getName() %>) = <%= StringEscapeUtils.escapeHtml4(
        CommonUtils.safeToString( sesAttrValue ) ) %></pre>
    <%
        }
      }
    %>
    </p>
    <hr>


    <b>Context attributes:</b>

    <p>
        <%
    for ( Enumeration<?> ctxAttrNames = application.getAttributeNames(); ctxAttrNames.hasMoreElements(); )
    {
      String ctxAttrName = (String) ctxAttrNames.nextElement();
      Object ctxAttrValue = application.getAttribute( ctxAttrName );
  %>
    <pre><%= StringEscapeUtils.escapeHtml4(
        ctxAttrName ) %> (<%= ctxAttrValue.getClass().getName() %>) = <%= StringEscapeUtils.escapeHtml4(
        CommonUtils.safeToString( ctxAttrValue ) ) %></pre>
    <%
      }
    %>
    </p>
    <hr>


    <b>System properties:</b>

    <p>
        <%
    for ( Enumeration<?> propNames = System.getProperties().keys(); propNames.hasMoreElements(); )
    {
      String propName = (String) propNames.nextElement();
      String propValue = System.getProperty( propName );
  %>
    <pre><%= StringEscapeUtils.escapeHtml4( propName ) %> = <%= StringEscapeUtils.escapeHtml4( propValue ) %></pre>
    <%
      }
    %>
    </p>

  </body>
</html>
