/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Encapsulates connection related data.
 */
Ext.define( 'QDesk.ConnectionContext', {
  requires : [
    'QDesk.Log'
  ],

  /**
   * Connection ID.
   */
  connectionId : undefined,

  /**
   * Connection node path in the Connections tree view.
   */
  connectionNodePath : undefined,

  /**
   * JvmAgentInfoAccessor instance.
   */
  jvmAgentInfo : undefined,

  /**
   * SchedulerInfoAccessor instance.
   */
  schedulerInfo : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.connectionNodePath )
      Ext.Error.raise( 'Required config parameter missing: connectionNodePath' );

    if ( !config.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: jvmAgentInfo' );

    if ( !config.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: schedulerInfo' );

    me.connectionId = config.connectionId;
    me.connectionNodePath = config.connectionNodePath;
    me.jvmAgentInfo = config.jvmAgentInfo;
    me.schedulerInfo = config.schedulerInfo;

    me.callParent( [ config ] );
  },


  /**
   * Returns the connection ID.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    var me = this;
    return me.connectionId;
  },


  /**
   * Returns the connection node path.
   * @return the connection node path.
   */
  getConnectionNodePath : function ()
  {
    var me = this;
    return me.connectionNodePath;
  },


  /**
   * Returns the connection node path.
   * @return the connection node path.
   */
  setConnectionNodePath : function ( connectionNodePath )
  {
    var me = this;
    me.connectionNodePath = connectionNodePath;
  },


  /**
   * Returns the JVM agent info.
   * @return the JVM agent info.
   */
  getJvmAgentInfo : function ()
  {
    var me = this;
    return me.jvmAgentInfo;
  },


  /**
   * Returns the scheduler info.
   * @return the scheduler info.
   */
  getSchedulerInfo : function ()
  {
    var me = this;
    return me.schedulerInfo;
  }
} );
