/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Encapsulates information on the system the QuartzDesk webapp is running on.
 */
Ext.define( 'QDesk.System', {

  requires : [
    'QDesk.Log'
  ],

  singleton : true,

  /**
   * The system locale (QDesk.model.common.Locale instance).
   */
  locale : null,

  /**
   * The system time zone (QDesk.model.common.TimeZone instance).
   */
  timeZone : null,

  /**
   * List of available system time zones (QDesk.model.common.TimeZone instances).
   */
  availableTimeZones : null,


  /**
   * Returns the system locale.
   * @returns the system locale.
   */
  getLocale : function ()
  {
    var me = this;
    return me.locale;
  },


  /**
   * Returns the system time zone.
   * @returns the system time zone.
   */
  getTimeZone : function ()
  {
    var me = this;
    return me.timeZone;
  },


  /**
   * Returns the available system time zones.
   * @returns the available system time zones.
   */
  getAvailableTimeZones : function ()
  {
    var me = this;
    return me.availableTimeZones;
  },


  /**
   * Updates the system information using the provided system info object with the
   * following structure:
   * {
   *    locale : (QDesk.model.common.Locale)
   *    timeZone : (QDesk.model.common.TimeZone)
   *    availableTimeZones : (list of QDesk.model.common.TimeZone)
   * }
   *
   * @param systemInfo a system info.
   */
  update : function ( systemInfo )
  {
    var me = this;

    me.locale = systemInfo.locale;
    me.timeZone = systemInfo.timeZone;
    me.availableTimeZones = systemInfo.availableTimeZones;
  }
} );
