/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the About window.
 */
Ext.define( 'QDesk.controller.command.About', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.license.LicenseType',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.about.AboutWindow'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.About.maskAboutDataLoad' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.aboutActionData,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var aboutWindow = Ext.widget( 'AboutWindow' );
            var aboutForm = aboutWindow.down( '#aboutForm' );
            // assert aboutForm != null

            var record = Ext.create( 'QDesk.model.gui.AboutData', json.data );
            aboutForm.loadRecord( record );

            /*
             var buildDateField = aboutForm.down( '#buildDate' );
             buildDateField.setValue( Ext.util.Format.date( record.get( 'buildDate' ), 'F j, Y' ) );
             */

            var productEdition = record.get( 'productEdition' );
            var licenseType = record.get( 'licenseType' );

            if ( licenseType == QDesk.model.license.LicenseType.TEMPORARY )
            {
              // show license expiry date
              var licenseExpiryText = record.get( 'licenseExpiryText' );

              var licenseExpiryTextField = aboutForm.down( '#licenseExpiryText' );
              licenseExpiryTextField.setValue( licenseExpiryText );
              licenseExpiryTextField.setVisible( true );
            }
            else if ( licenseType == QDesk.model.license.LicenseType.PERPETUAL )
            {
              if ( productEdition == QDesk.model.license.ProductEdition.LITE )
              {
                // lite edition: no SW Maintenance end date
              }
              else
              {
                // standard, enterprise edition:  show SW maintenance end date
                var swMaintenanceExpiryText = record.get( 'swMaintenanceExpiryText' );

                var swMaintenanceExpiryTextField = aboutForm.down( '#swMaintenanceExpiryText' );
                swMaintenanceExpiryTextField.setValue( swMaintenanceExpiryText );
                swMaintenanceExpiryTextField.setVisible( true );
              }
            }

            // // set and unhide the license expiration field in case the license expires
            // var licenseExpiresInDays = record.get( 'licenseExpiresInDays' );
            // if ( licenseExpiresInDays != null )
            // {
            //   if ( licenseExpiresInDays > 0 )
            //   {
            //     var licenseExpiresInDaysField = aboutForm.down( '#licenseExpiresInDays' );
            //
            //     licenseExpiresInDaysField.setValue(
            //         QDesk.Msg.get( 'QDesk.view.about.AboutForm.textLicenseExpiresIn' ) +
            //         ' ' + record.get( 'licenseExpiresInDaysText' ) );
            //
            //     licenseExpiresInDaysField.setVisible( true );
            //   }
            //   else
            //   {
            //     aboutForm.down( '#licenseExpiredText' ).setVisible( true );
            //   }
            // }

            // set the copyright message
            var copyrightField = aboutForm.down( '#copyright' );
            var copyrightYear = record.get( 'copyrightYear' );
            copyrightField.setValue( QDesk.Msg.get( 'QDesk.copyright', copyrightYear ? copyrightYear : ' ' ) );

            aboutWindow.show();

            // about window gets closed when the user clicks anywhere in the browser window
            Ext.getBody().on( 'click', function ()
            {
              aboutWindow.close();
            } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }
      }
    } );
  }
} );
