/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Displays the specified URL in a tab in the Main Tab Panel. If there is a tab with the specified tabId, the tab is
 * activated (and not reloaded), otherwise a new tab is added to the Main Tab Panel.
 */
Ext.define( 'QDesk.controller.command.ViewUrl', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.view.work.ViewUrlTab'
  ],

  /**
   * Required ID of of the URL to display.
   */
  id : undefined,

  /**
   * Required title for the tab which displays the URL.
   */
  tabTitle : undefined,

  /**
   * Required icon for the tab which displays the URL.
   */
  tabIcon : undefined,

  /**
   * Required URL to display in the tab.
   */
  url : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.id )
      Ext.Error.raise( 'Required config parameter missing: id' );

    if ( !config.tabTitle )
      Ext.Error.raise( 'Required config parameter missing: tabTitle' );

    if ( !config.tabIcon )
      Ext.Error.raise( 'Required config parameter missing: tabIcon' );

    if ( !config.url )
      Ext.Error.raise( 'Required config parameter missing: url' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var viewTabItemId = 'viewUrlTabPanel-' + me.id;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
    var viewUrlTab = mainTabPanel.down( '#' + viewTabItemId );

    // if there is no tab panel for the scheduler, create a new one and add it to the mainTabPanel, otherwise
    // just activate the existing tab panel
    if ( viewUrlTab == null )
    {
      viewUrlTab = Ext.widget( 'ViewUrlTab', {
        qdesk : {
          itemId : viewTabItemId,
          title : me.tabTitle,
          icon : me.tabIcon,
          url : me.url
        }
      } );

      mainTabPanel.add( viewUrlTab );
    }
    else
    {
      // panel already exists - we will only make it active
    }

    mainTabPanel.setActiveTab( viewUrlTab );
  }
} );
