/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Audit Trail grid in the Audit Trail tab.
 */
Ext.define( 'QDesk.controller.command.audit.RefreshAuditTrailGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.util.license.LicensedProductFeatures'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    if ( licensedFeatures.AuditTrail )
    {
      var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
      var auditTrailTab = mainTabPanel.down( 'AuditTrailTab' );

      var auditTrailGrid = auditTrailTab.down( '#auditTrailGrid' );
      var auditTrailGridStore = auditTrailGrid.getStore();

      auditTrailGridStore.getProxy().url = QDesk.util.constant.ActionUrl.auditTrailActionList;

      var extraStoreParams = {};

      var fetchPeriod = auditTrailGrid.down( '#fetchPeriod' ).getValue();
      Ext.apply( extraStoreParams, {
        fetchPeriod : Ext.encode( fetchPeriod )
      } );

      var maxFetchSize = auditTrailGrid.down( '#maxFetchSizeSelect' ).getValue();
      if ( maxFetchSize != -1 )  // -1 == unlimited
      {
        Ext.apply( extraStoreParams, {
          maxFetchSize : maxFetchSize
        } );
      }

      // TODO: check the index is enabled
      var auditTrailIndexEnabled = false;

      // if audit trail index enabled, we add the filter query text param
      if ( auditTrailIndexEnabled )
      {
        var filterQueryText = auditTrailGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      auditTrailGridStore.getProxy().extraParams = extraStoreParams;

      auditTrailGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
