/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the Audit Trail tab (AuditTrailTab).
 */
Ext.define( 'QDesk.controller.command.audit.ViewAuditTrail', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.audit.InitializeAuditTrailTab',
    'QDesk.controller.command.audit.RefreshAuditTrailTab',

    'QDesk.view.work.audit.AuditTrailTab'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
    var auditTrailTab = mainTabPanel.down( 'AuditTrailTab' );

    // if the AuditTrailTab is open, activate it
    if ( auditTrailTab )
    {
      mainTabPanel.setActiveTab( auditTrailTab );
    }
    else
    {
      me.mask( QDesk.Msg.get( 'QDesk.controller.command.audit.ViewAuditTrail.maskMsg' ) );

      // deferred so that the mask message gets displayed
      Ext.defer( function ()
      {
        try
        {
          auditTrailTab = Ext.widget( 'AuditTrailTab', {} );

          auditTrailTab.on( 'activate', function ()
          {
            /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
            if ( !auditTrailTab.getInitialized() )
            {
              Ext.create( 'QDesk.controller.command.audit.InitializeAuditTrailTab', {} ).execute();

              Ext.create( 'QDesk.controller.command.audit.RefreshAuditTrailTab', {} ).execute();
            }
          } );

          auditTrailTab = mainTabPanel.add( auditTrailTab );
          mainTabPanel.setActiveTab( auditTrailTab );
        }
        finally
        {
          me.unmask();
        }
      }, 100, me );
    }
  }
} );
