/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Scheduler connection related utility methods.
 */
Ext.define( 'QDesk.controller.command.connection.ConnectionUtils', {

  requires : [
    'QDesk.Msg',
    'QDesk.SchedulerInfoAccessor',

    'QDesk.model.scheduler.SchedulerConnection',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.util.license.LicensedProduct'
  ],

  singleton : true,


  /**
   * Returns the item ID of the scheduler view panel for the specified connection.
   * @param connectionId a connection ID.
   */
  getSchedulerViewItemId : function ( connectionId )
  {
    return 'schedulerView-' + connectionId;
  },


  /**
   * Returns the scheduler view (panel) for the specified Quartz connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler view.
   */
  getSchedulerView : function ( connectionId )
  {
    var viewItemId = QDesk.controller.command.connection.ConnectionUtils.getSchedulerViewItemId( connectionId );
    var views = Ext.ComponentQuery.query( '#mainTabPanel > #' + viewItemId );
    return Ext.isEmpty( views ) ? null : views[ 0 ];
  },


  // /**
  //  * Returns the QuartzDesk JVM Agent API warning message to show in the GUI if a QuartzDesk JVM Agent
  //  * installation-related problem has been detected.
  //  * @param connectionCtx a {@link ConnectionContext} instance.
  //  * @returns the warning message.
  //  */
  // getJvmAgentWarningMessage : function ( connectionCtx )
  // {
  //   var schedulerInfo = connectionCtx.getSchedulerInfo();
  //   var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
  //
  //   if ( jvmAgentInfo.isReady() )
  //     return null;
  //
  //   if ( jvmAgentInfo.isDetected() )
  //   {
  //     if ( !jvmAgentInfo.isCompatibleVersion() )
  //     {
  //       return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionMsg',
  //           QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
  //           QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ),
  //           QDesk.util.FormatUtils.formatVersion( QDesk.util.license.LicensedProduct.productVersion ) );
  //     }
  //     else if ( !jvmAgentInfo.isCompatibleEdition() )
  //     {
  //       return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionMsg',
  //           QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
  //           jvmAgentInfo.getEdition(),
  //           QDesk.util.license.LicensedProduct.productEdition );
  //     }
  //     else
  //     {
  //       return null;
  //     }
  //   }
  //   else
  //   {
  //     return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedMsg',
  //         QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
  //   }
  // },


  /**
   * Returns the QuartzDesk JVM Agent API warning tooltip to show in the GUI if a QuartzDesk JVM Agent
   * installation-related problem has been detected.
   * @param connectionCtx a {@link ConnectionContext} instance.
   * @returns the warning message.
   */
  getJvmAgentWarningTooltip : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  // /**
  //  * Returns the QuartzDesk Public API warning message to show in the GUI if a QuartzDesk Public API
  //  * installation-related problem has been detected.
  //  * @param connectionCtx a {@link ConnectionContext} instance.
  //  * @returns the warning message.
  //  */
  // getPublicApiWarningMessage : function ( connectionCtx )
  // {
  //   var schedulerInfo = connectionCtx.getSchedulerInfo();
  //
  //   if ( schedulerInfo.isPublicApiReady() )
  //     return null;
  //
  //   if ( schedulerInfo.isPublicApiDetected() )
  //   {
  //     if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
  //     {
  //       return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyMsg',
  //           QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
  //     }
  //     else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
  //     {
  //       var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
  //
  //       return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionMsg',
  //           QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
  //           QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
  //           QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ) );
  //     }
  //     else
  //     {
  //       return null;
  //     }
  //   }
  //   else
  //   {
  //     return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedMsg',
  //         QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
  //   }
  // },


  /**
   * Returns the QuartzDesk Public API warning tooltip to show in the GUI if a QuartzDesk Public API
   * installation-related problem has been detected.
   * @param connectionCtx a {@link ConnectionContext} instance.
   * @returns the warning message.
   */
  getPublicApiWarningTooltip : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the "The feature is not available in ... edition.' warning message.
   * @returns the warning message.
   */
  getNotLicensedWarningMessage : function ()
  {
    return QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
        QDesk.util.license.LicensedProduct.productName )
  },


  /**
   * Returns the "The feature is not available in ... edition.' warning tooltip.
   * @returns the warning message.
   */
  getNotLicensedWarningTooltip : function ()
  {
    return QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedTooltip',
        QDesk.util.license.LicensedProduct.productName )
  },


  /**
   * Returns the node ID of the specified connection.
   * @param connectionId a connection ID.
   * @return the node ID of the specified connection.
   */
  getConnectionNodeId : function ( connectionId )
  {
    return 'connection-' + connectionId;
  },


  /**
   * Returns the name of the connection with the specified ID.
   * @param connectionId a connection ID.
   */
  getConnectionName : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'name' );
  },


  /**
   * Returns the title of the connection panel for the connection with the specified ID.
   * @param connectionId a connection ID.
   */
  getConnectionPanelTitle : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified connection, or null if not found.
   * @param connectionId a connection ID.
   * @return the parent folder ID, or null.
   */
  getConnectionParentFolderId : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode && connectionNode.parentNode ? connectionNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the node ID of the specified folder.
   * @param folderId a folder ID.
   * @return the node ID of the specified folder.
   */
  getFolderNodeId : function ( folderId )
  {
    return folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID ?
        QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID : 'folder-' + folderId;
  },


  /**
   * Returns the name of the folder with the specified ID.
   * @param folderId a folder ID.
   * @return the folder name.
   */
  getFolderName : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.get( 'name' );
  },


  /**
   * Returns the path of the folder with the specified ID.
   * @param folderId a folder ID.
   * @return the folder path.
   */
  getFolderPath : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.getPath();
  },


  /**
   * Returns the ID of the parent folder of the specified folder, or null if not found.
   * @param folderId a folder ID.
   * @return the parent folder ID, or null.
   */
  getFolderParentFolderId : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode && folderNode.parentNode ? folderNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the scheduler type for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler type (QDesk.model.scheduler.SchedulerType value).
   */
  getSchedulerType : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'schedulerType' );
  },


  /**
   * Creates a new {@link QDesk.model.scheduler.SchedulerConnection} instance and populates it
   * from the specified EditSchedulerConnectionWindow window.
   * @param schedulerType a scheduler type ({@link QDesk.model.scheduler.SchedulerType} enum value).
   * @param window an EditSchedulerConnectionWindow window.
   * @returns the created {QDesk.model.scheduler.SchedulerConnection} instance.
   */
  createSchedulerConnection : function ( schedulerType, window )
  {
    var me = this;

    var connectionDataFormPanel = window.down( '#connectionDataForm' );

    var useServiceParams = connectionDataFormPanel.down( '#serviceParamsRadio' ).getValue();
    var useServiceUrl = connectionDataFormPanel.down( '#serviceUrlRadio' ).getValue();

    var form = connectionDataFormPanel.getForm();
    var fieldValues = form.getFieldValues();

    var connection = Ext.create( 'QDesk.model.scheduler.SchedulerConnection',
        {
          id : fieldValues.id,
          //createdAt : fieldValues.createdAt,
          //updatedAt : fieldValues.updatedAt,
          name : fieldValues.name,
          remoteJmxService : {
            protocol : useServiceParams ? fieldValues.protocol : null,
            host : useServiceParams ? fieldValues.host : null,
            port : useServiceParams ? fieldValues.port : null,
            serviceUrl : useServiceUrl ? fieldValues.serviceUrl : null,
            authentication : {
              username : fieldValues.username ? fieldValues.username : null,
              password : fieldValues.password ? fieldValues.password : null
            },
            secure : fieldValues.secure == null ? false : fieldValues.secure
          },
          schedulerType : schedulerType,
          schedulerObjectName : fieldValues.schedulerObjectName
        }
    );

    if ( fieldValues.username === '' && fieldValues.password === '' )
      connection.get( 'remoteJmxService' ).authentication = null;

    /*
     * Collect config entry values.
     */
    var configEntries = [];
    Ext.each( Ext.ComponentQuery.query( 'form', window ),
        function ( formPanel )
        {
          // skip connectionDataForm - config values are on other forms
          if ( formPanel.getItemId() !== 'connectionDataForm' )
          {
            Ext.each( Ext.ComponentQuery.query( 'field', formPanel ),
                function ( field )
                {
                  // skip all displayfields
                  if ( !field.isXType( 'displayfield' ) )
                  {
                    configEntries.push( {
                      name : field.getName(),
                      value : field.getValue()
                    } );
                  }
                } );
          }
        } );
    connection.set( 'configEntry', configEntries );

    return connection;
  },


  handleTestResponse : function ( json, schedulerType )
  {
    if ( json.scheduler )
    {
      // Lite edition
      if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.model.license.ProductEdition.LITE )
      {
        QDesk.util.UxUtils.showInfoDialog(
            QDesk.Msg.get(
                'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessMsg' ) );
      }
      else
      {
        // Standard and Enterprise editions
        if ( json.jvmAgentReady )
        {
          //
          // jvm agent is ready
          //
          if ( json.scheduler.instrumented )
          {
            //
            // scheduler has been instrumented
            //
            var publicApiInfo = json.scheduler.publicApiInfo;

            if ( publicApiInfo.ready )
            {
              // public API is ready
              QDesk.util.UxUtils.showInfoDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiReadyMsg',
                      QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
            }
            else
            {
              // public API is not ready
              if ( publicApiInfo.detected )
              {
                if ( !publicApiInfo.loadedCorrectly )
                {
                  QDesk.util.UxUtils.showWarningDialog(
                      QDesk.Msg.get(
                          'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiNotLoadedCorrectlyMsg',
                          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
                }
                else if ( !publicApiInfo.compatibleVersion )
                {
                  QDesk.util.UxUtils.showWarningDialog(
                      QDesk.Msg.get(
                          'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiIncompatibleVersionMsg',
                          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ),
                          QDesk.util.FormatUtils.formatVersion( publicApiInfo.version ),
                          QDesk.util.FormatUtils.formatVersion( json.jvmAgentVersion ) ) );
                }
              }
              else
              {
                QDesk.util.UxUtils.showWarningDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentReadyPublicApiNotDetectedMsg',
                        QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
              }
            }
          }
          else
          {
            //
            // scheduler has NOT been instrumented
            //
            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessSchedulerNotInstrumentedMsg',
                    QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
          }
        }
        else
        {
          //
          // jvm agent is not ready
          //
          if ( json.jvmAgentDetected )
          {
            // jvm agent detected

            if ( !json.jvmAgentCompatibleVersion )
            {
              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentIncompatibleVersionMsg',
                      QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ),
                      QDesk.util.FormatUtils.formatVersion( json.jvmAgentVersion ),
                      QDesk.util.FormatUtils.formatVersion(
                          QDesk.util.license.LicensedProduct.productVersion ) ) );
            }
            else if ( !json.jvmAgentCompatibleEdition )
            {
              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentIncompatibleEditionMsg',
                      QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ),
                      json.jvmAgentEdition,
                      QDesk.util.license.LicensedProduct.productEdition ) );
            }
          }
          else
          {
            //
            // jvm agent not detected
            //
            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.connection.ConnectionUtils.connectionSuccessJvmAgentNotDetectedMsg',
                    QDesk.util.EnumUtils.getSchedulerTypeName( schedulerType ) ) );
          }
        }
      }
    }
    else
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get(
              'QDesk.controller.command.connection.ConnectionUtils.noSchedulerFoundMsg' ) );
    }
  }
} );
