/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens an editor to add a new message channel profiles folder.
 */
Ext.define( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',

    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.message.channel.FolderPanel'
  ],

  /*
   * Required parameter with the ID of the parent folder.
   */
  parentFolderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.parentFolderId )
      Ext.Error.raise( 'Required config parameter missing: parentFolderId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

    var formPanel = Ext.widget( 'FolderPanel' );

    // focus the name text field
    formPanel.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      QDesk.util.UxUtils.focus( nameField, true );
    } );

    QDesk.util.MessageChannelProfileUtils.addAndShowPanel( formPanel );

    // Apply button
    formPanel.down( '#saveButton' ).on( 'click', function ()
    {
      me._submitForm();
    } );
  },


  _submitForm : function ()
  {
    var me = this;

    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();

    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    var fieldValues = form.getFieldValues();
    var folderName = fieldValues[ 'name' ];

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionAddFolder,
      params : {
        parentFolderId : me.parentFolderId,
        folderName : folderName
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder.waitMsg',
          Ext.String.htmlEncode( folderName ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

          me._refreshTargetFolder();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.AddMessageChannelProfileFolder.successMsg',
                  Ext.String.htmlEncode( folderName ) ), me.success );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  /**
   * Refreshes the parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
      folderId : me.parentFolderId
    } ).execute();
  }
} );
