/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the specified message channel profiles folder.
 */
Ext.define( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',

    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.message.channel.FolderPanel'
  ],

  /*
   * Required folder record ({@link QDesk.model.message.MessageChannelProfilesTreeNode} instance)
   * to edit.
   */
  folder : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folder )
      Ext.Error.raise( 'Required config parameter missing: folder' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

    var formPanel = Ext.widget( 'FolderPanel' );

    // focus the name text field
    formPanel.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      // QDesk.util.UxUtils.focus( nameField, true );
    } );

    QDesk.util.MessageChannelProfileUtils.addAndShowPanel( formPanel );

    // Apply button
    formPanel.down( '#saveButton' ).on( 'click', function ()
    {
      me._submitForm();
    } );

    formPanel.populate.apply( formPanel, [ me.folder.data ] );
  },


  _submitForm : function ()
  {
    var me = this;

    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();
    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    var fieldValues = form.getFieldValues();
    var folderId = fieldValues[ 'id' ];
    var folderName = fieldValues[ 'name' ];

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionUpdateFolder,
      params : {
        folderId : folderId,
        folderName : folderName
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder.waitMsg',
          Ext.String.htmlEncode( folderName ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          //QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

          me._refreshTargetFolder();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfileFolder.successMsg',
                  Ext.String.htmlEncode( folderName ) ), me.success );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  /**
   * Refreshes the parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    var folderId = me.folder.get( 'folderId' );
    var parentFolderId = QDesk.util.MessageChannelProfileUtils.getFolderParentFolderId(
        folderId );
    if ( parentFolderId )
    {
      Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
        folderId : parentFolderId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Parent folder ID for profile ID: ' + me.profileId + ' not found.' );
    }
  }
} );
