/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Moves the message channel profile to a new target folder.
 */
Ext.define( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',

    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required ID of the folder the profile should be moved to.
   */
  targetFolderId : undefined,

  /*
   * Required ID of the profile to move.
   */
  profileId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.targetFolderId )
      Ext.Error.raise( 'Required config parameter missing: targetFolderId' );

    if ( !config.profileId )
      Ext.Error.raise( 'Required config parameter missing: profileId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

    var profileName = QDesk.util.MessageChannelProfileUtils.getProfileName( me.profileId );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile.maskMsg',
        Ext.String.htmlEncode( profileName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionMoveProfile,
      params : {
        targetFolderId : me.targetFolderId,
        profileId : me.profileId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._refreshFolder( me.targetFolderId );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.message.channel.MoveMessageChannelProfile.successMsg',
                    Ext.String.htmlEncode( profileName ) ), me.success );
          }
          else
          {
            me._refreshFolder( null );

            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          me._refreshFolder( null );

          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  /**
   * Refreshes the specified folder.
   */
  _refreshFolder : function ( folderId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
      folderId : folderId
    } ).execute();
  }
} );
