/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Generates the SNMP MIB configuration file.
 */
Ext.define( 'QDesk.controller.command.message.channel.snmp.GenerateMib', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required SNMP message channel (QDesk.model.message.channel.MessageChannel).
   */
  channel : undefined,

  /*
   * Required SNMP message channel configuration.
   */
  channelConfig : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.channel )
      Ext.Error.raise( 'Required config parameter missing: channel' );

    if ( !config.channelConfig )
      Ext.Error.raise( 'Required config parameter missing: channelConfig' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.snmp.GenerateMib.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.snmpGenerateMib,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfoDialog(
                QDesk.Msg.get( 'QDesk.controller.command.message.channel.snmp.GenerateMib.successMsg' ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
