/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a modal window with the list of all triggers and prompts the user to select a trigger.
 * The selected QuartzTrigger is passed to the success handler if defined.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.SelectTrigger', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.quartz.QuartzUtils',

    'QDesk.view.work.scheduler.quartz.SelectTriggerWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required title for the job-selection window.
   */
  title : undefined,

  /**
   * Optional group name of the trigger to preselect in the trigger selection window when it opens.
   */
  preselectedTriggerGroupName : undefined,

  /**
   * Optional name of the trigger to preselect in the trigger selection window when it opens.
   */
  preselectedTriggerName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.title )
      Ext.Error.raise( 'Required config parameter missing: title' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

//    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectTrigger.maskLoadMsg' ) );

    var window = Ext.widget( 'QuartzSelectTriggerWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        title : me.title,
        preselectedTriggerGroupName : me.preselectedTriggerGroupName,
        preselectedTriggerName : me.preselectedTriggerName
      }
    } );

    me._bindEventHandlers( window );

    window.show();
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );
    var selectTriggerGrid = formPanel.down( '#selectTriggerGrid' );
    var actionButton = formPanel.down( '#actionButton' );

    // the action button is disabled by default
    actionButton.disable();


    // job selection
    selectTriggerGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          actionButton.setDisabled( selectedRecords.length == 0 );
        } );


    // Save/Update button
    actionButton.on( 'click',
        function ()
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            if ( selectTriggerGrid.getSelectionModel().hasSelection() )
            {
              var triggerWithjobDetail = selectTriggerGrid.getSelectionModel().getSelection()[ 0 ];

              window.close();

              var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger',
                  triggerWithjobDetail.get( 'trigger' ) );

              me.success.apply( me, [ trigger ] );
            }
          }
        } );
  }
} );