/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits an existing job chain with the specified ID.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.SchedulerType',

    'QDesk.util.CommonUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',
    'QDesk.util.quartz.QuartzJobChainUtils',

    'QDesk.view.work.scheduler.common.chain.EditJobChainFormMode',
    'QDesk.view.work.scheduler.quartz.chain.EditJobChainWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required ID of the job chain to edit.
   */
  jobChainId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.jobChainId )
      Ext.Error.raise( 'Required config parameter missing: jobChainId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'QuartzEditJobChainWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        mode : QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.EDIT
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      // QDesk.util.UxUtils.focus( nameField, true );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    /* Load the form data */
    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain.maskLoadMsg' ) );

    formPanel.load( {
      url : QDesk.util.constant.ActionUrl.quartz.jobChainActionGet,
      scope : me,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        jobChainId : me.jobChainId
      },
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          formPanel.loadSpecialFields( json.data );

          me.unmask();
          window.show();
        }
        else
        {
          me.unmask();
          window.close();
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        me.unmask();
        window.close();
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();
    var jobChainTargets = formPanel.getJobChainTargetsData();

    var jobChain = QDesk.util.quartz.QuartzJobChainUtils.createJobChain( fieldValues, jobChainTargets );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.jobChainActionUpdate,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        jobChain : QDesk.util.CommonUtils.encode( jobChain )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain.waitMsg',
          Ext.String.htmlEncode( fieldValues.name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain.successMsg',
                  Ext.String.htmlEncode( fieldValues.name ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.JOB_CHAIN_UPDATED, {
                  connectionId : me.connectionCtx.getConnectionId(),
                  jobChain : jobChain
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  }
} );
