/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Job Chains tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.RefreshJobChainsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.controller.command.scheduler.quartz.chain.RefreshJobJobChainsGrid',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshTriggerJobChainsGrid'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var jobChainsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobChainsTab(
        me.connectionCtx.getConnectionId() );

    var activeView = jobChainsTab.getActiveView();

    if ( activeView.getItemId() == 'schedulerJobChainsView' )
    {
      // refreshes the Scheduler-Wide job chains grid
      Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeView.getItemId() == 'jobJobChainsView' )
    {
      // refreshes the Job-Specific job chains grid
      Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshJobJobChainsGrid', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeView.getItemId() == 'triggerJobChainsView' )
    {
      // refreshes the Trigger-Specific job chains grid
      Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshTriggerJobChainsGrid', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Unrecognized view item ID: ' + activeView.getItemId() );
    }
  }
} );
