/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Trigger-Specific Job Chains grid under the top-level Job Chains tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.RefreshTriggerJobChainsGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var jobChainsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobChainsTab(
          me.connectionCtx.getConnectionId() );

      var jobChainsGrid = jobChainsTab.down( '#triggerJobChainsView #jobChainsGrid' );

      var jobChainsGridStore = jobChainsGrid.getStore();

      jobChainsGridStore.getProxy().url =
          QDesk.util.constant.ActionUrl.quartz.jobChainActionListForTriggers;

      var extraStoreParams = {
        connectionId : me.connectionCtx.getConnectionId()
      };

      // if job chains index enabled, we add the filter query text param
      if ( me.connectionCtx.getJvmAgentInfo().isQuartzJobChainsIndexEnabled() )
      {
        var filterQueryText = jobChainsGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      jobChainsGridStore.getProxy().extraParams = extraStoreParams;

      jobChainsGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
