/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the scheduler Execution History grid in the scheduler Execution History panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var execHistoryTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getExecHistoryTab(
          me.connectionCtx.getConnectionId() );

      var execHistoryGrid = execHistoryTab.down( '#execHistoryGrid' );

      var store = execHistoryGrid.getStore();

      // set defaults for reload
      store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      store.getProxy().extraParams = {};

      if ( execHistoryGrid.filtersValid() )
      {
        store.getProxy().url =
            QDesk.util.constant.ActionUrl.quartz.execHistoryActionListForScheduler;

        var extraStoreParams = {
          connectionId : me.connectionCtx.getConnectionId()
        };

        var fetchPeriod = execHistoryGrid.down( '#fetchPeriod' ).getValue();
        Ext.apply( extraStoreParams, {
          fetchPeriod : Ext.encode( fetchPeriod )
        } );

        var maxFetchSize = execHistoryGrid.down( '#maxFetchSizeSelect' ).getValue();
        if ( maxFetchSize != -1 )  // -1 == unlimited
        {
          Ext.apply( extraStoreParams, {
            maxFetchSize : maxFetchSize
          } );
        }

        // if exec history index enabled, we add the filter query text param
        if ( me.connectionCtx.getJvmAgentInfo().isQuartzExecHistoryIndexEnabled() )
        {
          var filterQueryText = execHistoryGrid.down( '#filterQueryText' ).getValue();
          if ( !Ext.isEmpty( filterQueryText ) )
          {
            Ext.apply( extraStoreParams, {
              filterQuery : filterQueryText
            } );
          }
        }

        store.getProxy().extraParams = extraStoreParams;
      }

      store.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
