/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Saves the job execution log data of the specified execution history record.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.SaveExecHistoryLog', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the execution history record ID.
   */
  execHistoryId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.execHistoryId )
      Ext.Error.raise( 'Required config parameter missing: execHistoryId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.SaveExecHistoryLog.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionLogSave,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        execHistoryId : me.execHistoryId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.SaveExecHistoryLog.successMsg' ),
                function ()
                {
                  var handle = json.data.handle;
                  me._downloadLogData( handle );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  _downloadLogData : function ( handle )
  {
    var downloadUrl = QDesk.util.constant.ActionUrl.quartz.execHistoryActionLogDownload +
        '&handle=' + handle;

    var iframe = Ext.getBody().createChild( {
      tag : 'iframe',
      width : 0,
      height : 0,
      frameborder : 0,
      src : downloadUrl,
      cls : Ext.baseCSSPrefix + 'hide-display'
    } );

    // give the backend 10 seconds to initiate the download and then silently drop the iframe
    Ext.defer( function ()
        {
          iframe.remove();
        },
        10000 );
  }
} );
