/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config parameter with the job detail of the job to delete.
   */
  jobDetail : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: jobDetail' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.jobDelete' ) )
    {
      var jobGroupName = me.jobDetail.get( 'group' ).name;
      var jobName = me.jobDetail.get( 'name' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.confirmMsg',
              Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteJob();
            }
          }
      );
    }
    else
    {
      me._deleteJob();
    }
  },


  /**
   * Deletes the job.
   */
  _deleteJob : function ()
  {
    var me = this;

    var jobGroupName = me.jobDetail.get( 'group' ).name;
    var jobName = me.jobDetail.get( 'name' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.maskMsg',
        Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobActionDelete,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        jobGroupName : jobGroupName,
        jobName : jobName
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob.successMsg',
                    Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.JOB_DELETED, {
                    connectionId : me.connectionCtx.getConnectionId(),
                    jobGroupName : jobGroupName,
                    jobName : jobName
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
