/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Jobs tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.controller.command.PreviewChart',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.ViewError',

    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',

    'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.CloneTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.PauseTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger',

    'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog',
    'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult',
    'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryUserData',

    'QDesk.controller.command.scheduler.quartz.ViewJobDataMap',

    'QDesk.controller.command.scheduler.quartz.chain.AddJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EditJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail',

    'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory',

    'QDesk.controller.command.scheduler.quartz.jobs.AddJob',
    'QDesk.controller.command.scheduler.quartz.jobs.CloneJob',
    'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob',
    'QDesk.controller.command.scheduler.quartz.jobs.EditJob',
    'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob',
    'QDesk.controller.command.scheduler.quartz.jobs.PauseJobGroup',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshAuditTrailView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecHistoryView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecNotificationsView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobChainsView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshMisfiredTriggersView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshTriggersView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshStatisticsView',
    'QDesk.controller.command.scheduler.quartz.jobs.ResumeJobGroup',
    'QDesk.controller.command.scheduler.quartz.jobs.ViewNextFireTimes',

    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail',
    'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes',

    'QDesk.model.scheduler.quartz.QuartzTriggerGroupState',
    'QDesk.model.scheduler.quartz.QuartzTriggerState',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
        me.connectionCtx.getConnectionId() );

    me._bindEventHandlers( jobsTab );

    jobsTab.setInitialized( true );
  },


  /**
   * Binds events from the jobs tab to their respective handlers.
   */
  _bindEventHandlers : function ( jobsTab )
  {
    var me = this;
    me._bindEventHandlersJobsGrid( jobsTab );

    // individual views
    me._bindEventHandlersHistoryView( jobsTab );
    me._bindEventHandlersMisfiredTriggersView( jobsTab );
    me._bindEventHandlersTriggersView( jobsTab );
    me._bindEventHandlersStatisticsView( jobsTab );
    me._bindEventHandlersNotificationsView( jobsTab );
    me._bindEventHandlersJobChainsView( jobsTab );
    me._bindEventHandlersAuditTrailView( jobsTab );
  },


  /**
   * Binds the jobs grid events to their handlers.
   */
  _bindEventHandlersJobsGrid : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var refreshButton = jobsGrid.down( '#refreshButton' );

    // refresh button
    refreshButton.on( 'click',
        function ()
        {
          me._refreshJobsGrid();
        } );

    var triggerJobButton = jobsGrid.down( '#triggerJobButton' );
    triggerJobButton.on( 'click',
        function ()
        {
          var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];
          me._triggerJob( jobDetail );
        }, me );

    var addJobButton = jobsGrid.down( '#addJobButton' );
    addJobButton.on( 'click',
        function ()
        {
          Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob', {
            connectionCtx : me.connectionCtx,
            /*
             * Upon success refresh the jobs tab.
             */
            success : function ()
            {
              Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
                connectionCtx : me.connectionCtx
              } ).execute();
            }
          } ).execute();
        }, me );

    var deleteJobButton = jobsGrid.down( '#deleteJobButton' );
    deleteJobButton.on( 'click',
        function ()
        {
          var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];
          me._deleteJob( jobDetail );
        }, me );

    var editJobButton = jobsGrid.down( '#editJobButton' );
    editJobButton.on( 'click',
        function ()
        {
          var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];
          me._editJob( jobDetail );
        }, me );

    var cloneJobButton = jobsGrid.down( '#cloneJobButton' );
    cloneJobButton.on( 'click',
        function ()
        {
          var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];
          me._cloneJob( jobDetail );
        }, me );

    var filterByJobGroupCombo = jobsGrid.down( '#filterByJobGroupCombo' );
    filterByJobGroupCombo.on( 'change',
        function ( combo, newValue )
        {
          if ( newValue == null )
          {
            resumeJobGroupButton.disable();
            pauseJobGroupButton.disable();
          }
          else
          {
            resumeJobGroupButton.enable();
            pauseJobGroupButton.enable();
          }

          jobsGrid.getStore().clearFilter( true );

          if ( newValue != null )
          {
            jobsGrid.getStore().addFilter( [
              Ext.create( 'Ext.util.Filter', {
                property : 'group.name',
                value : newValue
              } )
            ], false );
          }

          // apply, or clear the filter and load the first page
          jobsGrid.getStore().loadPage( 1 );
        } );

    filterByJobGroupCombo.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          if ( successful )
          {
            // ok
          }
          else
          {
            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );

    var resumeJobGroupButton = jobsGrid.down( '#resumeJobGroupButton' );
    resumeJobGroupButton.on( 'click',
        function ()
        {
          var jobGroupName = filterByJobGroupCombo.getValue();
          me._resumeJobGroup( jobGroupName );
        }, me );

    var pauseJobGroupButton = jobsGrid.down( '#pauseJobGroupButton' );
    pauseJobGroupButton.on( 'click',
        function ()
        {
          var jobGroupName = filterByJobGroupCombo.getValue();
          me._pauseJobGroup( jobGroupName );
        }, me );

    var filterQueryText = jobsGrid.down( '#filterQueryText' );

    // TODO: expanding/collapsing all grid rows does not work reliably - produces "Uncaught TypeError: Cannot read property 'hasCls' of null"
    /*
     var expandAllButton = jobsGrid.down( '#expandAllButton' );
     expandAllButton.on( 'click',
     function ()
     {
     me._expandAllRows( jobsGrid );
     }, me );

     var collapseAllButton = jobsGrid.down( '#collapseAllButton' );
     collapseAllButton.on( 'click',
     function ()
     {
     me._collapseAllRows( jobsGrid );
     }, me );
     */

    var jobsIndexEnabled = false;
    if ( jvmAgentReady )
    {
      if ( publicApiReady )
      {
        jobsIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzJobsIndexEnabled();

        if ( !jobsIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.jobsIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexPublicApiNotReadyEmptyText' ) );
      }
    }
    else
    {
      filterQueryText.setEmptyText(
          QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexAgentNotReadyEmptyText' ) );
    }

    // event fired by the actioncolumn in the JobsGrid
    jobsGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap )
        {
          me._viewJobDataMap( jobDataMap );
        } );

    if ( jvmAgentReady )
    {
      // event fired by the actioncolumn in the JobsGrid
      jobsGrid.down( '#nextFireTimeColumn' ).on( 'requestViewNextFireTimes',
          function ( jobGroupName, jobName )
          {
            me._viewNextFireTimesOfJob( jobGroupName, jobName );
          } );
    }

    jobsGrid.getStore().on( 'beforeload',
        function ( store, operation )
        {
          refreshButton.disable();
          addJobButton.disable();

          filterByJobGroupCombo.disable();
          resumeJobGroupButton.disable();
          pauseJobGroupButton.disable();

          filterQueryText.disable();

          // TODO: expanding/collapsing all grid rows does not work reliably - produces "Uncaught TypeError: Cannot read property 'hasCls' of null"
          //expandAllButton.disable();
          //collapseAllButton.disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          jobsGrid.getSelectionModel().deselectAll();
        } );

    jobsGrid.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          refreshButton.enable();

          // enable only if the jobs index is enabled in the agent
          if ( jobsIndexEnabled )
          {
            filterQueryText.enable();
          }

          if ( successful )
          {
            addJobButton.enable();

            filterByJobGroupCombo.enable();

            if ( filterByJobGroupCombo.getValue() )
            {
              resumeJobGroupButton.enable();
              pauseJobGroupButton.enable();
            }

            // TODO: expanding/collapsing all grid rows does not work reliably - produces "Uncaught TypeError: Cannot read property 'hasCls' of null"
            //expandAllButton.enable();
            //collapseAllButton.enable();
          }
          else
          {
            store.removeAll();

            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );

    jobsGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length == 0 )
          {
            me._onJobUnselected( jobsTab );
          }
          else
          {
            me._onJobSelected( selectedRecords[ 0 ], jobsTab );
          }
        } );

    /* JobsGrid context menu actions */
    jobsGrid.on( 'triggerJob', me._triggerJob, me );
    jobsGrid.on( 'deleteJob', me._deleteJob, me );
    jobsGrid.on( 'editJob', me._editJob, me );
    jobsGrid.on( 'cloneJob', me._cloneJob, me );
  },


  /**
   * Binds the job execution history panel events to their handlers.
   */
  _bindEventHandlersHistoryView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var historyView = jobsTab.down( '#historyView' );
    historyView.on( 'activate', function ()
    {
      me._refreshHistoryView( true );
      me._refreshTriggersView( false );
      me._refreshMisfiredTriggersView( false );
      me._refreshStatisticsView( false );
      me._refreshExecNotificationsView( false );
      me._refreshJobChainsView( false );
      me._refreshAuditTrailView( false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory && jvmAgentReady )
    {
      var execHistoryGrid = jobsTab.down( '#execHistoryGrid' );
      // assert execHistoryGrid != null - the job execution history grid should exist

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshHistoryView( true );
          } );

      // export CSV button
      execHistoryGrid.down( '#exportCsvButton' ).on( 'click',
          function ()
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];
            me._exportExecHistory( jobDetail );
          } );

      // filters changed
      execHistoryGrid.on( 'changeFilters',
          function ()
          {
            me._refreshHistoryView( true );
          } );

      // event fired by the actioncolumn in the JobExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( execHistoryId );
          } );

      var filterQueryText = execHistoryGrid.down( '#filterQueryText' );

      var execHistoryIndexEnabled = false;
      if ( jvmAgentReady )
      {
        execHistoryIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzExecHistoryIndexEnabled();

        if ( !execHistoryIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.execHistoryIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexAgentNotReadyEmptyText' ) );
      }

      execHistoryGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            execHistoryGrid.down( '#refreshButton' ).disable();
            execHistoryGrid.down( '#fetchPeriod' ).disable();
            execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();
            filterQueryText.disable();
            execHistoryGrid.down( '#exportCsvButton' ).disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            execHistoryGrid.getSelectionModel().deselectAll();
          } );

      execHistoryGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

            if ( jobSelected )
            {
              execHistoryGrid.down( '#refreshButton' ).enable();
              execHistoryGrid.down( '#fetchPeriod' ).enable();
              execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();

              // enable only if the exec history index is enabled in the agent
              if ( execHistoryIndexEnabled )
              {
                filterQueryText.enable();
              }

              if ( jvmAgentReady )
              {
                execHistoryGrid.down( '#exportCsvButton' ).enable();
              }

              if ( successful )
              {
                // currently no logic
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );
    }
  },


  /**
   * Binds the job triggers grid events to their handlers.
   */
  _bindEventHandlersTriggersView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    // when the triggers view is activated, we need to refresh it
    var triggersView = jobsTab.down( '#triggersView' );
    triggersView.on( 'activate',
        function ()
        {
          me._refreshTriggersView( true );
          me._refreshHistoryView( false );
          me._refreshMisfiredTriggersView( false );
          me._refreshStatisticsView( false );
          me._refreshExecNotificationsView( false );
          me._refreshJobChainsView( false );
          me._refreshAuditTrailView( false );
        } );

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );

    var refreshButton = triggersGrid.down( '#refreshButton' );
    refreshButton.on( 'click',
        function ()
        {
          me._refreshTriggersView( true );
        }, me );

    var resumeTriggerButton = triggersGrid.down( '#resumeTriggerButton' );
    resumeTriggerButton.on( 'click',
        function ()
        {
          var me = this;
          var trigger = triggersGrid.getSelectionModel().getSelection()[ 0 ];
          me._resumeTrigger( trigger );
        }, me );

    var pauseTriggerButton = triggersGrid.down( '#pauseTriggerButton' );
    pauseTriggerButton.on( 'click',
        function ()
        {
          var me = this;
          var trigger = triggersGrid.getSelectionModel().getSelection()[ 0 ];
          me._pauseTrigger( trigger );
        }, me );

    var addTriggerButton = triggersGrid.down( '#addTriggerButton' );
    Ext.each( Ext.ComponentQuery.query( 'menuitem', addTriggerButton ),
        function ( menuitem )
        {
          menuitem.on( 'click',
              function ()
              {
                var jobsGrid = jobsTab.down( '#jobsGrid' );
                var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];

                me._addTrigger( jobDetail, menuitem.qdesk.triggerType );
              }, me );
        } );

    var deleteTriggerButton = triggersGrid.down( '#deleteTriggerButton' );
    deleteTriggerButton.on( 'click',
        function ()
        {
          var me = this;
          var trigger = triggersGrid.getSelectionModel().getSelection()[ 0 ];
          me._deleteTrigger( trigger );
        }, me );

    var editTriggerButton = triggersGrid.down( '#editTriggerButton' );
    editTriggerButton.on( 'click',
        function ()
        {
          var me = this;
          var trigger = triggersGrid.getSelectionModel().getSelection()[ 0 ];
          me._editTrigger( trigger );
        }, me );

    var cloneTriggerButton = triggersGrid.down( '#cloneTriggerButton' );
    cloneTriggerButton.on( 'click',
        function ()
        {
          var me = this;
          var trigger = triggersGrid.getSelectionModel().getSelection()[ 0 ];
          me._cloneTrigger( trigger );
        }, me );

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );

    filterByTriggerGroupCombo.on( 'change',
        function ( combo, newValue )
        {
          if ( newValue == null )
          {
            resumeTriggerGroupButton.disable();
            pauseTriggerGroupButton.disable();
          }
          else
          {
            // based on the selected trigger group's state enable either the resume, or pause button
            var triggerGroup = filterByTriggerGroupCombo.findRecord( 'name', newValue );
            var triggerGroupState = triggerGroup.get( 'state' );

            switch ( triggerGroupState )
            {
              case QDesk.model.scheduler.quartz.QuartzTriggerGroupState.NORMAL:
                resumeTriggerGroupButton.disable();
                pauseTriggerGroupButton.enable();
                break;

              case QDesk.model.scheduler.quartz.QuartzTriggerGroupState.PAUSED:
                resumeTriggerGroupButton.enable();
                pauseTriggerGroupButton.disable();
                break;

              default:
                Ext.Error.raise( 'Unexpected trigger group state: ' + triggerGroupState );
            }
          }

          triggersGrid.getStore().clearFilter( true );

          if ( newValue != null )
          {
            triggersGrid.getStore().addFilter( [
              Ext.create( 'Ext.util.Filter', {
                property : 'group.name',
                value : newValue
              } )
            ], false );
          }

          // apply, or clear the filter and load the first page
          triggersGrid.getStore().loadPage( 1 );
        } );

    filterByTriggerGroupCombo.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          if ( successful )
          {
            // ok
          }
          else
          {
            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );

    var resumeTriggerGroupButton = triggersGrid.down( '#resumeTriggerGroupButton' );
    resumeTriggerGroupButton.on( 'click',
        function ()
        {
          var me = this;
          var triggerGroupName = filterByTriggerGroupCombo.getValue();
          me._resumeTriggerGroup( triggerGroupName );
        }, me );

    var pauseTriggerGroupButton = triggersGrid.down( '#pauseTriggerGroupButton' );
    pauseTriggerGroupButton.on( 'click',
        function ()
        {
          var me = this;
          var triggerGroupName = filterByTriggerGroupCombo.getValue();
          me._pauseTriggerGroup( triggerGroupName );
        }, me );

    var filterQueryText = triggersGrid.down( '#filterQueryText' );

    var triggersIndexEnabled = false;
    if ( jvmAgentReady )
    {
      if ( publicApiReady )
      {
        triggersIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzTriggersIndexEnabled();

        if ( !triggersIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.triggersIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexPublicApiNotReadyEmptyText' ) );
      }
    }
    else
    {
      filterQueryText.setEmptyText(
          QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexAgentNotReadyEmptyText' ) );
    }

    // event fired by the actioncolumn in the JobTriggersGrid
    triggersGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap, mergedJobDataMap )
        {
          me._viewJobDataMap( jobDataMap, mergedJobDataMap );
        } );

    // event fired by the actioncolumn in the TriggersGrid
    triggersGrid.down( '#nextFireTimeColumn' ).on( 'requestViewNextFireTimes',
        function ( triggerGroupName, triggerName )
        {
          me._viewNextFireTimesOfTrigger( triggerGroupName, triggerName );
        } );

    triggersGrid.getStore().on( 'beforeload',
        function ( store, operation )
        {
          refreshButton.disable();
          triggersGrid.down( '#addTriggerButton' ).disable();
          filterByTriggerGroupCombo.disable();
          filterQueryText.disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          triggersGrid.getSelectionModel().deselectAll();
        } );

    triggersGrid.getStore().on( 'load',
        function ( triggerStore, records, successful, operation )
        {
          var jobsGrid = jobsTab.down( '#jobsGrid' );
          var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

          if ( jobSelected )
          {
            refreshButton.enable();

            if ( successful )
            {
              // if the JVM Agent and the public API are ready
              if ( jvmAgentReady && publicApiReady )
              {
                triggersGrid.down( '#addTriggerButton' ).enable();
              }

              if ( records.length > 0 )
                filterByTriggerGroupCombo.enable();
            }

            // enable only if the triggers index is enabled in the agent
            if ( triggersIndexEnabled )
            {
              filterQueryText.enable();
            }
          }

          if ( successful )
          {
            // currently no logic
          }
          else
          {
            triggerStore.removeAll();

            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );

    triggersGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length == 0 )
          {
            me._onTriggerUnselected( jobsTab );
          }
          else
          {
            me._onTriggerSelected( selectedRecords[ 0 ], jobsTab );
          }
        } );

    /* JobTriggersGrid context menu actions */
    triggersGrid.on( 'resumeTrigger', me._resumeTrigger, me );
    triggersGrid.on( 'pauseTrigger', me._pauseTrigger, me );
    triggersGrid.on( 'deleteTrigger', me._deleteTrigger, me );
    triggersGrid.on( 'editTrigger', me._editTrigger, me );
    triggersGrid.on( 'cloneTrigger', me._cloneTrigger, me );
  },


  /**
   * Binds the job misfired triggers panel events to their handlers.
   */
  _bindEventHandlersMisfiredTriggersView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var misfiredTriggersView = jobsTab.down( '#misfiredTriggersView' );
    misfiredTriggersView.on( 'activate',
        function ()
        {
          me._refreshMisfiredTriggersView( true );
          me._refreshHistoryView( false );
          me._refreshTriggersView( false );
          me._refreshStatisticsView( false );
          me._refreshExecNotificationsView( false );
          me._refreshJobChainsView( false );
          me._refreshAuditTrailView( false );
        } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.MisfiredTriggers && jvmAgentReady )
    {
      var misfiredTriggersGrid = jobsTab.down( '#misfiredTriggersGrid' );

      // refresh button
      misfiredTriggersGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshMisfiredTriggersView( true );
          } );

      // filters changed
      misfiredTriggersGrid.on( 'changeFilters',
          function ()
          {
            me._refreshMisfiredTriggersView( true );
          } );

      var filterQueryText = misfiredTriggersGrid.down( '#filterQueryText' );

      var misfiredTriggersIndexEnabled = false;
      if ( jvmAgentReady )
      {
        misfiredTriggersIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzMisfiredTriggersIndexEnabled();

        if ( !misfiredTriggersIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.misfiredTriggersIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexAgentNotReadyEmptyText' ) );
      }

      misfiredTriggersGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            misfiredTriggersGrid.down( '#refreshButton' ).disable();
            misfiredTriggersGrid.down( '#fetchPeriod' ).disable();
            misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            misfiredTriggersGrid.getSelectionModel().deselectAll();
          } );

      misfiredTriggersGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

            if ( jobSelected )
            {
              misfiredTriggersGrid.down( '#refreshButton' ).enable();
              misfiredTriggersGrid.down( '#fetchPeriod' ).enable();
              misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).enable();

              // enable only if the index is enabled in the agent
              if ( misfiredTriggersIndexEnabled )
              {
                filterQueryText.enable();
              }

              if ( successful )
              {
                // currently no logic
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );
    }
  },


  /**
   * Binds the job statistics panel events to their handlers.
   */
  _bindEventHandlersStatisticsView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var statisticsView = jobsTab.down( '#statisticsView' );
    statisticsView.on( 'activate',
        function ()
        {
          me._refreshStatisticsView( true );
          me._refreshHistoryView( false );
          me._refreshTriggersView( false );
          me._refreshMisfiredTriggersView( false );
          me._refreshExecNotificationsView( false );
          me._refreshJobChainsView( false );
          me._refreshAuditTrailView( false );
        } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics && jvmAgentReady )
    {
      var statisticsPanel = statisticsView.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView( true );
          } );

      // preview chart button
      statisticsPanel.down( '#previewChartButton' ).on( 'click',
          function ()
          {
            me._previewStatisticsChart();
          } );

      // save chart button
      //statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( 'image/jpeg' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( 'image/png' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( 'image/svg+xml' );
      //    } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView( true );
          } );
    }
  },


  /**
   * Binds the job notifications panel events to their handlers.
   */
  _bindEventHandlersNotificationsView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var notificationsView = jobsTab.down( '#notificationsView' );
    notificationsView.on( 'activate',
        function ()
        {
          me._refreshExecNotificationsView( true );
          me._refreshHistoryView( false );
          me._refreshTriggersView( false );
          me._refreshMisfiredTriggersView( false );
          me._refreshStatisticsView( false );
          me._refreshJobChainsView( false );
          me._refreshAuditTrailView( false );
        } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications && jvmAgentReady )
    {
      var execNotificationRulesGrid = jobsTab.down( '#execNotificationRulesGrid' );

      // refresh button
      execNotificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshExecNotificationsView( true );
          } );

      execNotificationRulesGrid.on( 'selectionchange',
          function ( selectionModel, selectedRecords )
          {
            if ( selectedRecords.length == 0 )
            {
              me._onNotificationRuleUnselected( jobsTab );
            }
            else
            {
              me._onNotificationRuleSelected( selectedRecords[ 0 ], jobsTab );
            }
          } );

      var filterQueryText = execNotificationRulesGrid.down( '#filterQueryText' );

      var notificationRulesIndexEnabled = false;
      if ( jvmAgentReady )
      {
        notificationRulesIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzExecNotificationRulesIndexEnabled();

        if ( !notificationRulesIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.notificationRulesIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexAgentNotReadyEmptyText' ) );
      }

      execNotificationRulesGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            execNotificationRulesGrid.down( '#refreshButton' ).disable();
            execNotificationRulesGrid.down( '#addButton' ).disable();
            execNotificationRulesGrid.down( '#deleteButton' ).disable();
            execNotificationRulesGrid.down( '#editButton' ).disable();
            execNotificationRulesGrid.down( '#cloneButton' ).disable();
            execNotificationRulesGrid.down( '#viewAuditTrailButton' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            execNotificationRulesGrid.getSelectionModel().deselectAll();
          } );

      execNotificationRulesGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

            if ( jobSelected )
            {
              execNotificationRulesGrid.down( '#refreshButton' ).enable();

              if ( successful )
              {
                execNotificationRulesGrid.down( '#addButton' ).enable();
              }

              // enable only if the notification rules index is enabled in the agent
              if ( notificationRulesIndexEnabled )
              {
                filterQueryText.enable();
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );

      execNotificationRulesGrid.on( 'addRule',
          function ()
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];

            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
              connectionCtx : me.connectionCtx,
              jobGroupName : jobDetail.get( 'group' ).name,
              jobName : jobDetail.get( 'name' ),
              /*
               * Upon success refresh the notifications view.
               */
              success : function ()
              {
                me._refreshExecNotificationsView( true );
              }
            } ).execute();
          } );

      execNotificationRulesGrid.on( 'enableRule',
          function ( execNotifRule )
          {
            me._enableExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'disableRule',
          function ( execNotifRule )
          {
            me._disableExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'deleteRule',
          function ( execNotifRule )
          {
            me._deleteNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'editRule',
          function ( execNotifRule )
          {
            me._editExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'cloneRule',
          function ( execNotifRule )
          {
            me._cloneExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'viewAuditTrail',
          function ( execNotifRule )
          {
            me._viewExecNotificationRuleAuditTrail( execNotifRule );
          } );

      // opens a window with messages enqueued by the notification rule
      execNotificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages',
          function ( execNotificationRuleId )
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
              connectionCtx : me.connectionCtx,
              creatorId : execNotificationRuleId,
              creatorType : QDesk.model.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
            } ).execute();
          } );
    }
  },


  /**
   * Binds the job job chain panel events to their handlers.
   */
  _bindEventHandlersJobChainsView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var jobChainsView = jobsTab.down( '#jobChainsView' );
    jobChainsView.on( 'activate',
        function ()
        {
          me._refreshJobChainsView( true );
          me._refreshHistoryView( false );
          me._refreshTriggersView( false );
          me._refreshMisfiredTriggersView( false );
          me._refreshStatisticsView( false );
          me._refreshExecNotificationsView( false );
          me._refreshAuditTrailView( false );
        } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining && jvmAgentReady )
    {
      var jobChainsGrid = jobsTab.down( '#jobChainsGrid' );

      // refresh button
      jobChainsGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshJobChainsView( true );
          } );

      jobChainsGrid.on( 'selectionchange',
          function ( selectionModel, selectedRecords )
          {
            if ( selectedRecords.length == 0 )
            {
              me._onJobChainUnselected( jobsTab );
            }
            else
            {
              me._onJobChainSelected( selectedRecords[ 0 ], jobsTab );
            }
          } );

      var filterQueryText = jobChainsGrid.down( '#filterQueryText' );

      var jobChainsIndexEnabled = false;
      if ( jvmAgentReady )
      {
        jobChainsIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzJobChainsIndexEnabled();

        if ( !jobChainsIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.jobChainsIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.indexAgentNotReadyEmptyText' ) );
      }

      jobChainsGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            jobChainsGrid.down( '#refreshButton' ).disable();
            jobChainsGrid.down( '#addButton' ).disable();
            jobChainsGrid.down( '#deleteButton' ).disable();
            jobChainsGrid.down( '#editButton' ).disable();
            jobChainsGrid.down( '#cloneButton' ).disable();
            jobChainsGrid.down( '#viewAuditTrailButton' ).disable();
            filterQueryText.disable();

            // TODO: expanding/collapsing all grid rows does not work reliably - produces "Uncaught TypeError: Cannot read property 'hasCls' of null"
            //jobChainsGrid.down( '#expandAllButton' ).disable();
            //jobChainsGrid.down( '#collapseAllButton' ).disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            jobChainsGrid.getSelectionModel().deselectAll();
          } );

      jobChainsGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

            if ( jobSelected )
            {
              jobChainsGrid.down( '#refreshButton' ).enable();

              if ( successful )
              {
                jobChainsGrid.down( '#addButton' ).enable();
              }

              // enable only if the notification rules index is enabled in the agent
              if ( jobChainsIndexEnabled )
              {
                filterQueryText.enable();
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );

      jobChainsGrid.on( 'addJobChain',
          function ()
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobDetail = jobsGrid.getSelectionModel().getSelection()[ 0 ];

            me._addJobChain( jobDetail );
          } );

      jobChainsGrid.on( 'enableJobChain',
          function ( jobChain )
          {
            me._enableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'disableJobChain',
          function ( jobChain )
          {
            me._disableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'deleteJobChain',
          function ( jobChain )
          {
            me._deleteJobChain( jobChain );
          } );

      jobChainsGrid.on( 'editJobChain',
          function ( jobChain )
          {
            me._editJobChain( jobChain );
          } );

      jobChainsGrid.on( 'cloneJobChain',
          function ( jobChain )
          {
            me._cloneJobChain( jobChain );
          } );

      jobChainsGrid.on( 'viewAuditTrail',
          function ( jobChain )
          {
            me._viewJobChainAuditTrail( jobChain );
          } );
    }
  },


  /**
   * Binds the job audit trail panel events to their handlers.
   */
  _bindEventHandlersAuditTrailView : function ( jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var viewPanel = jobsTab.down( '#auditTrailView' );
    viewPanel.on( 'activate',
        function ()
        {
          me._refreshAuditTrailView( true );
          me._refreshJobChainsView( false );
          me._refreshHistoryView( false );
          me._refreshTriggersView( false );
          me._refreshMisfiredTriggersView( false );
          me._refreshStatisticsView( false );
          me._refreshExecNotificationsView( false );
        } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;
    if ( licensedFeatures.AuditTrail )
    {
      var auditTrailGrid = viewPanel.down( '#auditTrailGrid' );

      // refresh button
      auditTrailGrid.on( 'refresh',
          function ()
          {
            me._refreshAuditTrailView( true );
          } );

      // filters changed
      auditTrailGrid.on( 'changeFilters',
          function ()
          {
            me._refreshAuditTrailView( true );
          } );

      var auditTrailIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzAuditTrailIndexEnabled();

      auditTrailGrid.on( 'loadFailed',
          function ( operation )
          {
            auditTrailGrid.down( '#refreshButton' ).enable();
          } );

      auditTrailGrid.on( 'loadSuccess',
          function ( store, records )
          {
            var jobsGrid = jobsTab.down( '#jobsGrid' );
            var jobSelected = jobsGrid.getSelectionModel().getSelection().length > 0;

            if ( jobSelected )
            {
              auditTrailGrid.down( '#refreshButton' ).enable();
              auditTrailGrid.down( '#fetchPeriod' ).enable();
              auditTrailGrid.down( '#maxFetchSizeSelect' ).enable();
              auditTrailGrid.down( '#fetchRelatedCheckbox' ).enable();

              // enable only if the index is enabled
              if ( auditTrailIndexEnabled )
              {
                auditTrailGrid.down( '#filterQueryText' ).enable();
              }
            }

            // update empty text if there are no records
            var view = auditTrailGrid.getView();
            if ( Ext.isEmpty( records ) )
            {
              if ( jobSelected )
              {
                view.emptyText = QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.auditTrailEmptyTextNoMatchingDataForJob' );
              }
              else
              {
                view.emptyText = QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.auditTrailEmptyTextNoJobSelected' );
              }

              view.refresh();
            }
          } );
    }
  },


  /**
   * Refreshes the jobs grid.
   * @param connectionCtx a connection context.
   * @private
   */
  _refreshJobsGrid : function ( connectionCtx )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid', {
      connectionCtx : me.connectionCtx
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job execution history view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshHistoryView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecHistoryView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Opens a window for exporting job execution history data to a CSV file.
   * @param jobDetail a job detail whose execution history data should be exported.
   * @private
   */
  _exportExecHistory : function ( jobDetail )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory', {
      connectionCtx : me.connectionCtx,
      jobDetail : jobDetail
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job triggers view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshTriggersView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshTriggersView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job misfired triggers view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshMisfiredTriggersView : function ( populate )
  {
    var me = this;

    var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
        me.connectionCtx.getConnectionId() );

    var misfiredTriggersView = jobsTab.down( '#misfiredTriggersView' );

    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshMisfiredTriggersView', {
      connectionCtx : me.connectionCtx,
      populate : populate,
      maskEl : misfiredTriggersView.getEl()
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job statistics view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshStatisticsView : function ( populate )
  {
    var me = this;

    var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
        me.connectionCtx.getConnectionId() );

    var statisticsView = jobsTab.down( '#statisticsView' );

    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshStatisticsView', {
      connectionCtx : me.connectionCtx,
      populate : populate,
      maskEl : statisticsView.getEl()
    } ).execute();
  },


  /**
   * Invoked to view the chart in a new modal window which allows the user to save the chart
   * through the browser context menu.
   */
  _previewStatisticsChart : function ()
  {
    var me = this;

    var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
        me.connectionCtx.getConnectionId() );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = jobsTab.down( '#statisticsView #statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.PreviewChart', {
      chart : chart
    } ).execute();
  },


  /**
   * Invoked to save the chart in the jobs statistics view.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( mimeType )
  {
    var me = this;

    var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
        me.connectionCtx.getConnectionId() );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = jobsTab.down( '#statisticsView #statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job notifications view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshExecNotificationsView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecNotificationsView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job job chain view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshJobChainsView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobChainsView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job audit trail view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshAuditTrailView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshAuditTrailView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked when a job in the JobsGrid in the JobsTab is selected.
   * @param jobDetail a QDesk.model.scheduler.quartz.QuartzJobDetail instance representing the selected job.
   * @param jobsTab the jobs tab.
   */
  _onJobSelected : function ( jobDetail, jobsTab )
  {
    var me = this;

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var triggerJobButton = jobsGrid.down( '#triggerJobButton' );
    //var addJobButton = jobsGrid.down( '#addJobButton' );
    var deleteJobButton = jobsGrid.down( '#deleteJobButton' );
    var editJobButton = jobsGrid.down( '#editJobButton' );
    var cloneJobButton = jobsGrid.down( '#cloneJobButton' );

    triggerJobButton.enable();
    //addJobButton.enable();
    deleteJobButton.enable();
    editJobButton.enable();
    cloneJobButton.enable();

    var views = jobsTab.down( '#views' );
    // assert views != null - the views should exist

    var activeView = views.getActiveTab();

    me._refreshHistoryView( activeView.getItemId() == 'historyView' );
    me._refreshTriggersView( activeView.getItemId() == 'triggersView' );
    me._refreshMisfiredTriggersView( activeView.getItemId() == 'misfiredTriggersView' );
    me._refreshStatisticsView( activeView.getItemId() == 'statisticsView' );
    me._refreshExecNotificationsView( activeView.getItemId() == 'notificationsView' );
    me._refreshJobChainsView( activeView.getItemId() == 'jobChainsView' );
    me._refreshAuditTrailView( activeView.getItemId() == 'auditTrailView' );
  },


  /**
   * Invoked when a job in the JobsGrid in the JobsTab is unselected.
   * @param jobsTab a JobsTab.
   */
  _onJobUnselected : function ( jobsTab )
  {
    var me = this;

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var triggerJobButton = jobsGrid.down( '#triggerJobButton' );
    var addJobButton = jobsGrid.down( '#addJobButton' );
    var deleteJobButton = jobsGrid.down( '#deleteJobButton' );
    var editJobButton = jobsGrid.down( '#editJobButton' );
    var cloneJobButton = jobsGrid.down( '#cloneJobButton' );

    triggerJobButton.disable();
    addJobButton.disable();
    editJobButton.disable();
    deleteJobButton.disable();
    cloneJobButton.disable();

    me._refreshHistoryView( false );
    me._refreshTriggersView( false );
    me._refreshMisfiredTriggersView( false );
    me._refreshStatisticsView( false );
    me._refreshExecNotificationsView( false );
    me._refreshJobChainsView( false );
    me._refreshAuditTrailView( false );
  },


  /**
   * Invoked when a trigger in the JobTriggersGrid in the JobsTab is selected.
   * @param trigger a QDesk.model.scheduler.quartz.QuartzTrigger instance representing the selected trigger.
   * @param jobsTab a JobsTab.
   */
  _onTriggerSelected : function ( trigger, jobsTab )
  {
    var me = this;

    var jvmAgentReady = me.connectionCtx.getJvmAgentInfo().isReady();
    var publicApiReady = me.connectionCtx.getSchedulerInfo().isPublicApiReady();

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );
    var triggerState = trigger.get( 'state' );

    switch ( triggerState )
    {
      case QDesk.model.scheduler.quartz.QuartzTriggerState.NONE:
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.NORMAL:
        triggersGrid.down( '#resumeTriggerButton' ).disable();
        triggersGrid.down( '#pauseTriggerButton' ).enable();
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.PAUSED:
        triggersGrid.down( '#resumeTriggerButton' ).enable();
        triggersGrid.down( '#pauseTriggerButton' ).disable();
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.COMPLETE:
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.ERROR:
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.BLOCKED:
        break;

      default:
        Ext.Error.raise( 'Unexpected trigger state value: ' + triggerState );
    }

    triggersGrid.down( '#deleteTriggerButton' ).enable();

    // enable only if the JVM Agent and public API are ready
    if ( jvmAgentReady && publicApiReady )
    {
      triggersGrid.down( '#editTriggerButton' ).enable();
      triggersGrid.down( '#cloneTriggerButton' ).enable();
    }
  },


  /**
   * Invoked when a trigger in the JobTriggersGrid in the JobsTab is unselected.
   * @param jobsTab a JobsTab.
   */
  _onTriggerUnselected : function ( jobsTab )
  {
    var me = this;

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );

    triggersGrid.down( '#resumeTriggerButton' ).disable();
    triggersGrid.down( '#pauseTriggerButton' ).disable();
    triggersGrid.down( '#deleteTriggerButton' ).disable();
    triggersGrid.down( '#editTriggerButton' ).disable();
    triggersGrid.down( '#cloneTriggerButton' ).disable();
  },


  /**
   * Invoked when a notification rule record in the JobNotificationRulesGrid in the JobNotificationsView is selected.
   * @param execNotifRule a QDesk.model.scheduler.quartz.QuartzExecNotificationRule instance representing the selected
   * notification rule.
   * @param jobsTab the JobsTab.
   */
  _onNotificationRuleSelected : function ( execNotifRule, jobsTab )
  {
    var me = this;

    var execNotificationRulesGrid = jobsTab.down( '#execNotificationRulesGrid' );

    execNotificationRulesGrid.down( '#deleteButton' ).enable();
    execNotificationRulesGrid.down( '#editButton' ).enable();
    execNotificationRulesGrid.down( '#cloneButton' ).enable();
    execNotificationRulesGrid.down( '#viewAuditTrailButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the JobNotificationRulesGrid in the JobNotificationsView is unselected.
   * @param jobsTab the JobsTab.
   */
  _onNotificationRuleUnselected : function ( jobsTab )
  {
    var me = this;

    var execNotificationRulesGrid = jobsTab.down( '#execNotificationRulesGrid' );

    execNotificationRulesGrid.down( '#deleteButton' ).disable();
    execNotificationRulesGrid.down( '#editButton' ).disable();
    execNotificationRulesGrid.down( '#cloneButton' ).disable();
    execNotificationRulesGrid.down( '#viewAuditTrailButton' ).disable();
  },


  /**
   * Invoked when a job chain record in the JobChainsGrid in the JobJobChainsCtxView is selected.
   * @param jobChain a QDesk.model.scheduler.quartz.QuartzJobChain instance representing the selected
   * job chain.
   * @param jobsTab the JobsTab.
   */
  _onJobChainSelected : function ( jobChain, jobsTab )
  {
    var me = this;

    var jobChainsGrid = jobsTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteButton' ).enable();
    jobChainsGrid.down( '#editButton' ).enable();
    jobChainsGrid.down( '#cloneButton' ).enable();
    jobChainsGrid.down( '#viewAuditTrailButton' ).enable();
  },


  /**
   * Invoked when a job chain record in the JobChainsGrid in the JobJobChainsCtxView is unselected.
   * @param jobsTab the JobsTab.
   */
  _onJobChainUnselected : function ( jobsTab )
  {
    var me = this;

    var jobChainsGrid = jobsTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteButton' ).disable();
    jobChainsGrid.down( '#editButton' ).disable();
    jobChainsGrid.down( '#cloneButton' ).disable();
    jobChainsGrid.down( '#viewAuditTrailButton' ).disable();
  },


  _triggerJob : function ( jobDetail )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob', {
      connectionCtx : me.connectionCtx,
      jobDetail : jobDetail
    } ).execute();
  },


  _deleteJob : function ( jobDetail )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.DeleteJob', {
      connectionCtx : me.connectionCtx,
      jobDetail : jobDetail,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _editJob : function ( jobDetail )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob', {
      connectionCtx : me.connectionCtx,
      jobDetail : jobDetail,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _cloneJob : function ( jobDetail )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.CloneJob', {
      connectionCtx : me.connectionCtx,
      jobDetail : jobDetail,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _resumeJobGroup : function ( jobGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.ResumeJobGroup', {
      connectionCtx : me.connectionCtx,
      jobGroupName : jobGroupName,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _pauseJobGroup : function ( jobGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.PauseJobGroup', {
      connectionCtx : me.connectionCtx,
      jobGroupName : jobGroupName,
      /*
       * Upon success refresh the jobs tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionCtx : me.connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _pauseTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTrigger', {
      connectionCtx : me.connectionCtx,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _resumeTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger', {
      connectionCtx : me.connectionCtx,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _addTrigger : function ( jobDetail, triggerType )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger', {
      connectionCtx : me.connectionCtx,
      triggerType : triggerType,
      jobGroupName : jobDetail.get( 'group' ).name,
      jobName : jobDetail.get( 'name' ),
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _deleteTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger', {
      connectionCtx : me.connectionCtx,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _editTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger', {
      connectionCtx : me.connectionCtx,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _cloneTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.CloneTrigger', {
      connectionCtx : me.connectionCtx,
      trigger : trigger,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _enableExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
      connectionCtx : me.connectionCtx,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( true );
      }
    } ).execute();
  },


  _disableExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule', {
      connectionCtx : me.connectionCtx,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( true );
      }
    } ).execute();
  },


  _deleteNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
      connectionCtx : me.connectionCtx,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( true );
      }
    } ).execute();
  },


  _editExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
      connectionCtx : me.connectionCtx,
      notificationRuleId : execNotifRule.get( 'id' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( true );
      }
    } ).execute();
  },


  _cloneExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
      connectionCtx : me.connectionCtx,
      notificationRuleId : execNotifRule.get( 'id' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( true );
      }
    } ).execute();
  },


  _viewExecNotificationRuleAuditTrail : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail', {
      connectionCtx : me.connectionCtx,
      notificationRuleId : execNotifRule.get( 'id' )
    } ).execute();
  },


  _addJobChain : function ( jobDetail )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.AddJobChain', {
      connectionCtx : me.connectionCtx,
      jobGroupName : jobDetail.get( 'group' ).name,
      jobName : jobDetail.get( 'name' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshJobChainsView( true );
      }
    } ).execute();
  },


  _enableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain', {
      connectionCtx : me.connectionCtx,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( true );
      }
    } ).execute();
  },


  _disableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain', {
      connectionCtx : me.connectionCtx,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( true );
      }
    } ).execute();
  },


  _deleteJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
      connectionCtx : me.connectionCtx,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( true );
      }
    } ).execute();
  },


  _editJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
      connectionCtx : me.connectionCtx,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( true );
      }
    } ).execute();
  },


  _cloneJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain', {
      connectionCtx : me.connectionCtx,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( true );
      }
    } ).execute();
  },


  _viewJobChainAuditTrail : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail', {
      connectionCtx : me.connectionCtx,
      jobChainId : jobChain.get( 'id' )
    } ).execute();
  },


  _resumeTriggerGroup : function ( triggerGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTriggerGroup', {
      connectionCtx : me.connectionCtx,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  _pauseTriggerGroup : function ( triggerGroupName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup', {
      connectionCtx : me.connectionCtx,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the job triggers view.
       */
      success : function ()
      {
        me._refreshTriggersView( true );
      }
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap the job data map.
   * @param mergedJobDataMap the merged job data map.
   */
  _viewJobDataMap : function ( jobDataMap, mergedJobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      // exec history record can have null job data map !!!
      jobDataMap : jobDataMap == null ? [] : jobDataMap,
      mergedJobDataMap : mergedJobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the next fire times for the specified job.
   * @param jobGroupName a job group name.
   * @param jobName a job name.
   */
  _viewNextFireTimesOfJob : function ( jobGroupName, jobName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.ViewNextFireTimes', {
      connectionCtx : me.connectionCtx,
      jobGroupName : jobGroupName,
      jobName : jobName
    } ).execute();
  },


  /**
   * Opens a window showing the next fire times for the specified trigger.
   * @param triggerGroupName a trigger group name.
   * @param triggerName a trigger name.
   */
  _viewNextFireTimesOfTrigger : function ( triggerGroupName, triggerName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes', {
      connectionCtx : me.connectionCtx,
      triggerGroupName : triggerGroupName,
      triggerName : triggerName
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param exception the exception data.
   */
  _viewException : function ( exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog', {
      connectionCtx : me.connectionCtx,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult', {
      connectionCtx : me.connectionCtx,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryUserData', {
      connectionCtx : me.connectionCtx,
      execHistoryId : execHistoryId
    } ).execute();
  },


  _expandAllRows : function ( grid )
  {
    var rowExpanderPlugin = grid.getPlugin( 'rowExpander' );
    if ( rowExpanderPlugin )
    {
      rowExpanderPlugin.expandAll();
    }
  },


  _collapseAllRows : function ( grid )
  {
    var rowExpanderPlugin = grid.getPlugin( 'rowExpander' );
    if ( rowExpanderPlugin )
    {
      rowExpanderPlugin.collapseAll();
    }
  }
} );
