/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Pauses the specified job group.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.PauseJobGroup', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the job group name.
   */
  jobGroupName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.jobGroupName )
      Ext.Error.raise( 'Required config parameter missing: jobGroupName' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.jobGroupPause' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.PauseJobGroup.confirmMsg',
              Ext.String.htmlEncode( me.jobGroupName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._pauseJobGroup();
            }
          }
      );
    }
    else
    {
      me._pauseJobGroup();
    }
  },


  _pauseJobGroup : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.PauseJobGroup.maskMsg',
        Ext.String.htmlEncode( me.jobGroupName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobGroupActionPause,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        jobGroupName : me.jobGroupName
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.PauseJobGroup.successMsg',
                    Ext.String.htmlEncode( me.jobGroupName ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.JOB_GROUP_PAUSED, {
                    connectionId : me.connectionCtx.getConnectionId(),
                    jobGroupName : me.jobGroupName
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
