/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Execution History view in the scheduler view panel's Jobs tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecHistoryView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory && me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
          me.connectionCtx.getConnectionId() );

      var jobsGrid = jobsTab.down( '#jobsGrid' );

      var selectedJobDetails = jobsGrid.getSelectionModel().getSelection();

      var execHistoryGrid = jobsTab.down( '#execHistoryGrid' );

      var execHistoryGridStore = execHistoryGrid.getStore();

      // set defaults for reload
      execHistoryGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      execHistoryGridStore.getProxy().extraParams = {};

      if ( me.populate )
      {
        // if a job is selected and filters are valid
        if ( !Ext.isEmpty( selectedJobDetails ) && execHistoryGrid.filtersValid() )
        {
          var jobDetail = selectedJobDetails[ 0 ];

          execHistoryGridStore.getProxy().url =
              QDesk.util.constant.ActionUrl.quartz.execHistoryActionListForJob;

          var extraStoreParams = {
            connectionId : me.connectionCtx.getConnectionId(),
            jobGroupName : jobDetail.get( 'group' ).name,
            jobName : jobDetail.get( 'name' )
          };

          var fetchPeriod = execHistoryGrid.down( '#fetchPeriod' ).getValue();
          Ext.apply( extraStoreParams, {
            fetchPeriod : Ext.encode( fetchPeriod )
          } );

          var maxFetchSize = execHistoryGrid.down( '#maxFetchSizeSelect' ).getValue();
          if ( maxFetchSize != -1 )  // -1 == unlimited
          {
            Ext.apply( extraStoreParams, {
              maxFetchSize : maxFetchSize
            } );
          }

          // if exec history index enabled, we add the filter query text param
          if ( me.connectionCtx.getJvmAgentInfo().isQuartzExecHistoryIndexEnabled() )
          {
            var filterQueryText = execHistoryGrid.down( '#filterQueryText' ).getValue();
            if ( !Ext.isEmpty( filterQueryText ) )
            {
              Ext.apply( extraStoreParams, {
                filterQuery : filterQueryText
              } );
            }
          }

          execHistoryGridStore.getProxy().extraParams = extraStoreParams;
        }
      }

      execHistoryGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
