/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window with a grid populated with the next fire times of the specified job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.ViewNextFireTimes', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.scheduler.quartz.ViewNextFireTimesWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /*
   * Job group name.
   */
  jobGroupName : undefined,

  /*
   * Job name.
   */
  jobName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.jobGroupName )
      Ext.Error.raise( 'Required config parameter missing: jobGroupName' );

    if ( !config.jobName )
      Ext.Error.raise( 'Required config parameter missing: jobName' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var fullJobName = me.jobGroupName + ' / ' + me.jobName;

    var window = Ext.widget( 'QuartzViewNextFireTimesWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        jobName : fullJobName
      }
    } );

    me._bindEventHandlers( window );

    var store = window.down( '#quartzViewNextFireTimesGrid' ).getStore();

    store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.jobActionListNextFireTimes;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.ViewNextFireTimes.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          window.show();

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      //window.maximize();
      window.toFront( false );
    } );

    var grid = window.down( '#quartzViewNextFireTimesGrid' );

    var store = grid.getStore();

    // in before load we add all required parameters
    store.on( 'beforeload', function ( store, operation )
    {
      grid.down( '#refreshButton' ).setDisabled( true );
      grid.down( '#maxResultsSelect' ).setDisabled( true );

      var params = operation.getParams() || {};

      var maxResults = grid.down( '#maxResultsSelect' ).getValue();

      Ext.apply( params, {
        connectionId : me.connectionCtx.getConnectionId(),
        maxResults : maxResults ? maxResults : null
      } );

      Ext.apply( params, {
        jobGroupName : me.jobGroupName,
        jobName : me.jobName
      } );

      operation.setParams( params );
    } );

    store.on( 'load',
        function ( store, records, successful, operation )
        {
          grid.down( '#refreshButton' ).setDisabled( false );
          grid.down( '#maxResultsSelect' ).setDisabled( false );
        } );

    grid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshGrid( window );
        } );

    grid.down( '#maxResultsSelect' ).on( 'change',
        function ( combo, newValue, oldValue )
        {
          me._refreshGrid( window );
        } );
  },


  /**
   * Refreshes the contents of the message grid.
   * @param window the messages window.
   * @private
   */
  _refreshGrid : function ( window )
  {
    var me = this;

    var grid = window.down( '#quartzViewNextFireTimesGrid' );

    var store = grid.getStore();
    //store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.jobActionListNextFireTimes;

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        if ( !success )
        {
          store.removeAll( false );

          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  }
} );
