/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Enables the specified notification rule.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config parameter with the notification rule to enable.
   */
  notificationRule : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.notificationRule )
      Ext.Error.raise( 'Required config parameter missing: notificationRule' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.execNotificationRuleEnable' ) )
    {
      var name = me.notificationRule.get( 'name' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule.confirmMsg',
              Ext.String.htmlEncode( name ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._enableNotificationRule();
            }
          }
      );
    }
    else
    {
      me._enableNotificationRule();
    }
  },


  /**
   * Enables the notification rule.
   */
  _enableNotificationRule : function ()
  {
    var me = this;

    var notificationRuleId = me.notificationRule.get( 'id' );
    var name = me.notificationRule.get( 'name' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule.maskMsg',
        Ext.String.htmlEncode( name ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionEnable,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        notificationRuleId : notificationRuleId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule.successMsg',
                    Ext.String.htmlEncode( name ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.EXEC_NOTIF_RULE_ENABLED, {
                    connectionId : me.connectionCtx.getConnectionId(),
                    notificationRule : me.notificationRule
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
