/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Job-Specific Execution Notification Rules grid under the top-level Execution Notification Rules tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshJobExecNotificationRulesGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var execNotificationRulesTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getExecNotificationRulesTab(
          me.connectionCtx.getConnectionId() );

      var notificationRulesGrid = execNotificationRulesTab.down(
          '#jobExecNotificationRulesView #execNotificationRulesGrid' );

      var store = notificationRulesGrid.getStore();

      store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionListForJobs;

      var extraStoreParams = {
        connectionId : me.connectionCtx.getConnectionId()
      };

      // if jobs index enabled, we add the filter query text param
      if ( me.connectionCtx.getJvmAgentInfo().isQuartzExecNotificationRulesIndexEnabled() )
      {
        var filterQueryText = notificationRulesGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      store.getProxy().extraParams = extraStoreParams;

      store.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
