/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Triggers tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecHistoryView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecNotificationsView'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    // refreshes the triggers grid
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid', {
      connectionCtx : me.connectionCtx,
      success : function ()
      {
        me._refreshNotificationsView( true );
        me._refreshHistoryView( true );
        me._refreshStatisticsView( true );
      },
      failure : function ()
      {
        me._refreshNotificationsView( false );
        me._refreshHistoryView( false );
        me._refreshStatisticsView( false );
      }
    } ).execute();


  },


  /**
   * Invoked to refresh the data in the trigger execution history view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshHistoryView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecHistoryView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger statistics view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshStatisticsView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger notifications view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshNotificationsView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecNotificationsView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  }
} );
