/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Logs out the currently logged in user.
 */
Ext.define( 'QDesk.controller.command.security.Logout', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.common.Result',
    'QDesk.model.security.LogoutError',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.Logout.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.securityLogoutAction,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var result = json.result;

            // update the authentication status and profile of the current user
            QDesk.CurrentUser.update(
                result.sessionToken,
                {
                  authenticated : false,
                  userProfile : null
                },
                false );

            if ( result.result === QDesk.model.common.Result.SUCCESS )
            {
              me.application.fireEvent( QDesk.util.constant.EventName.LOGOUT_SUCCESS, {
                sessionToken : result.sessionToken,
                automatic : false
              } );

              // invoke the success callback handler
              if ( me.success && Ext.isFunction( me.success ) )
              {
                me.success.apply( me );
              }
            }
            else
            {
              if ( result.error )
              {
                switch ( result.error )
                {
                  case QDesk.model.security.LogoutError.SESSION_NOT_FOUND:
                    break;
                }
              }

              me.application.fireEvent( QDesk.util.constant.EventName.LOGOUT_FAILED, {
                sessionToken : result.sessionToken,
                error : result.error
              } );

              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
