/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the grid in the specified ServiceEndpointsTab panel.
 */
Ext.define( 'QDesk.controller.command.system.ws.RefreshServiceEndpointsGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.util.constant.ActionUrl'
  ],


  /**
   * Required config parameter with the ServiceEndpointsTab panel.
   */
  panel : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.panel )
      Ext.Error.raise( 'Required config parameter missing: panel' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var serviceEndpointsGrid = me.panel.down( 'ServiceEndpointsGrid' );
    var serviceEndpointsGridStore = serviceEndpointsGrid.getStore();

    serviceEndpointsGridStore.getProxy().url =
        QDesk.util.constant.ActionUrl.systemServiceEndpointRegistryActionListEndpoints;

    var extraStoreParams = {};

    Ext.apply( extraStoreParams, {
      serviceId : me.panel.getService().get( 'id' )
    } );

    serviceEndpointsGridStore.getProxy().extraParams = extraStoreParams;

    serviceEndpointsGridStore.loadPage( 1, {
      scope : me,
      callback : function ( records, operation, success )
      {
        if ( success )
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          // invoke the failure callback handler
          if ( me.failure && Ext.isFunction( me.failure ) )
          {
            me.failure.apply( me );
          }
        }
      }
    } );
  }
} );
