/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Resets the statistics of the specified Web Service Endpoint.
 */
Ext.define( 'QDesk.controller.command.system.ws.ResetServiceEndpointStatistics', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],


  /**
   * Required config property containing the service.
   */
  service : undefined,

  /**
   * Required config property containing the service endpoint.
   */
  serviceEndpoint : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.service )
      Ext.Error.raise( 'Required config parameter missing: service' );

    if ( !config.serviceEndpoint )
      Ext.Error.raise( 'Required config parameter missing: serviceEndpoint' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.serviceEndpointResetStatistics' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.system.ws.ResetServiceEndpointStatistics.confirmMsg',
              me.service.get( 'name' ),
              QDesk.util.FormatUtils.formatVersion( me.serviceEndpoint.get( 'version' ) ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._resetEndpointStatistics();
            }
          }
      );
    }
    else
    {
      me._resetEndpointStatistics();
    }
  },


  _resetEndpointStatistics : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.system.ws.ResetServiceEndpointStatistics.maskLoadMsg',
        me.service.get( 'name' ),
        QDesk.util.FormatUtils.formatVersion( me.serviceEndpoint.get( 'version' ) )
    ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.systemServiceEndpointRegistryActionResetEndpointStatistics,
      params : {
        endpointId : me.serviceEndpoint.get( 'id' )
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.system.ws.ResetServiceEndpointStatistics.successMsg',
                    me.service.get( 'name' ),
                    QDesk.util.FormatUtils.formatVersion( me.serviceEndpoint.get( 'version' ) ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
