/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the Web Service Endpoints tab (ServicesTab).
 */
Ext.define( 'QDesk.controller.command.system.ws.ViewServices', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.system.ws.InitializeServiceEndpointsTab',
    'QDesk.controller.command.system.ws.RefreshServiceEndpointsTab',

    'QDesk.model.system.ws.Service',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.system.ws.ServicesTab',
    'QDesk.view.work.system.ws.ServiceEndpointsTab'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
    var servicesTab = mainTabPanel.down( 'ServicesTab' );

    // if the ServicesTab is open, activate it
    if ( servicesTab )
    {
      mainTabPanel.setActiveTab( servicesTab );
    }
    else
    {
      me.mask( QDesk.Msg.get( 'QDesk.controller.command.system.ws.ViewServices.maskMsg' ) );

      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.systemServiceEndpointRegistryActionListServices,
        callback : function ( opts, success, resp )
        {
          me.unmask();

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              var rawServices = json.data;
              if ( Ext.isEmpty( rawServices ) )
              {
                QDesk.util.UxUtils.showWarningDialog(
                    QDesk.Msg.get( 'QDesk.controller.command.system.ws.ViewServices.noServicesDefinedMsg' ) );
              }
              else
              {
                servicesTab = Ext.widget( 'ServicesTab', {} );

                var firstServiceEndpointsTab = null;
                Ext.each( rawServices,
                    function ( rawService )
                    {
                      var service = Ext.create( 'QDesk.model.system.ws.Service', rawService );

                      var serviceEndpointsTab = Ext.widget( 'ServiceEndpointsTab', {
                        qdesk : {
                          service : service
                        }
                      } );

                      if ( firstServiceEndpointsTab == null )
                        firstServiceEndpointsTab = serviceEndpointsTab;

                      serviceEndpointsTab.on( 'activate', function ()
                      {
                        /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
                        if ( !serviceEndpointsTab.getInitialized() )
                        {
                          Ext.create( 'QDesk.controller.command.system.ws.InitializeServiceEndpointsTab', {
                            panel : serviceEndpointsTab
                          } ).execute();

                          Ext.create( 'QDesk.controller.command.system.ws.RefreshServiceEndpointsTab', {
                            panel : serviceEndpointsTab
                          } ).execute();
                        }
                      } );

                      servicesTab.addTab( serviceEndpointsTab );
                    }
                );

                // activate the first service endpoints tab
                servicesTab.on( 'afterrender',
                    function ()
                    {
                      servicesTab.setActiveTab( firstServiceEndpointsTab );
                    } );

                mainTabPanel.add( servicesTab );
                mainTabPanel.setActiveTab( servicesTab );
              }
            }
            else
            {
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
          else
          {
            QDesk.util.UxUtils.showExtJsAjaxError( resp );
          }
        }
      } );
    }
  }
} );
