/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Saves the current user's profile.
 */
Ext.define( 'QDesk.controller.command.user.SaveProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.User',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required config parameter with the EditProfileForm panel.
   */
  formPanel : undefined,

  /**
   * Required config parameter with the User instance containing the current user's profile data.
   */
  user : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.formPanel )
      Ext.Error.raise( 'Required config parameter missing: formPanel' );

    if ( !config.user )
      Ext.Error.raise( 'Required config parameter missing: user' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me._saveProfile( me.user );
  },


  _saveProfile : function ( user )
  {
    var me = this;

    var form = me.formPanel.getForm();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.userActionUpdateProfile,
      params : {
        user : QDesk.util.CommonUtils.encode( user )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.user.SaveProfile.updateWaitMsg' ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          me.application.fireEvent( QDesk.util.constant.EventName.USER_UPDATED, {
            user : me.user
          } );

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.user.SaveProfile.updateSuccessMsg' ),
              function ()
              {
                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  }
} );
