/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.security.SecurityController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.util.constant.EventName'
  ],


  init : function ()
  {
    var me = this;

    var application = me.getApplication();

    application.addListener( QDesk.util.constant.EventName.LOGIN_SUCCESS, me._onLoginSuccess, me );
    application.addListener( QDesk.util.constant.EventName.LOGIN_FAILED, me._onLoginFailed, me );

    application.addListener( QDesk.util.constant.EventName.LOGOUT_SUCCESS, me._onLogoutSuccess, me );
    application.addListener( QDesk.util.constant.EventName.LOGOUT_FAILED, me._onLogoutFailed, me );

    application.addListener( QDesk.util.constant.EventName.HEARTBEAT, me._onHeartbeat, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );
  },


  _onLoginSuccess : function ( params )
  {
    var me = this;

    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];
    viewport.activateWorkPanel();

    var userProfile = params.userProfile;

    QDesk.util.UxUtils.showInfoBubble(
        QDesk.Msg.get( 'QDesk.controller.security.SecurityController.loginSuccessMsg',
            Ext.String.htmlEncode( userProfile.username ) ) );
  },


  _onLoginFailed : function ( params )
  {
    //var me = this;
  },


  _onLogoutSuccess : function ( params )
  {
    var me = this;

    /*
     * Activate the security login panel.
     */
    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];

    // unlock the viewport if it is locked
    viewport.unlock();

    viewport.activateSecurityLoginPanel();

    var automatic = params.automatic;
    if ( automatic )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get( 'QDesk.controller.security.SecurityController.userSessionExpired' ) );
    }
    else
    {
      // manual logout
      QDesk.util.UxUtils.showInfoBubble(
          QDesk.Msg.get( 'QDesk.controller.security.SecurityController.logoutSuccessMsg' ) );
    }
  },


  _onLogoutFailed : function ( params )
  {
    var me = this;

    /*
     * Activate the security login panel.
     */
    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];

    // unlock the viewport if it is locked
    viewport.unlock();

    viewport.activateSecurityLoginPanel();

    switch ( params.error )
    {
        /*
         * At the moment, the only recognized logout error is QDesk.model.security.LogoutError.SESSION_NOT_FOUND.
         * We treat this error as a logout success.
         */
      case QDesk.model.security.LogoutError.SESSION_NOT_FOUND:
        QDesk.util.UxUtils.showInfoBubble(
            QDesk.Msg.get( 'QDesk.controller.security.SecurityController.logoutSuccessMsg' ) );
        break;
    }
  },


  _onHeartbeat : function ( params )
  {
    var me = this;
    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];

    // update license warnings (yellow box) in the security panel
    var securityPanel = viewport.down( '#securityPanel' );
    securityPanel.updateLicenseWarnings( params.license );
  }
} );
