/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * QDesk configuration singleton that provides access to the application configuration data.
 */
Ext.define( 'QDesk.util.constant.DateFormat', {

  singleton : true,

  /**
   * Format used to parse and render date values.
   * 'Y-m-d' -> '2012-01-25'
   */
  dateFormat : 'Y-m-d',

  /**
   * Format used to parse rawValue in FormattedTimestamp instances.
   *
   * ISO 8601 format w time zone offset -> '2012-01-25T14:48:08.000+0000'
   */
  timestampFormatIso8601WithTz : 'Y-m-d\\TH:i:s.uO',

  /**
   * Format used to parse targetValue Date values in FormattedTimestamp instances and to encode (format) Date objects
   * in all outgoing JSON messages.
   *
   * ISO 8601 format w/ time zone -> '2012-01-25T14:48:08.000'
   *
   * Encoded dates have no timestamp information because JavaScript Dates always use local time zone anyway.
   */
  timestampFormatIso8601WithoutTz : 'Y-m-d\\TH:i:s.u',

  /**
   * Format used to parse and render time values.
   * 'H:i' -> '15:48'
   */
  timeFormatShort : 'H:i',

  /**
   * Format used to parse and render time values.
   * 'H:i:s' -> '15:48:08'
   */
  timeFormatMedium : 'H:i:s',

  /**
   * Format used to parse and render time values.
   * 'H:i:s.u' -> '15:48:08.000'
   */
  timeFormatLong : 'H:i:s.u',

  /**
   * Format used to parse and render com.quartzdesk.domain.type.DayTime values.
   * 'H:i:s' -> '15:48:08'
   */
  dayTimeFormat : 'H:i:s'
} );
