/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various Quartz executing job-specific utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzExecutingJobUtils', {
  requires : [
    'QDesk.Msg',

    'QDesk.util.license.LicensedProductFeatures'
  ],

  singleton : true,


  /**
   * Creates and returns HTML code with executing job details displayed by the rowexpander in the QuartzExecutingJobsGrid.
   * @param values an executing job value object (not a QuartzExecutingJob model instance!).
   * @param ownerGrid the owner grid.
   * @returns {string}
   */
  createRowDetailsHtml : function ( values, ownerGrid )
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var html = '<table class="rowDetailsTable">' +
        '<thead>' +
        '<tr><th colspan="2" style="width: 100px;">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzExecutingJobUtils.rowDetailsTable.columnJob' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    /* Start of Job Description */
    var jobDescription = Ext.isEmpty( values.jobDetail.description ) ? QDesk.Msg.get( 'QDesk.notAvailable' ) :
        values.jobDetail.description;

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.description' ) + '</td>';

    html += '<td>' + jobDescription + '</td>';
    html += '</tr>';
    /* End of Job Description */

    /* Start of Job Monitoring URL */
    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.monitoringUrl' ) + '</td>';

    html += '<td>';

    var jobMonitoringLinkUrl = values.jobDetail.monitoringUrl;
    var jobMonitoringLinkTooltip;
    if ( licensedFeatures.PollMonitoring )
    {
      jobMonitoringLinkTooltip =
          Ext.String.htmlEncode(
              QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobUtils.rowDetailsTable.tooltipMonitoringUrl' ) );

      html += '<a href="' + jobMonitoringLinkUrl +
          '" class="newWindow" target="_blank" data-qtip="' + jobMonitoringLinkTooltip + '">' + jobMonitoringLinkUrl +
          '</a>';
    }
    else
    {
      jobMonitoringLinkTooltip = Ext.String.htmlEncode(
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobUtils.rowDetailsTable.tooltipMonitoringUrlNotLicensed' ) );

      html += '<a href="' + jobMonitoringLinkUrl +
          '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
          jobMonitoringLinkTooltip + '">' + jobMonitoringLinkUrl + '</a>';
    }

    html += '</td></tr>';
    /* End of Job Monitoring URL */

    html += '</tbody>' +
        '</table>';

    html += '<p/>';

    html += '<table class="rowDetailsTable">' +
        '<thead>' +
        '<tr><th colspan="2" style="width: 100px;">' +
        QDesk.Msg.get( 'QDesk.util.quartz.QuartzExecutingJobUtils.rowDetailsTable.columnTrigger' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    /* Start of Trigger Description */
    var triggerDescription = Ext.isEmpty( values.trigger.description ) ? QDesk.Msg.get( 'QDesk.notAvailable' ) :
        values.trigger.description;

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.description' ) + '</td>';

    html += '<td>' + triggerDescription + '</td>';
    html += '</tr>';
    /* End of Trigger Description */

    /* Start of Trigger Monitoring URL */
    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.monitoringUrl' ) + '</td>';

    html += '<td>';

    var triggerMonitoringLinkUrl = values.trigger.monitoringUrl;
    var triggerMonitoringLinkTooltip;
    if ( licensedFeatures.PollMonitoring )
    {
      triggerMonitoringLinkTooltip =
          Ext.String.htmlEncode(
              QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.tooltipMonitoringUrl' ) );

      html += '<a href="' + triggerMonitoringLinkUrl +
          '" class="newWindow" target="_blank" data-qtip="' + triggerMonitoringLinkTooltip + '">' +
          triggerMonitoringLinkUrl +
          '</a>';
    }
    else
    {
      triggerMonitoringLinkTooltip = Ext.String.htmlEncode(
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.tooltipMonitoringUrlNotLicensed' ) );

      html += '<a href="' + triggerMonitoringLinkUrl +
          '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
          triggerMonitoringLinkTooltip + '">' + triggerMonitoringLinkUrl + '</a>';
    }

    html += '</td></tr>';
    /* End of Job Monitoring URL */

    html += '</tbody>' +
        '</table>';

    return html;
  }
} );
