/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.about.AboutForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',

    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.AboutForm',

  cls : 'aboutForm',

  border : false,

  bodyPadding : '20px 30px 5px 30px',

  fieldDefaults : {
    style : {
      minHeight : '15px', // override the default displayfield min-height of 22px
      marginTop : 0
    }
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'fieldcontainer',
          layout : 'anchor',
          items : [
            {
              xtype : 'fieldcontainer',
              layout : {
                type : 'hbox'
              },
              style : {
                marginBottom : '2px'
              },
              items : [
                {
                  xtype : 'displayfield',
                  itemId : 'productNameAndVersion',
                  name : 'productNameAndVersion',
                  cls : 'productNameAndVersion'
                },
                {
                  xtype : 'component',
                  flex : 1
                },
                {
                  xtype : 'component',
                  itemId : 'copyToClipboard',
                  cls : 'copyToClipboard',
                  html : QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldCopyToClipboard' )
                }
              ]
            },
            {
              xtype : 'displayfield',
              itemId : 'buildDate',
              name : 'buildDateFormatted',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldBuildDate', value );
              }
            }
          ]
        },

        {
          xtype : 'fieldcontainer',
          items : [
            {
              xtype : 'displayfield',
              itemId : 'licenseeName',
              name : 'licenseeName',
              cls : 'licenseeName',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldLicensedTo', value );
              }
            },
            {
              xtype : 'displayfield',
              itemId : 'licenseType',
              name : 'licenseType',
              cls : 'licenseType',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldLicenseType',
                    QDesk.util.EnumUtils.getLicenseTypeName( value ) );
              }
            },
            {
              xtype : 'displayfield',
              itemId : 'licenseSerialNumber',
              name : 'licenseSerialNumber',
              cls : 'licenseSerialNumber',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldLicenseSerialNumber', value );
              }
            },

            {
              xtype : 'displayfield',
              itemId : 'licenseExpiryText',
              cls : 'licenseExpiryText',
              hidden : true
            },

            {
              xtype : 'displayfield',
              itemId : 'swMaintenanceExpiryText',
              cls : 'swMaintenanceExpiryText',
              hidden : true
            }

            // {
            //   xtype : 'displayfield',
            //   itemId : 'licenseExpiresInDays',
            //   fieldStyle : {
            //     color : '#ff3f05 !important',
            //     fontWeight : 'bold !important'
            //   },
            //   hidden : true
            // },
            // {
            //   xtype : 'displayfield',
            //   itemId : 'licenseExpiredText',
            //   value : QDesk.Msg.get( 'QDesk.view.about.AboutForm.textLicenseExpired' ),
            //   fieldStyle : {
            //     color : '#ff3f05 !important',
            //     fontWeight : 'bold !important'
            //   },
            //   hidden : true
            // }
          ]
        },

        {
          xtype : 'fieldcontainer',
          items : [
            {
              xtype : 'displayfield',
              itemId : 'jreInfo',
              name : 'jreInfo',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldJreInfo', value );
              }
            },
            {
              xtype : 'displayfield',
              itemId : 'jvmInfo',
              name : 'jvmInfo',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldJvmInfo', value );
              }
            }
          ]
        },

        {
          xtype : 'fieldcontainer',
          items : [
            {
              xtype : 'displayfield',
              itemId : 'appServerInfo',
              name : 'appServerInfo',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldAppServerInfo', value );
              }
            },
            {
              xtype : 'displayfield',
              itemId : 'databaseInfo',
              name : 'databaseInfo',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldDatabaseInfo', value );
              }
            },
            {
              xtype : 'displayfield',
              itemId : 'osInfo',
              name : 'osInfo',
              renderer : function ( value )
              {
                return QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldOsInfo', value );
              }
            }
          ]
        },

        {
          xtype : 'displayfield',
          itemId : 'poweredByOss',
          value : QDesk.Msg.get( 'QDesk.view.about.AboutForm.fieldPoweredByOss' ),
          cls : 'poweredByOss'
        },

        {
          xtype : 'textarea',
          itemId : 'clipboard',
          cls : 'clipboard'
        },

        {
          xtype : 'displayfield',
          itemId : 'copyright',
          name : 'copyright',
          cls : 'copyright'
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;
    me.callParent();

    var copyToClipboardField = me.down( '#copyToClipboard' );

    me.on( 'afterrender', function ()
    {
      var meEl = me.getEl();
      var copyToClipboardEl = copyToClipboardField.getEl();

      // Copy to Clipboard link clicked - copy license and deployment data to the clipboard
      copyToClipboardEl.on( 'click', function ()
      {
        me._copyToClipboard( me );
      } );

      meEl.on( 'mouseenter', function ()
      {
        copyToClipboardEl.setOpacity( 0 );
        copyToClipboardEl.fadeIn( {
          opacity : 1,
          duration : 500
        } );

        // install CTRL key down handler
      } );

      meEl.on( 'mouseleave', function ()
      {
        copyToClipboardEl.setOpacity( 1 );
        copyToClipboardEl.fadeOut( {
          opacity : 0,
          duration : 500
        } );
      } );
    } );


    // var components = Ext.ComponentQuery.query( 'displayfield', me );
    //
    // Ext.each( components, function ( comp )
    // {
    //   comp.on( 'afterrender', function ()
    //   {
    //     comp.getEl().on( 'click', function ( event, elem )
    //     {
    //       if ( elem.tagName.toLowerCase() == 'a' )
    //       {
    //         // stop the event propagation so that the About window does not get closed by the click handler
    //         // that closes the window when the user clicks anywhere on the screen
    //         event.stopPropagation();
    //       }
    //     } );
    //   } );
    // } );
  },


  /**
   * Copies license and deployment data shown in the About dialog to the clipboard.
   * @param form the About dialog form.
   * @private
   */
  _copyToClipboard : function ( form )
  {
    var me = this;

    var clipboardField = form.down( '#clipboard' );

    var data = form.down( '#productNameAndVersion' ).getDisplayValue() + '\n' +
        form.down( '#buildDate' ).getDisplayValue() + '\n' +
        '\n' +
        form.down( '#licenseeName' ).getDisplayValue() + '\n' +
        form.down( '#licenseType' ).getDisplayValue() + '\n' +
        form.down( '#licenseSerialNumber' ).getDisplayValue() + '\n';

    if ( form.down( '#licenseExpiryText' ).isVisible() )
    {
      // remove HTML tags
      var text = QDesk.util.FormatUtils.stripHtmlTags( form.down( '#licenseExpiryText' ).getDisplayValue() );
      // remove the purchase text that is behind |
      var i = text.indexOf( '|' );
      if ( i > -1 )
      {
        text = text.substring( 0, i );
      }
      data += text.trim() + '\n';
    }
    if ( form.down( '#swMaintenanceExpiryText' ).isVisible() )
    {
      // remove HTML tags
      var text = QDesk.util.FormatUtils.stripHtmlTags( form.down( '#swMaintenanceExpiryText' ).getDisplayValue() );
      // remove the renewal text that is behind |
      var i = text.indexOf( '|' );
      if ( i > -1 )
      {
        text = text.substring( 0, i );
      }
      data += text.trim() + '\n';
    }
    data += '\n';

    data += form.down( '#jreInfo' ).getDisplayValue() + '\n';
    data += form.down( '#jvmInfo' ).getDisplayValue() + '\n';
    data += '\n';

    data += form.down( '#appServerInfo' ).getDisplayValue() + '\n';
    data += form.down( '#databaseInfo' ).getDisplayValue() + '\n';
    data += form.down( '#osInfo' ).getDisplayValue() + '\n';

    clipboardField.setValue( data );

    clipboardField.inputEl.dom.focus();
    clipboardField.inputEl.dom.select();

    try
    {
      var successful = document.execCommand( 'copy' );
      if ( successful )
      {
        me.up( 'window' ).close();

        QDesk.util.UxUtils.showInfo(
            QDesk.Msg.get( 'QDesk.view.about.AboutForm.copyToClipboardSuccessMsg' ) );
      }
      else
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get( 'QDesk.view.about.AboutForm.copyToClipboardErrorMsg' ) );
      }
    }
    catch ( err )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get( 'QDesk.view.about.AboutForm.copyToClipboardErrorWithDetailMsg', err ) );
    }
  }
} );
