/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Panel with a form that allows a user to change their password.
 */
Ext.define( 'QDesk.view.security.ChangePasswordPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.security.ChangePassword',

    'QDesk.util.UxUtils',

    'QDesk.view.security.ChangePasswordForm'
  ],

  alias : 'widget.ChangePasswordPanel',

  border : false,

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'center'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'ChangePasswordForm',
          flex : 1
        }

        // {
        //   xtype : 'panel',
        //   border : false,
        //   itemId : 'navigationLinks',
        //   layout : {
        //     type : 'hbox',
        //     align : 'stretch',
        //     pack : 'center'
        //   },
        //   items : [
        //     {
        //       xtype : 'displayfield',
        //       cls : 'link',
        //       itemId : 'loginLink',
        //       value : QDesk.Msg.get( 'QDesk.view.security.ChangePasswordPanel.loginLink' ),
        //       margin : '0 10 0 10'
        //     }
        //   ]
        // }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Binds event handlers.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.on( 'activate', function ()
    {
      var currentPasswordField = me.down( '#currentPassword' );
      QDesk.util.UxUtils.focus( currentPasswordField, true );

      return true;
    } );


    /*
     * Add click events to all links in the navigationLinks panel. These are not HTML links and as such they
     * do not fire click events when clicked.
     */
    Ext.each( Ext.ComponentQuery.query( '#navigationLinks > displayfield[cls=link]', me ),
        function ( link )
        {
          link.on( 'afterrender', function ()
          {
            link.getEl().on( 'click', function ()
            {
              link.fireEvent( 'click' );
            } );
          } );
        } );

    me.down( '#changePasswordButton' ).on( 'click', function ()
    {
      var currentPassword = me.down( '#currentPassword' ).getValue();
      var newPassword = me.down( '#newPassword' ).getValue();
      var newPassword2 = me.down( '#newPassword2' ).getValue();

      Ext.create( 'QDesk.controller.command.security.ChangePassword', {
        currentPassword : currentPassword,
        newPassword : newPassword,
        newPassword2 : newPassword2
      } ).execute();
    } );


    // me.down( '#loginLink' ).on( 'click', function ()
    // {
    //   var securityPanel = me.up( '#securityPanel' );
    //   securityPanel.activateLoginPanel();
    // } );
  }
} );
