/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.WMainButton', {
  extend : 'Ext.Button',

  requires : [
    'QDesk.CurrentUser',
    'QDesk.Msg',

    'QDesk.controller.command.About',
    'QDesk.controller.command.ViewLicenseInfo',
    'QDesk.controller.command.security.Logout',
    'QDesk.controller.command.user.EditProfile',

    'QDesk.model.security.UserAvatarType',
    'QDesk.util.constant.EventName'
  ],

  alias : 'widget.WMainButton',

  DEFAULT_AVATAR : 'images/icon/avatar-default-35x35.png',

  scale : 'large',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var avatarUrl = me.DEFAULT_AVATAR;
    if ( QDesk.CurrentUser.isAuthenticated() )
    {
      var userProfile = QDesk.CurrentUser.getUserProfile();
      if ( userProfile.avatarType != QDesk.model.security.UserAvatarType.NONE )
      {
        avatarUrl = userProfile.avatarUrl;
      }
    }

    Ext.apply( config, {
      icon : avatarUrl,

      menu : [
        // 'Some HTML <b>text</b> <i>that</i> can be long and can span multiple lines.',
        {
          text : QDesk.Msg.get( 'QDesk.view.widget.WMainButton.menuItemProfile' ),
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.user.EditProfile', {} ).execute();
          }
        },
        '-',
        {
          text : QDesk.Msg.get( 'QDesk.view.widget.WMainButton.menuItemLicense' ),
          iconCls : 'license',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.ViewLicenseInfo', {} ).execute();
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.widget.WMainButton.menuItemAbout' ),
          iconCls : 'about',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.About', {} ).execute();
          }
        },
        '-',
        {
          text : QDesk.Msg.get( 'QDesk.view.widget.WMainButton.menuItemLogout' ),
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.security.Logout', {} ).execute();
          }
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent( arguments );

    var application = QDesk.getApplication();

    application.addListener( QDesk.util.constant.EventName.LOGIN_SUCCESS, me._onLoginSuccess, me );
    application.addListener( QDesk.util.constant.EventName.LOGOUT_SUCCESS, me._onLogoutSuccess, me );
    application.addListener( QDesk.util.constant.EventName.USER_UPDATED, me._onUserUpdated, me );
  },


  _onLoginSuccess : function ( params )
  {
    var me = this;

    var userProfile = params.userProfile;

    if ( userProfile.avatarType !== QDesk.model.security.UserAvatarType.NONE )
    {
      me.setIcon( userProfile.avatarUrl );
    }
    else
    {
      // user has no avatar, restore the default avatar
      me.setIcon( me.DEFAULT_AVATAR );
    }
  },


  _onLogoutSuccess : function ( params )
  {
    var me = this;
    me.setIcon( me.DEFAULT_AVATAR );
  },


  _onUserUpdated : function ( params )
  {
    var me = this;
    var user = params.user;

    /*
     * If the current user was updated, update the button's avatar.
     */
    if ( QDesk.CurrentUser.authenticated )
    {
      if ( QDesk.CurrentUser.userProfile.userId === user.get( 'id' ) )
      {
        if ( user.get( 'avatarType' ) !== QDesk.model.security.UserAvatarType.NONE )
        {
          me.setIcon( user.get( 'avatarUrl' ) );
        }
        else
        {
          // user has no avatar, restore the default avatar
          me.setIcon( me.DEFAULT_AVATAR );
        }
      }
    }
  }
} );