/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * Tree selection combo box that is used to select the message channel profile.
 *
 * Base on the code published at http://www.sencha.com/forum/showthread.php?38654-Tree-in-a-Combo./page6
 */
Ext.define( 'QDesk.view.widget.WMessageChannelProfileCombo', {
  extend : 'Ext.ux.form.TreeIconCombo',

  requires : [
    'QDesk.Msg',

    'QDesk.store.message.MessageChannelProfilesTreeStore',

    'QDesk.util.constant.Const'
  ],

  alias : 'widget.WMessageChannelProfileCombo',

  idValuePrefix : 'profile-',
  valueField : 'profileId',
  displayField : 'name',
  iconClsField : 'messageChannel',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      emptyValueText : QDesk.Msg.get( 'QDesk.view.widget.WMessageChannelProfileCombo.emptyValueText' ),
      valueNotFoundText : QDesk.Msg.get( 'QDesk.view.widget.WMessageChannelProfileCombo.valueNotFoundText' ),
      loadFailedText : QDesk.Msg.get( 'QDesk.view.widget.WMessageChannelProfileCombo.loadFailedText' ),
      loadingText : QDesk.Msg.get( 'QDesk.view.widget.WMessageChannelProfileCombo.loadingText' ),

      root : {
        id : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID,  // ROOT folder ID
        folderId : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID,
        name : QDesk.Msg.get( 'QDesk.view.widget.WMessageChannelProfileCombo.rootNodeName' ),
        expanded : true,
        nodeType : 'folder'
      },

      store : Ext.create( 'QDesk.store.message.MessageChannelProfilesTreeStore' )
    } );

    Ext.apply( this, config );

    me.callParent( [ config ] );
  },


  /**
   * Returns the message channel (QDesk.model.message.channel.MessageChannel) for the currently selected
   * profile.
   * @return the message channel.
   */
  getMessageChannel : function ()
  {
    var me = this;
    var record = me.getSelectedRecord();
    return record == null ? null : record.get( 'messageChannel' );
  }
} );