/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.WTimestampFormatCombo', {
  extend : 'Ext.form.FieldContainer',

  alias : 'widget.WTimestampFormatCombo',

  requires : [
    'QDesk.Msg',

    'QDesk.store.TimestampFormatStore'
  ],

  mixins : {
    field : 'Ext.form.field.Field'
  },

  //configurables
  combineErrors : true,
  msgTarget : 'under',
  layout : 'vbox',
  allowBlank : true,

  /*
   * Initial value.
   */
  value : null,

  /**
   * Additional config options for the format field.
   */
  formatConfig : {},

  formatField : null,
  exampleField : null,

  exampleDate : null,

  /**
   * @cfg {String} readOnlyCls
   * The CSS class applied to the component's main element when it is {@link #readOnly}.
   */
  readOnlyCls : Ext.baseCSSPrefix + 'form-readonly',

  initComponent : function ()
  {
    var me = this;

    me.exampleDate = new Date();

    Ext.apply( me, {
      isFormField : true, // this container will be included in form field queries

      items : [
        Ext.apply( {
          xtype : 'combo',
          store : 'TimestampFormatStore',
          displayField : 'valueLabel',
          valueField : 'valueJava',
          editable : false,

          isFormField : false, //exclude from field query's
          listeners : {
            change : function ( combo, newValue, oldValue )
            {
              //alert( 'Value changed from: ' + oldValue + ' to: ' + newValue );
              if ( newValue )
              {
                me.updateExampleText();
              }
            },
            blur : function ()
            {
              me.onFieldChange();
            },
            scope : me
          },
          submitValue : false,
          validateOnChange : false,
          name : me.name,
          allowBlank : me.allowBlank,
          labelWidth : me.labelWidth,
          minWidth : me.labelWidth + 210
        }, me.formatConfig ),

        {
          xtype : 'displayfield',
          hideEmptyLabel : false,
          height : 18,
          labelWidth : me.labelWidth,
          minWidth : me.labelWidth + 240,
          margin : '-7 0 0 0',
          fieldStyle : {
            fontStyle : 'italic',
            fontSize : '0.9em'
          },
          qdesk : {
            ignore : true
          }
        }
      ]
    } );

    me.callParent();

    me.formatField = me.down( 'combo' );
    me.exampleField = me.down( 'displayfield' );

    me.initField();
  },


  updateExampleText : function ()
  {
    var me = this;

    var value = me.formatField.getValue();
    var record = me.formatField.getStore().findRecord( me.formatField.valueField, value );

    var exampleText = '';
    if ( record )
    {
      // use the JavaScript format value to format the date used in the example
      exampleText = QDesk.Msg.get( 'QDesk.view.widget.WTimestampFormatCombo.exampleText',
          Ext.Date.format( me.exampleDate, record.get( 'valueJs' ) ) );
    }

    me.exampleField.setValue( exampleText );
  },


  // private
  beforeDestroy : function ()
  {
    Ext.destroy( this.fieldCt );
    this.callParent( arguments );
  },


  // private
  delegateFn : function ( fn )
  {
    this.items.each( function ( item )
    {
      if ( item[ fn ] )
      {
        item[ fn ]();
      }
    } );
  },


  // @inheritdoc
  getErrors : function ()
  {
    return [].concat( this.formatField.getErrors() );
  },


  getSubmitValue : function ()
  {
    return this.formatField.getValue();
  },


  /**
   * @return {Date}
   */
  getValue : function ()
  {
    return this.formatField.getValue();
  },


  // @inheritdoc
  isDirty : function ()
  {
    var dirty = false;
    if ( this.rendered && !this.disabled )
    {
      this.items.each( function ( item )
      {
        if ( item.isDirty() )
        {
          dirty = true;
          return false;
        }
      } );
    }
    return dirty;
  },


  // private
  onDisable : function ()
  {
    this.delegateFn( 'disable' );
  },


  // private
  onEnable : function ()
  {
    this.delegateFn( 'enable' );
  },


  // private
  onFieldChange : function ()
  {
    this.fireEvent( 'change', this, this.getValue() );
  },


  // @inheritdoc
  reset : function ()
  {
    this.delegateFn( 'reset' );
  },


  // @inheritdoc
  resetOriginalValue : function ()
  {
    this.formatField.resetOriginalValue();
  },


  markInvalid : function ( errors )
  {
    this.formatField.markInvalid( errors );
  },


  clearInvalid : function ( errors )
  {
    this.formatField.clearInvalid( errors );
  },


  setReadOnly : function ( readOnly )
  {
    this.formatField.setReadOnly( readOnly );
    this[ readOnly ? 'addCls' : 'removeCls' ]( this.readOnlyCls );
    this.readOnly = readOnly;
  },


  setValue : function ( value )
  {
    this.formatField.setValue( value );
  },


  isValid : function ()
  {
    return this.formatField.isValid();
  }
} );