Ext.define( 'QDesk.view.widget.panel.XPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.CurrentUser',
    'QDesk.Log',

    'QDesk.util.constant.EventName'
  ],

  alias : 'widget.XPanel',

  _qdInitialized : false,
  _qdAuthorized : false,
  _qdStandardStateEnabled : false,
  _qdStandardIconCls : null,


  qdRequiresAnyPermissions : null,
  qdNotAuthorizedIconCls : null,


  constructor : function ( config )
  {
    var me = this;
    config = config || {};

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me._qdAuthorized = me.isAuthorized();
    me._qdStandardStateEnabled = !me.isDisabled();
    me._qdStandardIconCls = me.iconCls ? me.iconCls : null;

    // QDesk.Log.info( 'Initialized xpanel: ' + me.getItemId() + ' (' + me.getId() + ')' +
    //     ' [ _qdStandardStateEnabled= ' + me._qdStandardStateEnabled +
    //     ', _qdStandardIconCls=' + me._qdStandardIconCls +
    //     ', _qdAuthorized=' + me._qdAuthorized +
    //     ' ]' );

    if ( me._qdAuthorized )
    {
      me._applyAuthorized();
    }
    else
    {
      me._applyNotAuthorized();
    }

    QDesk.getApplication().on( QDesk.util.constant.EventName.CURRENT_USER_PERMISSIONS_CHANGED,
        function ( params )
        {
          if ( me.destroyed )
          {
            // component has been destroyed, do nothing
            return;
          }

          var oldEffectivePermissions = params.oldEffectivePermissions;
          var newEffectivePermissions = params.newEffectivePermissions;

          var oldAuthorized = me._qdAuthorized;
          var newAuthorized = me.isAuthorized();

          if ( oldAuthorized !== newAuthorized )
          {
            if ( newAuthorized )
            {
              me._applyAuthorized();
            }
            else
            {
              me._applyNotAuthorized();
            }
          }
        } );
  },


  enable : function ( internal )
  {
    var me = this;

    if ( !internal )
    {
      me._qdStandardStateEnabled = true;
    }

    me.callSuper();
  },


  disable : function ( internal )
  {
    var me = this;

    if ( !internal )
    {
      me._qdStandardStateEnabled = false;
    }

    me.callSuper();
  },


  setDisabled : function ( disabled, internal )
  {
    var me = this;

    if ( !internal )
    {
      me._qdStandardStateEnabled = !disabled;
    }

    me.callSuper( [ disabled ] );
  },


  // TODO: remove and replace with isInitialized in all calling objects
  getInitialized : function ()
  {
    return this.isInitialized();
  },

  isInitialized : function ()
  {
    return this._qdInitialized;
  },


  setInitialized : function ( initialized )
  {
    this._qdInitialized = initialized;
  },


  isAuthorized : function ()
  {
    // TODO: refactor to SecurityUtils etc.

    var me = this;

    var requiredAnyPermissions = me.qdRequiresAnyPermissions;
    if ( requiredAnyPermissions == null )
    {
      // authentication / authorization not required
      return true;
    }
    else
    {
      if ( Ext.isEmpty( requiredAnyPermissions ) )
      {
        // required permissions are empty, the user must be at least authenticated
        return QDesk.CurrentUser.isAuthenticated();
      }
      else
      {
        return QDesk.CurrentUser.hasAnyPermissions( requiredAnyPermissions );
      }
    }
  },


  /**
   * This method sets the panel to the desired (saved) state and restores the original icon.
   */
  _applyAuthorized : function ()
  {
    var me = this;

    if ( me.destroyed )
    {
      // do nothing
    }
    else
    {
      me._qdAuthorized = true;

      // restore the standard panel icon (if any)
      me.setIconCls( me._qdStandardIconCls );

      // remove notAuthorized style from the panel's title link
      var tabPanel = me.up( 'tabpanel' );
      if ( tabPanel )
      {
        if ( tabPanel.rendered )
        {
          me._updateTabLink( true );
        }
        else
        {
          tabPanel.on( 'afterrender', function ()
          {
            me._updateTabLink( true );
          }, me );
        }
      }
      else
      {
        // can happen for dynamically added tabs
        //QDesk.Log.warn( 'XPanel ID: ' + me.getId() + ' has no parent tabpanel.' );
      }

      // restore the standard panel state (enabled/disabled)
      me.setDisabled( !me._qdStandardStateEnabled, true );

      // if ( !me.isDisabled() !== me._qdStandardStateEnabled )
      // {
      //   me.setDisabled( !me._qdStandardStateEnabled, true );
      // }
    }
  },


  /**
   * This method disables the panel and makes the panel use the configured "NAUTH" icon.
   */
  _applyNotAuthorized : function ()
  {
    var me = this;

    if ( me.destroyed )
    {
      // do nothing
    }
    else
    {
      me._qdAuthorized = false;

      // set the new NAUTH icon CLS
      var iconCls = me.qdNotAuthorizedIconCls ? me.qdNotAuthorizedIconCls : 'NAUTH';
      me.setIconCls( iconCls );

      // add notAuthorized style to the panel's title link
      var tabPanel = me.up( 'tabpanel' );
      if ( tabPanel )
      {
        if ( tabPanel.rendered )
        {
          me._updateTabLink( false );
        }
        else
        {
          tabPanel.on( 'afterrender', function ()
          {
            me._updateTabLink( false );
          }, me );
        }
      }
      else
      {
        // can happen for dynamically added tabs
        //QDesk.Log.warn( 'XPanel ID: ' + me.getId() + ' has no parent tabpanel.' );
      }

      // disable the panel
      // QDesk.Log.info( '_applyNotAuthorized disabling tab: ' + me.getItemId() );

      me.disable( true );
    }
  },


  _updateTabLink : function ( authorized )
  {
    var me = this;

    var tabPanel = me.up( 'tabpanel' );

    if ( tabPanel )
    {
      var tabIndex = tabPanel.items.indexOf( me );
      if ( tabIndex >= 0 )
      {
        var tabTitleLink = tabPanel.getTabBar().items.getAt( tabIndex );
        if ( authorized )
        {
          tabTitleLink.removeCls( 'notAuthorized' );
        }
        else
        {
          tabTitleLink.addCls( 'notAuthorized' );
        }
      }
    }
  }
} );