/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.ExceptionErrorWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg'
  ],

  alias : 'widget.ExceptionErrorWindow',

  // cls : 'exceptionErrorWindow',
  // iconCls : 'exceptionErrorWindowIcon',

  scrollable : false,

  modal : true,
  resizable : true,
  maximizable : true,

  // width : 500,
  // height : 500,
  //

  minWidth : 600,
  minHeight : 300,

  width : 600,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  qdesk : {
    /*
     * Required error text.
     */
    errorText : undefined,

    /*
     * Required exception text.
     */
    exceptionText : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.errorText )
      Ext.Error.raise( 'Required config parameter missing: qdesk.errorText' );

    if ( !config.qdesk.exceptionText )
      Ext.Error.raise( 'Required config parameter missing: qdesk.exceptionText' );

    Ext.apply( config, {
      items : [
        {
          xtype : 'container',
          layout : {
            type : 'hbox',
            align : 'center',
            pack : 'start'
          },
          padding : 10,
          items : [
            {
              xtype : 'component',
              cls : 'errorIcon'
            },
            {
              xtype : 'container',
              flex : 1,
              html : config.qdesk.errorText,
              margin : '0 0 0 8'
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.ExceptionErrorWindow.fieldsetLabelException' ),
          flex : 1,
          layout : {
            type : 'fit'
          },
          items : [
            {
              xtype : 'htmleditor',
              itemId : 'exceptionEditor',
              enableAlignments : false,
              enableColors : false,
              enableFont : false,
              enableFontSize : false,
              enableFormat : false,
              enableLinks : false,
              enableLists : false,
              enableSourceEdit : false,
              padding : '0 0 5 0',
              minHeight : 150
            }
          ]
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          layout : {
            pack : 'center'
          },
          defaults : {
            minWidth : 75
          },
          items : [
            {
              // xtype : 'button',
              itemId : 'okButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelOk' )
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    var exceptionTextColor = QDesk.ApplicationConfig.get( 'viewError.textColor' );

    var formattedExceptionText = '<pre style="margin-bottom: 0; margin-top: 0;">';
    formattedExceptionText += '<span style="color: ' + exceptionTextColor + '">';
    formattedExceptionText += Ext.String.htmlEncode( config.qdesk.exceptionText );
    formattedExceptionText += '</span>';
    formattedExceptionText += '</pre>';

    var exceptionEditor = me.down( '#exceptionEditor' );
    exceptionEditor.setValue( formattedExceptionText );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var exceptionEditor = me.down( '#exceptionEditor' );

    // for some reason the empty toolbar is not hidden
    exceptionEditor.getToolbar().hide();
    exceptionEditor.setReadOnly( true );

    var okButton = me.down( '#okButton' );
    okButton.on( 'click', function ()
    {
      me.close();

      // invoke the callback function if specified
      if ( Ext.isFunction( me.config.qdesk.callback ) )
      {
        me.config.qdesk.callback.apply( me );
      }
    } );
  }
} );