/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.audit.AuditTrailWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.audit.AuditTrailGrid'
  ],

  alias : 'widget.AuditTrailWindow',

  cls : 'auditTrailWindow',
  iconCls : 'windowIcon',

  scrollable : true,

  modal : true,
  resizable : true,
  maximizable : true,

  width : 800,
  height : 500,

  minWidth : 800,
  minHeight : 500,

  layout : 'fit',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailWindow.title' ),
      items : [
        {
          xtype : 'AuditTrailGrid',
          itemId : 'auditTrailGrid',
          qdesk : {
            gridId : config.qdesk.gridId,
            connectionCtx : config.qdesk.connectionCtx  // optional param, available only in some contexts!
          }
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'closeButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } );
  }
} );