/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.config.general.GeneralPanel', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.System',

    'QDesk.model.common.TimeZone',

    'QDesk.view.widget.WDateFormatCombo',
    'QDesk.view.widget.WTimestampFormatCombo'
  ],

  alias : 'widget.GeneralPanel',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var timeZoneStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.common.TimeZone'
    } );
    timeZoneStore.loadData( QDesk.System.getAvailableTimeZones() );

    Ext.apply( config, {
      items : [
        {
          xtype : 'fieldset',
          fieldDefaults : {
            labelWidth : 110,
            width : 200
          },
          title : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldsetLabelInfoMessages' ),
          items : [
            {
              xtype : 'checkboxfield',
              itemId : 'infoBubblesEnabledCheckbox',
              name : 'infoBubblesEnabled',
              boxLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.boxLabelInfoBubblesEnabled' ),
              boxTooltip : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.boxTooltipInfoBubblesEnabled' )
            },

            {
              xtype : 'fieldcontainer',
              itemId : 'infoBubblesTimeoutContainer',
              layout : {
                type : 'hbox',
                align : 'left'
              },
              items : [
                {
                  xtype : 'numberfield',
                  name : 'infoBubblesTimeout',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.config.general.GeneralPanel.fieldLabelInfoBubblesTimeout' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.config.general.GeneralPanel.fieldTooltipInfoBubblesTimeout' ),
                  minValue : 1,
                  maxValue : 20,
                  allowBlank : false
                },
                {
                  xtype : 'panel',
                  border : false,
                  height : 22,
                  bodyPadding : '3 0 0 5',
                  html : QDesk.Msg.get( 'QDesk.seconds' )
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.model.license.ProductEdition.LITE,
              QDesk.model.license.ProductEdition.STANDARD,
              QDesk.model.license.ProductEdition.ENTERPRISE
            ]
          }
        },

        {
          xtype : 'fieldset',
          fieldDefaults : {
            labelWidth : 110,
            width : 440
          },
          title : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldsetLabelDateAndTime' ),
          items : [
            {
              xtype : 'combo',
              name : 'timeZone',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.config.general.GeneralPanel.fieldLabelTimeZone' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.config.general.GeneralPanel.fieldTooltipTimeZone' ),
              displayField : 'displayName',
              valueField : 'id',
              allowBlank : false,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : timeZoneStore,
              qdesk : {
                licensedFor : [
                  QDesk.model.license.ProductEdition.LITE,
                  QDesk.model.license.ProductEdition.STANDARD,
                  QDesk.model.license.ProductEdition.ENTERPRISE
                ]
              }
            },
            {
              xtype : 'WDateFormatCombo',
              name : 'dateFormat',
              labelWidth : 110,
              formatConfig : {
                allowBlank : false,
                fieldLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldLabelDateFormat' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.config.general.GeneralPanel.fieldTooltipDateFormat' ),
                qdesk : {
                  licensedFor : [
                    QDesk.model.license.ProductEdition.LITE,
                    QDesk.model.license.ProductEdition.STANDARD,
                    QDesk.model.license.ProductEdition.ENTERPRISE
                  ]
                }
              }
            },
            {
              xtype : 'WTimestampFormatCombo',
              name : 'timestampFormat',
              labelWidth : 110,
              formatConfig : {
                allowBlank : false,
                fieldLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldLabelTimestampFormat' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.config.general.GeneralPanel.fieldTooltipTimestampFormat' ),
                qdesk : {
                  licensedFor : [
                    QDesk.model.license.ProductEdition.LITE,
                    QDesk.model.license.ProductEdition.STANDARD,
                    QDesk.model.license.ProductEdition.ENTERPRISE
                  ]
                }
              }
            }
          ]
        },

        {
          xtype : 'fieldset',
          fieldDefaults : {
            labelWidth : 100,
            width : 200
          },
          title : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldsetLabelJavaScript' ),
          items : [
            {
              xtype : 'fieldcontainer',
              layout : {
                type : 'hbox',
                align : 'left'
              },
              items : [
                {
                  xtype : 'numberfield',
                  name : 'ajaxTimeout',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldLabelJsAjaxTimeout' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.config.general.GeneralPanel.fieldTooltipJsAjaxTimeout' ),
                  minValue : 10,
                  allowBlank : false
                },
                {
                  xtype : 'panel',
                  border : false,
                  height : 22,
                  bodyPadding : '3 0 0 5',
                  html : QDesk.Msg.get( 'QDesk.seconds' )
                }
              ],
              qdesk : {
                licensedFor : [
                  QDesk.model.license.ProductEdition.LITE,
                  QDesk.model.license.ProductEdition.STANDARD,
                  QDesk.model.license.ProductEdition.ENTERPRISE
                ]
              }
            },

            {
              xtype : 'combo',
              name : 'jsLoggingLevel',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldLabelJsLoggingLevel' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldTooltipJsLoggingLevel' ),
              displayField : 'valueLabel',
              valueField : 'value',
              store : 'enums.JsLoggingLevelStore',
              allowBlank : false,
              editable : false,
              forceSelection : true,
              qdesk : {
                licensedFor : [
                  QDesk.model.license.ProductEdition.LITE,
                  QDesk.model.license.ProductEdition.STANDARD,
                  QDesk.model.license.ProductEdition.ENTERPRISE
                ]
              }
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  setValues : function ( data )
  {
    var me = this;

    me.callParent( [ data ] );

    var combo = me.down( 'combo[name=timeZone]' );
    combo.setValue( data.timeZone.id );
  }
} );
