/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.config.tasks.TasksPanel', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg'
  ],

  alias : 'widget.TasksPanel',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [

        /* Start of auditTrailPurge field set */
        {
          xtype : 'fieldset',
          itemId : 'auditTrailPurge',
          title : QDesk.Msg.get(
              'QDesk.view.work.config.tasks.TasksPanel.fieldsetLabelAuditTrailPurge' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.config.tasks.TasksPanel.fieldsetTooltipAuditTrailPurge' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          fieldDefaults : {
            labelWidth : 110,
            width : 200
          },
          items : [
            {
              xtype : 'checkboxfield',
              itemId : 'auditTrailPurgeEnabled',
              name : 'tasks.auditTrail.purge.enabled',
              boxLabel : QDesk.Msg.get(
                  'QDesk.view.work.config.tasks.TasksPanel.boxLabelAuditTrailPurgeEnabled' ),
              boxTooltip : QDesk.Msg.get(
                  'QDesk.view.work.config.tasks.TasksPanel.boxTooltipAuditTrailPurgeEnabled' ),
              labelAlign : 'right',
              allowBlank : true
            },
            {
              xtype : 'fieldcontainer',
              layout : {
                type : 'hbox',
                align : 'left'
              },
              disabled : true,
              items : [
                {
                  xtype : 'numberfield',
                  itemId : 'auditTrailPurgeKeepDays',
                  name : 'tasks.auditTrail.purge.keepDays',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.config.tasks.TasksPanel.fieldLabelAuditTrailPurgeKeepDays' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.config.tasks.TasksPanel.fieldTooltipAuditTrailPurgeKeepDays' ),
                  allowBlank : false,
                  minValue : 1,
                  maxValue : 65535
                }
              ]
            }
          ],
          qdesk : {
            licensedFor : [
              QDesk.model.license.ProductEdition.STANDARD,
              QDesk.model.license.ProductEdition.ENTERPRISE
            ]
          }
        }
        /* End of auditTrailPurge field set */
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var taskStatusCheckboxes = [
      me.down( '#auditTrailPurgeEnabled' )
    ];

    Ext.each( taskStatusCheckboxes, function ( checkbox )
    {
      checkbox.on( 'change', me._taskStatusChanged, me );
    } );

    // set the initial state of individual task configuration fieldcontainers using the current task status values
    Ext.each( taskStatusCheckboxes, function ( checkbox )
    {
      me._taskStatusChanged( checkbox, checkbox.getValue() );
    } );
  },


  _taskStatusChanged : function ( checkbox, taskEnabled )
  {
    checkbox.nextSibling( 'fieldcontainer' ).setDisabled( !taskEnabled );
  }
} );
