/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.MessageChannelProfilesWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.message.channel.MessageChannelProfilesTreePanel',
    'QDesk.view.work.message.channel.email.EmailProfilePanel',
    'QDesk.view.work.message.channel.im.aim.AimProfilePanel',
    'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel',
    'QDesk.view.work.message.channel.im.icq.IcqProfilePanel',
    'QDesk.view.work.message.channel.im.xmpp.XmppProfilePanel',
    'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel',
    'QDesk.view.work.message.channel.web_service.WebServiceProfilePanel'
  ],

  alias : 'widget.MessageChannelProfilesWindow',

  cls : 'messageChannelProfilesWindow',
  iconCls : 'windowIcon',

  modal : true,
  resizable : true,

  width : 850,
  minWidth : 850,

  height : 650,
  minHeight : 650,

  layout : {
    type : 'border'
  },

  qdesk : {},


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.message.channel.MessageChannelProfilesWindow.windowTitle' ),

      items : [
        {
          xtype : 'MessageChannelProfilesTreePanel',
          itemId : 'messageChannelProfilesTreePanel',
          region : 'west',
          split : true,
          header : false,
          collapsible : true,
          collapseMode : 'mini',
          width : 250
        },
        {
          xtype : 'panel',
          itemId : 'editPanel',
          region : 'center',
          layout : 'card',
          items : [
            {
              xtype : 'panel',
              itemId : 'infoPanel',
              border : false,
              bodyPadding : 5,
              items : [
                {
                  xtype : 'InfoMessagePanel',
                  qdesk : {
                    msg : QDesk.Msg.get( 'QDesk.view.work.message.channel.MessageChannelProfilesWindow.infoText' )
                  }
                }
              ]
            }
          ]
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'closeButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } )
  }
} )
;
