/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.Properties'
  ],

  alias : 'widget.CollaborationSlackBasicPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 95
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.tabLabelBasic' ),

      items : [
        {
          xtype : 'textfield',
          itemId : 'webhookUrl',
          name : 'webhookUrl',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldLabelWebhookUrl' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldTooltipWebhookUrl' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldEmptyTxtWebhookUrl' ),
          allowBlank : false,
          maxLength : 128,
          vtype : 'url',
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'channel',
          name : 'channel',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldLabelChannel' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldTooltipChannel' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldEmptyTxtChannel' ),
          allowBlank : true,
          maxLength : 64,
          qdesk : {
            required : false
          }
        },


        {
          xtype : 'checkboxfield',
          itemId : 'messageFormattingEnabled',
          name : 'messageFormattingEnabled',
          boxLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelMessageFormattingEnabled' ),
          boxTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipMessageFormattingEnabled' ),
          value : false
        },


        // BOT username fields
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldsetLabelBotUsername' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldsetTooltipBotUsername' ),
          padding : '5 5 0 5',
          defaults : {
            labelWidth : 95
          },
          items : [
            {
              xtype : 'radiogroup',
              itemId : 'botUsername',
              columns : 1,
              vertical : true,
              allowBlank : false,
              layout : {
                type : 'vbox',
                align : 'stretch'
              },
              qdesk : {
                required : true
              },
              items : [
                {
                  xtype : 'radiofield',
                  itemId : 'useDefaultBotUsername',
                  name : 'botUsername',
                  inputValue : 'useDefaultBotUsername',
                  boxLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelUseDefaultBotUsername' ),
                  boxTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipUseDefaultBotUsername' ),
                  checked : true
                },

                {
                  xtype : 'fieldcontainer',
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : [
                    {
                      xtype : 'radiofield',
                      itemId : 'useConfiguredBotUsername',
                      name : 'botUsername',
                      inputValue : 'useConfiguredBotUsername',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelUseConfiguredBotUsername' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipUseConfiguredBotUsername' )
                    },
                    {
                      xtype : 'textfield',
                      margin : '0 0 0 17',
                      value : QDesk.Properties.get( 'messageChannel.collaboration.slack.botUsername' ),
                      maxLength : 64,
                      readOnly : true
                    }
                  ]
                },

                {
                  xtype : 'fieldcontainer',
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : [
                    {
                      xtype : 'radiofield',
                      itemId : 'useCustomBotUsername',
                      name : 'botUsername',
                      inputValue : 'useCustomBotUsername',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelUseCustomBotUsername' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipUseCustomBotUsername' )
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'customBotUsername',
                      name : 'customBotUsername',
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldEmptyTxtCustomBotUsername' ),
                      disabled : true,
                      margin : '0 0 0 17',
                      allowBlank : false,
                      maxLength : 64,
                      qdesk : {
                        required : true
                      }
                    }
                  ]
                }
              ]
            }
          ]
        },


        // BOT icon fields
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldsetLabelBotIcon' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldsetTooltipBotIcon' ),
          padding : '5 5 0 5',
          layout : {
            type : 'vbox',
            align : 'stretch'
          },
          defaults : {
            labelWidth : 95
          },
          items : [
            {
              xtype : 'radiogroup',
              itemId : 'botIcon',
              columns : 1,
              vertical : true,
              layout : {
                type : 'vbox',
                align : 'stretch'
              },
              allowBlank : false,
              qdesk : {
                required : true
              },
              items : [
                {
                  xtype : 'radiofield',
                  itemId : 'useDefaultBotIcon',
                  name : 'botIcon',
                  inputValue : 'useDefaultBotIcon',
                  boxLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelUseDefaultBotIcon' ),
                  boxTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipUseDefaultBotIcon' ),
                  checked : true
                },

                {
                  xtype : 'fieldcontainer',
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : [
                    {
                      xtype : 'radiofield',
                      itemId : 'useConfiguredBotIcon',
                      name : 'botIcon',
                      inputValue : 'useConfiguredBotIcon',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelUseConfiguredBotIcon' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipUseConfiguredBotIcon' )
                    },
                    {
                      xtype : 'textfield',
                      margin : '0 0 0 17',
                      value : QDesk.Properties.get( 'messageChannel.collaboration.slack.botIconUrl' ),
                      maxLength : 128,
                      vtype : 'url',
                      readOnly : true
                    }
                  ]
                },

                {
                  xtype : 'fieldcontainer',
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : [
                    {
                      xtype : 'radiofield',
                      itemId : 'useCustomBotIcon',
                      name : 'botIcon',
                      inputValue : 'useCustomBotIcon',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxLabelUseCustomBotIcon' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.boxTooltipUseCustomBotIcon' )
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'customBotIconUrl',
                      name : 'customBotIconUrl',
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel.fieldEmptyTxtCustomBotIconUrl' ),
                      disabled : true,
                      margin : '0 0 0 17',
                      allowBlank : false,
                      maxLength : 128,
                      vtype : 'url',
                      qdesk : {
                        required : true
                      }
                    }
                  ]
                }
              ]
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var botUsernameRadioGroup = me.down( '#botUsername' );
    botUsernameRadioGroup.on( 'change', function ( radioGroup, newValue, oldValue )
    {
      var customBotUsernameField = me.down( '#customBotUsername' );

      if ( newValue.botUsername === 'useCustomBotUsername' )
      {
        customBotUsernameField.enable();
      }
      else
      {
        //customBotUsernameField.setValue( null );
        customBotUsernameField.disable();
      }
    } );

    var botIconRadioGroup = me.down( '#botIcon' );
    botIconRadioGroup.on( 'change', function ( radioGroup, newValue, oldValue )
    {
      var customBotIconUrlField = me.down( '#customBotIconUrl' );

      if ( newValue.botIcon === 'useCustomBotIcon' )
      {
        customBotIconUrlField.enable();
      }
      else
      {
        //customBotIconUrlField.setValue( null );
        customBotIconUrlField.disable();
      }
    } );
  }
} );
