/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.collaboration.slack.SlackProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.view.work.message.channel.HttpProxyPanel',
    'QDesk.view.work.message.channel.collaboration.slack.SlackBasicPanel'
  ],

  alias : 'widget.CollaborationSlackProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3 10 5 10',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'COLLABORATION_SLACK',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.work.message.channel.collaboration.slack.SlackProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.collaboration.slack.SlackProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.collaboration.slack.SlackProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.collaboration.slack.SlackProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              maxLength : 128,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'CollaborationSlackBasicPanel'
            },
            {
              xtype : 'HttpProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessage' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    // uncheck all radios in botUsername radiogroup
    Ext.each( Ext.ComponentQuery.query( '#botUsername radiofield', me ),
        function ( radiofield )
        {
          radiofield.setValue( false );
        } );

    // uncheck all radios in botIcon radiogroup
    Ext.each( Ext.ComponentQuery.query( '#botIcon radiofield', me ),
        function ( radiofield )
        {
          radiofield.setValue( false );
        } );

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      webhookUrl : profile.channelConfig.webhookUrl,
      messageFormattingEnabled : profile.channelConfig.messageFormattingEnabled,
      channel : profile.channelConfig.channel
    } );

    // bot username
    if ( profile.channelConfig.useDefaultBotUsername )
    {
      Ext.apply( values, {
        botUsername : 'useDefaultBotUsername'
      } );
    }

    if ( profile.channelConfig.useConfiguredBotUsername )
    {
      Ext.apply( values, {
        botUsername : 'useConfiguredBotUsername'
      } );
    }

    if ( profile.channelConfig.useCustomBotUsername )
    {
      Ext.apply( values, {
        botUsername : 'useCustomBotUsername',
        customBotUsername : profile.channelConfig.customBotUsername
      } );
    }

    // bot icon
    if ( profile.channelConfig.useDefaultBotIcon )
    {
      Ext.apply( values, {
        botIcon : 'useDefaultBotIcon'
      } );
    }

    if ( profile.channelConfig.useConfiguredBotIcon )
    {
      Ext.apply( values, {
        botIcon : 'useConfiguredBotIcon'
      } );
    }

    if ( profile.channelConfig.useCustomBotIcon )
    {
      Ext.apply( values, {
        botIcon : 'useCustomBotIcon',
        customBotIconUrl : profile.channelConfig.customBotIconUrl
      } );
    }

    var proxyPanel = me.down( 'HttpProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      webhookUrl : fieldValues.webhookUrl,
      messageFormattingEnabled : fieldValues.messageFormattingEnabled
    } );

    if ( !Ext.isEmpty( fieldValues.channel ) )
    {
      Ext.apply( channelConfig, {
        channel : fieldValues.channel
      } );
    }

    // bot username
    if ( fieldValues.botUsername === 'useDefaultBotUsername' )
    {
      Ext.apply( channelConfig, {
        useDefaultBotUsername : true,
        useConfiguredBotUsername : false,
        useCustomBotUsername : false
      } );
    }
    else if ( fieldValues.botUsername === 'useConfiguredBotUsername' )
    {
      Ext.apply( channelConfig, {
        useDefaultBotUsername : false,
        useConfiguredBotUsername : true,
        useCustomBotUsername : false
      } );
    }
    else if ( fieldValues.botUsername === 'useCustomBotUsername' )
    {
      Ext.apply( channelConfig, {
        useDefaultBotUsername : false,
        useConfiguredBotUsername : false,
        useCustomBotUsername : true,

        customBotUsername : fieldValues.customBotUsername
      } );
    }
    else
    {
      Ext.Error.raise( 'Unexpected value of botUsername form value: ' + fieldValues.botUsername );
    }

    // bot icon
    if ( fieldValues.botIcon === 'useDefaultBotIcon' )
    {
      Ext.apply( channelConfig, {
        useDefaultBotIcon : true,
        useConfiguredBotIcon : false,
        useCustomBotIcon : false
      } );
    }
    else if ( fieldValues.botIcon === 'useConfiguredBotIcon' )
    {
      Ext.apply( channelConfig, {
        useDefaultBotIcon : false,
        useConfiguredBotIcon : true,
        useCustomBotIcon : false
      } );
    }
    else if ( fieldValues.botIcon === 'useCustomBotIcon' )
    {
      Ext.apply( channelConfig, {
        useDefaultBotIcon : false,
        useConfiguredBotIcon : false,
        useCustomBotIcon : true,

        customBotIconUrl : fieldValues.customBotIconUrl
      } );
    }
    else
    {
      Ext.Error.raise( 'Unexpected value of botIcon form value: ' + fieldValues.botIcon );
    }


    var proxyPanel = me.down( 'HttpProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
