/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.Properties',

    'QDesk.model.message.channel.incident_mgmt.JiraIssueType',

    'QDesk.util.EnumUtils'
  ],

  alias : 'widget.IncidentMgmtJiraBasicPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 90
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get(
          'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.tabLabelBasic' ),

      items : [
        {
          xtype : 'textfield',
          itemId : 'username',
          name : 'username',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldLabelUsername' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldTooltipUsername' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldEmptyTxtUsername' ),
          allowBlank : false,
          maxLength : 64,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'apiToken',
          name : 'apiToken',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldLabelApiToken' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldTooltipApiToken' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldEmptyTxtApiToken' ),
          allowBlank : false,
          maxLength : 64,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'baseUrl',
          name : 'baseUrl',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldLabelBaseUrl' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldTooltipBaseUrl' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldEmptyTxtBaseUrl' ),
          allowBlank : false,
          maxLength : 128,
          vtype : 'url',
          qdesk : {
            required : true
          }
        },

        // issue properties
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldsetLabelIssueProperties' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldsetTooltipIssueProperties' ),
          padding : '5 5 0 5',
          defaults : {
            anchor : '100%',
            labelWidth : 85
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'projectKey',
              name : 'projectKey',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldLabelProjectKey' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldTooltipProjectKey' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldEmptyTxtProjectKey' ),
              allowBlank : false,
              maxLength : 64,
              qdesk : {
                required : true
              }
            },

            {
              xtype : 'combo',
              itemId : 'issueType',
              name : 'issueType',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldLabelIssueType' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldTooltipIssueType' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel.fieldEmptyTxtIssueType' ),
              anchor : '60%',
              value : '',
              editable : true,
              forceSelection : false,
              queryMode : 'local',
              store : new Ext.data.ArrayStore( {
                fields : [
                  'value',
                  'valueLabel'
                ],
                data : [
                  [ QDesk.model.message.channel.incident_mgmt.JiraIssueType.BUG,
                    QDesk.util.EnumUtils.getJiraIssueTypeName(
                        QDesk.model.message.channel.incident_mgmt.JiraIssueType.BUG ) ],
                  [ QDesk.model.message.channel.incident_mgmt.JiraIssueType.EPIC,
                    QDesk.util.EnumUtils.getJiraIssueTypeName(
                        QDesk.model.message.channel.incident_mgmt.JiraIssueType.EPIC ) ],
                  [ QDesk.model.message.channel.incident_mgmt.JiraIssueType.STORY,
                    QDesk.util.EnumUtils.getJiraIssueTypeName(
                        QDesk.model.message.channel.incident_mgmt.JiraIssueType.STORY ) ],
                  [ QDesk.model.message.channel.incident_mgmt.JiraIssueType.TASK,
                    QDesk.util.EnumUtils.getJiraIssueTypeName(
                        QDesk.model.message.channel.incident_mgmt.JiraIssueType.TASK ) ]
                ]
              } ),
              valueField : 'value',
              displayField : 'valueLabel',
              allowBlank : false,
              maxLength : 64,
              qdesk : {
                required : true
              }
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    // set default issue type
    var issueTypeCombo = me.down( '#issueType' );
    issueTypeCombo.setValue( QDesk.model.message.channel.incident_mgmt.JiraIssueType.BUG );
  }
} );
