/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.incident_mgmt.opsgenie.OpsGenieProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.view.work.message.channel.HttpProxyPanel',
    'QDesk.view.work.message.channel.incident_mgmt.opsgenie.OpsGenieBasicPanel'
  ],

  alias : 'widget.IncidentMgmtOpsGenieProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3 10 5 10',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'INCIDENT_MGMT_OPSGENIE',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.opsgenie.OpsGenieProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.opsgenie.OpsGenieProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.opsgenie.OpsGenieProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.opsgenie.OpsGenieProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              maxLength : 128,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'IncidentMgmtOpsGenieBasicPanel'
            },
            {
              xtype : 'HttpProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessage' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      apiKey : profile.channelConfig.apiKey,
      source : profile.channelConfig.source,
      priority : profile.channelConfig.priority,
      entity : profile.channelConfig.entity
    } );

    if ( !Ext.isEmpty( profile.channelConfig.tag ) )
    {
      Ext.apply( values, {
        tag : profile.channelConfig.tag
      } );
    }

    if ( !Ext.isEmpty( profile.channelConfig.action ) )
    {
      Ext.apply( values, {
        action : profile.channelConfig.action
      } );
    }

    var proxyPanel = me.down( 'HttpProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.INCIDENT_MGMT_OPSGENIE );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      apiKey : fieldValues.apiKey,
      source : fieldValues.source,
      priority : fieldValues.priority,
      entity : fieldValues.entity
    } );

    // tag array
    if ( !Ext.isEmpty( fieldValues.tag ) )
    {
      Ext.apply( channelConfig, {
        tag : fieldValues.tag
      } );
    }

    // action array
    if ( !Ext.isEmpty( fieldValues.action ) )
    {
      Ext.apply( channelConfig, {
        action : fieldValues.action
      } );
    }

    var proxyPanel = me.down( 'HttpProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
