/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.message.channel.web_service.MessageReceiverWSVersion',
    'QDesk.store.enums.MessageReceiverWSVersionStore'
  ],

  alias : 'widget.WebServiceBasicPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 80
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.tabLabelBasic' ),

      items : [
        {
          xtype : 'combo',
          itemId : 'version',
          name : 'version',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldLabelVersion' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldTooltipVersion' ),
          value : QDesk.model.message.channel.web_service.MessageReceiverWSVersion.V5_0,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.MessageReceiverWSVersionStore',
          valueField : 'value',
          displayField : 'valueLabel',
          allowBlank : false,
          maxLength : 64,
          qdesk : {
            required : true
          }
        },
        {
          xtype : 'textfield',
          itemId : 'endpointUrl',
          name : 'endpointUrl',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldLabelEndpoint' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldTooltipEndpoint' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldEmptyTxtEndpoint' ),
          allowBlank : false,
          maxLength : 128,
          vtype : 'url',
          qdesk : {
            required : true
          }
        },
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldsetLabelAuthentication' ),
          padding : '5 0 0 5',
          defaults : {
            labelWidth : 75
          },
          items : [
            {
              xtype : 'combo',
              itemId : 'authenticationType',
              name : 'authenticationType',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldLabelAuthenticationType' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldTooltipAuthenticationType' ),
              value : '',
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : 'enums.WSAuthenticationTypeStore',
              valueField : 'value',
              displayField : 'valueLabel',
              allowBlank : false,
              maxLength : 64,
              qdesk : {
                required : true
              }
            },

            {
              xtype : 'textfield',
              itemId : 'authenticationUsername',
              name : 'authenticationUsername',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldLabelUsername' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldTooltipUsername' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldEmptyTxtUsername' ),
              allowBlank : false,
              maxLength : 64,
              qdesk : {
                required : true
              }
            },
            {
              xtype : 'textfield',
              inputType : 'password',
              itemId : 'authenticationPassword',
              name : 'authenticationPassword',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldLabelPassword' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldTooltipPassword' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.web_service.WebServiceBasicPanel.fieldEmptyTxtPassword' ),
              allowBlank : false,
              maxLength : 64,
              qdesk : {
                required : true
              }
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var authenticationTypeCombo = me.down( '#authenticationType' );

    /*
     * Before the form is rendered and whenever the authentication type combo value changes,
     * enable/disable and adjust value of allowBlank attributes of the username and password fields.
     */
    authenticationTypeCombo.on( 'beforerender', me._updateAuthenticationFields, me );
    authenticationTypeCombo.on( 'change', me._updateAuthenticationFields, me );
  },


  /**
   * Enables/disables authentication username and password field based on the current authentication type selection.
   * @private
   */
  _updateAuthenticationFields : function ()
  {
    var me = this;

    var authenticationTypeCombo = me.down( '#authenticationType' );
    var usernameField = me.down( '#authenticationUsername' );
    var passwordField = me.down( '#authenticationPassword' );

    if ( Ext.isEmpty( authenticationTypeCombo.getValue() ) )
    {
      // None authentication selected
      usernameField.setValue( null );
      usernameField.setDisabled( true );
      usernameField.allowBlank = true;

      passwordField.setValue( null );
      passwordField.setDisabled( true );
      passwordField.allowBlank = true;
    }
    else
    {
      usernameField.setDisabled( false );
      usernameField.allowBlank = false;

      passwordField.setDisabled( false );
      passwordField.allowBlank = false;
    }

    usernameField.validate();
    passwordField.validate();
  }
} );
