/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.ConnectionContext',
    'QDesk.JvmAgentInfoAccessor',
    'QDesk.SchedulerInfoAccessor',

    'QDesk.controller.command.connection.AddConnectionFolder',
    'QDesk.controller.command.connection.AddSchedulerConnection',

    'QDesk.controller.command.connection.EditConnectionFolder',
    'QDesk.controller.command.connection.EditSchedulerConnection',
    'QDesk.controller.command.connection.DeleteConnectionFolder',
    'QDesk.controller.command.connection.DeleteSchedulerConnection',
    'QDesk.controller.command.connection.TestSchedulerConnection',

    'QDesk.controller.command.connection.MoveConnectionFolder',
    'QDesk.controller.command.connection.MoveSchedulerConnection',

    'QDesk.controller.command.connection.RefreshConnectionFolder',

    'QDesk.controller.command.scheduler.common.StartScheduler',
    'QDesk.controller.command.scheduler.common.PauseScheduler',
    'QDesk.controller.command.scheduler.common.StopScheduler',

    'QDesk.controller.command.scheduler.common.StartAllSchedulers',
    'QDesk.controller.command.scheduler.common.PauseAllSchedulers',
    'QDesk.controller.command.scheduler.common.StopAllSchedulers',

    'QDesk.controller.command.scheduler.quartz.OpenSchedulerView',

    'QDesk.model.scheduler.SchedulerType',
    'QDesk.model.security.Permission',

    'QDesk.store.ConnectionsTreeStore',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.ConnectionsTreePanel',

  cls : 'connectionsTreePanel',

  //scrollable : true,

  displayField : 'name',

  rootVisible : true,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      root : {
        id : QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID,
        folderId : QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID,
        name : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.rootNodeName' ),
        expanded : true,
        nodeType : 'folder'
      },

      viewConfig : {
        copy : false,

        loadingText : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.maskLoadMsg' ),

        plugins : {
          ptype : 'treeviewdragdrop',
          appendOnly : true,
          dragText : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.dragText' )
        }
      },

      dockedItems : [
        {
          xtype : 'toolbar',
          border : false,  // fixes the visual glitch with thick border on the top, left and right
          itemId : 'toolBar',
          dock : 'top',
          items : [
            {
              xtype : 'button',
              itemId : 'refreshButton',
              scale : 'small',
              iconCls : 'refresh',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.ConnectionsTreePanel.tooltipRefreshButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [],
              qdNotAuthorizedIconCls : 'refreshNAUTH'
            },
            {
              xtype : 'button',
              itemId : 'collapseAllButton',
              scale : 'small',
              iconCls : 'collapseAll',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.ConnectionsTreePanel.tooltipCollapseAllButton' ),
              disabled : true
            },
            '-',
            {
              xtype : 'button',
              itemId : 'addFolderOrConnectionButton',
              scale : 'small',
              iconCls : 'addFolderOrConnection',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.ConnectionsTreePanel.tooltipAddFolderOrConnectionButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [
                QDesk.model.security.Permission.CONNECTION_FLD_ADD,
                QDesk.model.security.Permission.CONNECTION_ADD
              ],
              qdNotAuthorizedIconCls : 'addFolderOrConnectionNAUTH',
              menu : [
                {
                  itemId : 'addFolderButton',
                  text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.addFolderButton' ),
                  iconCls : 'addFolder',
                  qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_FLD_ADD ],
                  qdNotAuthorizedIconCls : 'addFolderNAUTH'
                },
                {
                  itemId : 'addConnectionButton',
                  text : QDesk.Msg.get(
                      'QDesk.view.work.navigation.connections.ConnectionsTreePanel.addConnectionButton' ),
                  iconCls : 'quartzAddConnection',
                  qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_ADD ],
                  qdNotAuthorizedIconCls : 'quartzAddConnectionNAUTH'
                }
              ]
            },
            {
              xtype : 'button',
              itemId : 'deleteFolderOrConnectionButton',
              scale : 'small',
              iconCls : 'deleteFolderOrConnection',
              // no tooltip, it is set dynamically based on the selection (folder / connection)
              disabled : true,
              qdRequiresAnyPermissions : [
                QDesk.model.security.Permission.CONNECTION_FLD_DELETE,
                QDesk.model.security.Permission.CONNECTION_DELETE
              ],
              qdNotAuthorizedIconCls : 'deleteFolderOrConnectionNAUTH'
            },
            {
              xtype : 'button',
              itemId : 'editFolderOrConnectionButton',
              scale : 'small',
              iconCls : 'editFolderOrConnection',
              // no tooltip, it is set dynamically based on the selection (folder / connection)
              disabled : true,
              qdRequiresAnyPermissions : [
                QDesk.model.security.Permission.CONNECTION_FLD_UPDATE,
                QDesk.model.security.Permission.CONNECTION_UPDATE
              ],
              qdNotAuthorizedIconCls : 'editFolderOrConnectionNAUTH'
            },
            {
              xtype : 'button',
              itemId : 'testConnectionButton',
              scale : 'small',
              iconCls : 'testConnection',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.ConnectionsTreePanel.testConnectionButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [],
              qdNotAuthorizedIconCls : 'testConnectionNAUTH'
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    // column's inner renderer that HTML-encodes values displayed in the tree
    me.getColumns()[ 0 ].innerRenderer = function ( value, metaData, record )
    {
      var result = Ext.String.htmlEncode( value );

      var warningIconHtml = record.get( 'warningIconHtml' );
      if ( !Ext.isEmpty( warningIconHtml ) )
      {
        result += ' ' + warningIconHtml;
      }

      return result;
    };
  },


  initComponent : function ()
  {
    var me = this;

    // important to set the store before callParent as it expects the store
    me.store = Ext.create( 'QDesk.store.ConnectionsTreeStore' );

    //
    // Initially we use an empty store URL and later we replace it with the real URL to work around
    // a bug in Chrome/IE where the initial tree panel's loading message does not display.
    //
    me.store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;

    me.callParent();

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    me._bindToolbarEventHandlers();
    me._bindPanelEventHandlers();
  },


  _bindToolbarEventHandlers : function ()
  {
    var me = this;

    // tree reload events
    var treeStore = me.getStore();

    treeStore.on( 'beforeload',
        function ( store, operation )
        {
          var toolBar = me.down( '#toolBar' );
          Ext.each( Ext.ComponentQuery.query( 'button', toolBar ), function ( button )
          {
            button.disable();
          } );
        } );

    treeStore.on( 'load',
        function ( store, node, records, success )
        {
          var toolBar = me.down( '#toolBar' );
          toolBar.down( '#refreshButton' ).enable();
          if ( success )
          {
            toolBar.down( '#collapseAllButton' ).enable();
          }
        } );

  },


  _bindPanelEventHandlers : function ()
  {
    var me = this;
    var treeView = me.getView();

    treeView.on( 'beforedrop', function ( node, data, overModel, dropPosition, dropFunction, eOpts )
    {
      var draggedRecord = data.records[ 0 ];
      var targetRecord = overModel;

      var draggedRecordParent = draggedRecord.parentNode;

      // if node is dragged to its existing parent, then we cancel the operation
      if ( draggedRecordParent.get( 'id' ) == targetRecord.get( 'id' ) )
        return false;

      if ( !me._checkNodeMove( draggedRecord, targetRecord ) )
      {
        return false;
      }

      me._onNodeDrag( draggedRecord, targetRecord );

      return 1;
    } );


    treeView.on( 'itemcontextmenu', function ( view, record, item, index, event )
    {
      var nodeId = record.get( 'id' );
      event.stopEvent();

      var nodeType = record.get( 'nodeType' );
      if ( nodeId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID || nodeType == 'folder' )
      {
        var folderCtxMenu = me._createFolderCtxMenu( record );
        folderCtxMenu.showAt( event.getXY() );
      }
      else if ( nodeType == 'connection' )
      {
        var connectionCtxMenu = me._createConnectionCtxMenu( record );
        connectionCtxMenu.showAt( event.getXY() );
      }
    } );


    treeView.on( 'itemdblclick', function ( view, record )
    {
      var nodeType = record.get( 'nodeType' );
      if ( nodeType == 'folder' )
      {
        //me.fireEvent( 'connectionsTreeFolderDblClicked', record );
      }
      else if ( nodeType == 'connection' )
      {
        me._openSchedulerView( record );
      }
    } );

    var treeStore = me.getStore();

    //treeStore.on( 'beforeload', function ( store, operation )
    //{
    //  QDesk.util.UxUtils.maskEl( me.getEl(),
    //      QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.maskLoadMsg' ) );
    //} );

    treeStore.on( 'load',
        function ( store, records, successful, operation, node )
        {
          //QDesk.util.UxUtils.unmaskEl( me.getEl() );

          if ( !successful )
          {
            // node (failed folder) collapsing must be deferred a bit, otherwise the collapse does not work
            Ext.defer( function ()
            {
              node.collapse();
            }, 200 );

            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );
  },


  /**
   * Checks if the specified record can be dragged to the specified target record.
   * @param draggedRecord the dragged folder/connection record.
   * @param targetRecord the target record (target folder).
   * @return {boolean} true if the dragged folder/connection can be moved, false otherwise.
   * @private
   */
  _checkNodeMove : function ( draggedRecord, targetRecord )
  {
    var canBeMoved = true;

    var targetFolderId = targetRecord.get( 'folderId' );
    var targetFolderName = targetRecord.get( 'name' );

    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      if ( QDesk.CurrentUser.hasAnyPermissions( [ QDesk.model.security.Permission.CONNECTION_FLD_MOVE ] ) )
      {
        var folderId = draggedRecord.get( 'folderId' );
        var folderName = draggedRecord.get( 'name' );

        Ext.each( targetRecord.childNodes, function ( node )
        {
          if ( node.get( 'nodeType' ) == 'folder' && node.get( 'name' ) == folderName )
          {
            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCannotBeMovedMsg',
                    folderName, targetFolderName ) );

            canBeMoved = false;
            return false;
          }
        } );
      }
      else
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.view.work.navigation.connections.ConnectionsTreePanel.notEnoughPermissionsFolderMoveMsg' ) );
        canBeMoved = false;
      }
    }
    else if ( nodeType == 'connection' )
    {
      if ( QDesk.CurrentUser.hasAnyPermissions( [ QDesk.model.security.Permission.CONNECTION_MOVE ] ) )
      {
        var connectionId = draggedRecord.get( 'connectionId' );
        var connectionName = draggedRecord.get( 'name' );

        Ext.each( targetRecord.childNodes, function ( node )
        {
          if ( node.get( 'nodeType' ) == 'connection' && node.get( 'name' ) == connectionName )
          {
            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCannotBeMovedMsg',
                    connectionName, targetFolderName ) );

            canBeMoved = false;
            return false;
          }
        } );
      }
      else
      {
        QDesk.util.UxUtils.showWarningDialog(
            QDesk.Msg.get(
                'QDesk.view.work.navigation.connections.ConnectionsTreePanel.notEnoughPermissionsConnectionMoveMsg' ) );

        canBeMoved = false;
      }
    }

    return canBeMoved;
  },


  /**
   * Invoked when a folder, or connection in the connections tree panel has been dragged.
   * @param draggedRecord the dragged folder/connection record.
   * @param targetRecord the target record (target folder).
   * @private
   */
  _onNodeDrag : function ( draggedRecord, targetRecord )
  {
    var targetFolderId = targetRecord.get( 'folderId' );
    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      Ext.create( 'QDesk.controller.command.connection.MoveConnectionFolder', {
        folderId : folderId,
        targetFolderId : targetFolderId
      } ).execute();
    }
    else if ( nodeType == 'connection' )
    {
      var connectionId = draggedRecord.get( 'connectionId' );
      Ext.create( 'QDesk.controller.command.connection.MoveSchedulerConnection', {
        connectionId : connectionId,
        targetFolderId : targetFolderId
      } ).execute();
    }
  },


  /**
   * Returns the ID of the parent folder of the specified connection.
   * @param connectionId a connection ID.
   * @private the parent folder ID, or null if the connection was not found.
   */
  _getParentFolderForConnection : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var store = treePanel.getStore();

    var connectionNode = store.getNodeById( connectionId );
    var parentFolderNode = connectionNode ? connectionNode.parentNode : null;
    return parentFolderNode ? parentFolderNode.data.id : null;
  },


  /**
   * Creates and returns the context menu for a folder node in the connections tree.
   * @param folderRecord a folder record (see QDesk.model.gui.ConnectionsTreeNode}.
   * @return the created context menu.
   */
  _createFolderCtxMenu : function ( folderRecord )
  {
    var me = this;

    var folderId = folderRecord.get( 'folderId' );
    var folderPath = folderRecord.getPath();
    var isRootFolder = folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCtxMenuAddFolderOrConnection' ),
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.CONNECTION_FLD_ADD,
            QDesk.model.security.Permission.CONNECTION_ADD
          ],
          menu : {
            items : [
              {
                text : QDesk.Msg.get(
                    'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCtxMenuAddFolder' ),
                iconCls : 'addFolder',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_FLD_ADD ],
                qdNotAuthorizedIconCls : 'addFolderNAUTH',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.connection.AddConnectionFolder', {
                    parentFolderId : folderId
                  } ).execute();
                }
              },
              {
                text : QDesk.Msg.get(
                    'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCtxMenuAddConnection' ),
                iconCls : 'quartzAddConnection',
                qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_ADD ],
                qdNotAuthorizedIconCls : 'quartzAddConnectionNAUTH',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.connection.AddSchedulerConnection', {
                    parentFolderId : folderId,
                    schedulerType : QDesk.model.scheduler.SchedulerType.QUARTZ
                  } ).execute();
                }
              }
            ]
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCtxMenuRename' ),
          iconCls : 'renameFolder',
          disabled : isRootFolder,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_FLD_UPDATE ],
          qdNotAuthorizedIconCls : 'renameFolderNAUTH',
          handler : function ()
          {
            if ( folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID )
            {
              // ignore, root node cannot be renamed
            }
            else
            {
              Ext.create( 'QDesk.controller.command.connection.EditConnectionFolder', {
                folderId : folderId,
                folderName : folderRecord.get( 'name' )
              } ).execute();
            }
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCtxMenuDelete' ),
          iconCls : 'deleteFolder',
          disabled : isRootFolder,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_FLD_DELETE ],
          qdNotAuthorizedIconCls : 'deleteFolderNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.DeleteConnectionFolder', {
              folderId : folderId
            } ).execute();
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.folderCtxMenuRefresh' ),
          iconCls : 'refreshFolder',
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshFolderNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
              folderId : folderId
            } ).execute();
          }
        },
        '-',
        // start schedulers
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuStartAll' ),
          iconCls : 'startAllSchedulers',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_START ],
          qdNotAuthorizedIconCls : 'startAllSchedulersNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.StartAllSchedulers', {
              folderId : folderId
            } ).execute();
          }
        },
        // pause schedulers
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuPauseAll' ),
          iconCls : 'pauseAllSchedulers',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_PAUSE ],
          qdNotAuthorizedIconCls : 'pauseAllSchedulersNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.PauseAllSchedulers', {
              folderId : folderId
            } ).execute();
          }
        },
        // stop schedulers
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuStopAll' ),
          iconCls : 'stopAllSchedulers',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_STOP ],
          qdNotAuthorizedIconCls : 'stopAllSchedulersNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.StopAllSchedulers', {
              folderId : folderId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Creates and returns the context menu for a connection node in the connections tree.
   * @param connectionRecord a folder record (see QDesk.model.gui.ConnectionsTreeNode}.
   * @return the created context menu.
   */
  _createConnectionCtxMenu : function ( connectionRecord )
  {
    var me = this;

    var connectionId = connectionRecord.get( 'connectionId' );
    var schedulerStatus = connectionRecord.get( 'schedulerStatus' );
    var schedulerType = connectionRecord.get( 'schedulerType' );

    var menuItemIconOpen = null;
    switch ( schedulerType )
    {
      case  QDesk.model.scheduler.SchedulerType.QUARTZ :
        menuItemIconOpen = 'quartzOpenConnection';
        menuItemIconOpenNAUTH = 'quartzOpenConnectionNAUTH';
        break;

      default:
        Ext.Error.raise( 'Unexpected scheduler type: ' + schedulerType );
    }

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // open connection
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuOpen' ),
          iconCls : menuItemIconOpen,
          disabled : schedulerStatus == QDesk.model.scheduler.SchedulerStatus.UNREACHABLE,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : menuItemIconOpenNAUTH,
          handler : function ()
          {
            me._openSchedulerView( connectionRecord )
          }
        },
        // edit connection
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuEdit' ),
          iconCls : 'editConnection',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_UPDATE ],
          qdNotAuthorizedIconCls : 'editConnectionNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.EditSchedulerConnection', {
              connectionId : connectionId,
              schedulerType : schedulerType
            } ).execute();
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuTest' ),
          iconCls : 'testConnection',
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'testConnectionNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnection', {
              connectionId : connectionId,
              schedulerType : schedulerType
            } ).execute();
          }
        },
        // delete connection
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuDelete' ),
          iconCls : 'deleteConnection',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.CONNECTION_DELETE ],
          qdNotAuthorizedIconCls : 'deleteConnectionNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.DeleteSchedulerConnection', {
              connectionId : connectionId
            } ).execute();
          }
        },
        '-',
        // start scheduler
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuStart' ),
          iconCls : 'startScheduler',
          disabled : schedulerStatus != QDesk.model.scheduler.SchedulerStatus.PAUSED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_START ],
          qdNotAuthorizedIconCls : 'startSchedulerNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.StartScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        },
        // pause scheduler
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuPause' ),
          iconCls : 'pauseScheduler',
          disabled : schedulerStatus != QDesk.model.scheduler.SchedulerStatus.STARTED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_PAUSE ],
          qdNotAuthorizedIconCls : 'pauseSchedulerNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.PauseScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        },
        // stop scheduler
        {
          text : QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.connectionCtxMenuStop' ),
          iconCls : 'stopScheduler',
          disabled : ( schedulerStatus != QDesk.model.scheduler.SchedulerStatus.STARTED &&
              schedulerStatus != QDesk.model.scheduler.SchedulerStatus.PAUSED ),
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_STOP ],
          qdNotAuthorizedIconCls : 'stopSchedulerNAUTH',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.StopScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        }
      ]
    } );
  },


  _openSchedulerView : function ( record )
  {
    var me = this;

    var schedulerStatus = record.get( 'schedulerStatus' );

    if ( ( schedulerStatus == QDesk.model.scheduler.SchedulerStatus.UNREACHABLE ) )
    {
      // connection unreachable
      QDesk.util.UxUtils.showInfoDialog(
          QDesk.Msg.get( 'QDesk.view.work.navigation.connections.ConnectionsTreePanel.schedulerUnreachableMsg' ) );
    }
    else
    {
      var connectionId = record.get( 'connectionId' );

      var schedulerInfo = {
        type : record.get( 'schedulerType' ),
        version : record.get( 'schedulerVersion' ),
        timeZone : record.get( 'schedulerTimeZone' )
      };

      var jvmAgentInfo = record.get( 'jvmAgentInfo' );
      var publicApiInfo = record.get( 'publicApiInfo' );

      // QuartzDesk Public API Library info
      if ( publicApiInfo )
      {
        Ext.apply( schedulerInfo, {
          publicApiInfo : publicApiInfo
        } );
      }

      var connectionNodePath = record.getPath();

      var schedulerInfoAccessor = Ext.create( 'QDesk.SchedulerInfoAccessor', {
        data : schedulerInfo
      } );

      var jvmAgentInfoAccessor = Ext.create( 'QDesk.JvmAgentInfoAccessor', {
        data : jvmAgentInfo
      } );

      /*
       * QDesk.ConnectionContext holding the connection-related data.
       */
      var connectionCtx = Ext.create( 'QDesk.ConnectionContext', {
        connectionId : connectionId,
        connectionNodePath : connectionNodePath,
        jvmAgentInfo : jvmAgentInfoAccessor,
        schedulerInfo : schedulerInfoAccessor
      } );

      if ( schedulerInfo.type === QDesk.model.scheduler.SchedulerType.QUARTZ )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
          connectionCtx : connectionCtx
        } ).execute();
      }
      else
      {
        Ext.Error.raise( 'Unsupported scheduler type: ' + schedulerType );
      }
    }
  }
} );
