/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.ViewError',

    'QDesk.model.common.Result',
    'QDesk.model.connection.ConnectionOperationResult',

    'QDesk.util.EnumUtils'
  ],

  alias : 'widget.ConnectionsOperationResultsGrid',

  cls : 'connectionsOperationResultsGrid',
  iconCls : 'connectionsOperationResultsGridIcon',

  header : false,

  scrollable : true,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.data )
      Ext.Error.raise( 'Required config parameter missing: qdesk.data' );

    var store = Ext.create( 'Ext.data.Store', {
      model : 'QDesk.model.connection.ConnectionOperationResult',
      data : config.qdesk.data,
      sorters : [
        {
          property : 'connectionPath',
          direction : 'ASC'
        }
      ]
    } );

    Ext.apply( config, {
      columns : [
        // connectionId
        {
          itemId : 'connectionIdColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.textConnectionIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.tooltipConnectionIdColumn' ),
          dataIndex : 'connectionId',
          sortable : true,
          groupable : false,
          hidden : true,
          flex : 0.5
        },

        // connectionName
        {
          itemId : 'connectionNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.textConnectionNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.tooltipConnectionNameColumn' ),
          dataIndex : 'connectionName',
          sortable : true,
          groupable : false,
          flex : 1.5,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        // connectionPath
        {
          itemId : 'connectionPathColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.textConnectionPathColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.tooltipConnectionPathColumn' ),
          dataIndex : 'connectionPath',
          sortable : true,
          groupable : false,
          flex : 2,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        // result
        {
          itemId : 'resultColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.textResultColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.tooltipResultColumn' ),
          dataIndex : 'result',
          sortable : true,
          groupable : false,
          flex : 0.5,
          /* Need custom renderer to localize result status values */
          renderer : function ( value, metaData, record )
          {
            return QDesk.util.EnumUtils.getResultName( value );
          }
        },

        // resultDetail
        {
          xtype : 'actioncolumn',
          itemId : 'resultDetailColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.textResultDetailColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.textResultDetailColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.tooltipResultDetailColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/error-16x16.png',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid.tooltipResultDetailColumnIcon' ),
              getClass : function ( value, metaData, record )
              {
                // hide the icon in case the detail is not available
                var resultDetail = record.get( 'resultDetail' );
                return resultDetail ? 'clickable' : 'x-hidden';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var resultDetail = record.get( 'resultDetail' );

                Ext.create( 'QDesk.controller.command.ViewError', {
                  errorText : resultDetail
                } ).execute();
              }
            }
          ]
        }
      ],

      store : store
    } );

    me.callParent( [ config ] );

    // rows have different CSS style that reflect the status
    me.getView().getRowClass = function ( record )
    {
      var result = record.get( 'result' );
      if ( result == QDesk.model.common.Result.SUCCESS )
        return 'resultSuccess';
      else if ( result == QDesk.model.common.Result.ERROR )
        return 'resultError';
      else return '';
    };
  }
} )
;