/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.work.scheduler.common.AgentConfigurationGrid'
  ],

  alias : 'widget.JvmAgentInfoForm',

  border : 0,

  cls : 'jvmAgentInfoForm',
  bodyCls : 'jvmAgentInfoFormBody',

  frame : false,

  scrollable : true,

  fieldDefaults : {
    labelWidth : 90
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'displayfield',
          itemId : 'detected',
          hidden : true,
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelDetected' )
        },
        {
          xtype : 'displayfield',
          itemId : 'nameAndVersion',
          hidden : true,
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelNameAndVersion' )
        },
        {
          xtype : 'displayfield',
          itemId : 'version',
          hidden : true,
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelVersion' )
        },
        {
          xtype : 'displayfield',
          itemId : 'edition',
          name : 'jvmAgentEdition',
          hidden : true,
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelEdition' )
        },
        {
          xtype : 'displayfield',
          itemId : 'locale',
          hidden : true,
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelLocale' ),
          renderer : function ( value )
          {
            if ( Ext.isEmpty( value ) )
              return value;

            var result = value.id + ' (' + value.displayCountry;

            if ( value.language )
              result += '/' + value.displayLanguage;

            if ( result.variant )
              result += '/' + value.displayVariant;

            result += ')';

            return result;
          }
        },
        {
          xtype : 'displayfield',
          itemId : 'timeZone',
          hidden : true,
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelTimeZone' ),
          renderer : function ( value )
          {
            return Ext.isEmpty( value ) ? value : value.displayName;
          }
        },

        {
          xtype : 'textareafield',
          itemId : 'classLoaderInfo',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldLabelClassLoaderInfo' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.fieldTooltipClassLoaderInfo' ),
          labelAlign : 'top',
          hidden : true,
          height : 150,
          readOnly : true,
          readOnlyCls : '', /* prevent lower opacity of read-only fields */
          style : {
            marginBottom : '15px'
          }
        },

        {
          xtype : 'AgentConfigurationGrid',
          itemId : 'configuration',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.gridTitleConfiguration' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm.gridTooltipConfiguration' ),
          hidden : true,
          maxHeight : 300
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );
