/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.common.jvm.ThreadInfoGeneralForm',
    'QDesk.view.work.scheduler.common.jvm.ThreadInfoLockForm',
    'QDesk.view.work.scheduler.common.jvm.ThreadInfoLockedMonitorsPanel',
    'QDesk.view.work.scheduler.common.jvm.ThreadInfoLockedSynchronizersPanel',
    'QDesk.view.work.scheduler.common.jvm.ThreadInfoStackTraceForm'
  ],

  alias : 'widget.ThreadInfoPanel',

  /* The info panel should not be scrollable, but individual nested tabs should be! */
  scrollable : false,

  layout : 'fit',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    // if ( !config.qdesk )
    //   Ext.Error.raise( 'Required config parameter missing: qdesk' );
    //
    // if ( !config.qdesk.connectionCtx )
    //   Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var tabs = [];

    tabs.push(
        {
          xtype : 'panel',
          itemId : 'generalTab',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.titleGeneralTab' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.tooltipGeneralTab' ),
          iconCls : 'threadInfoGeneralTabIcon',
          layout : 'fit',
          items : [
            {
              xtype : 'ThreadInfoGeneralForm'
            }
          ]
          // qdesk : config.qdesk
        } );

    tabs.push(
        {
          xtype : 'panel',
          itemId : 'stackTraceTab',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.titleStackTraceTab' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.tooltipStackTraceTab' ),
          iconCls : 'threadInfoStackTraceTabIcon',
          layout : 'fit',
          items : [
            {
              xtype : 'ThreadInfoStackTraceForm'
            }
          ]
          // qdesk : config.qdesk
        } );

    tabs.push(
        {
          xtype : 'panel',
          itemId : 'lockTab',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.titleLockTab' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.tooltipLockTab' ),
          iconCls : 'threadInfoLockTabIcon',
          layout : 'fit',
          items : [
            {
              xtype : 'ThreadInfoLockForm'
            }
          ]
          // qdesk : config.qdesk
        } );

    tabs.push(
        {
          xtype : 'panel',
          itemId : 'lockedMonitorsTab',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.titleLockedMonitorsTab' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.tooltipLockedMonitorsTab' ),
          iconCls : 'threadInfoLockedMonitorsTabIcon',
          layout : 'fit',
          items : [
            {
              xtype : 'ThreadInfoLockedMonitorsPanel'
            }
          ]
          // qdesk : config.qdesk
        } );

    tabs.push(
        {
          xtype : 'panel',
          itemId : 'lockedSynchronizersTab',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.titleLockedSynchronizersTab' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.tooltipLockedSynchronizersTab' ),
          iconCls : 'threadInfoLockedSynchronizersTabIcon',
          layout : 'fit',
          items : [
            {
              xtype : 'ThreadInfoLockedSynchronizersPanel'
            }
          ]
          // qdesk : config.qdesk
        } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'tabpanel',
          defaults : {
            border : false,
            closable : false,
            frame : false,
            scrollable : false
          },
          items : tabs
        }
      ],

      buttons : [
        {
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.jvm.ThreadInfoPanel.tooltipRefreshButton' ),
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        { xtype : 'component', flex : 1 },
        {
          itemId : 'closeButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );
