/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.message.channel.MessageChannel',
    'QDesk.model.message.MessageCreatorType',
    'QDesk.model.message.MessageStatus',
    'QDesk.model.security.Permission',

    'QDesk.store.message.MessageStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.message.MessageQueueUtils',

    'QDesk.view.widget.WMaxTodayDateField',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.MessageQueueGrid',

  cls : 'messageQueueGrid',

  //scrollable : true,

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.message.MessageQueueUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var connectionCtx = config.qdesk.connectionCtx;
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // set the fetch period values
    var fetchPeriod = QDesk.util.CommonUtils.getDefaultFetchPeriod(
        schedulerInfo.getSchedulerTimeZone(),
        QDesk.ApplicationConfig.get( config.qdesk.gridId + '.fetchPeriodLength' ) );

    // set the max fetch size value
    var maxFetchSize = QDesk.ApplicationConfig.get( config.qdesk.gridId + '.maxFetchSize' );

    var store = Ext.create( 'QDesk.store.message.MessageStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'WDateTimePeriod',
          itemId : 'fetchPeriod',
          allowBlank : false,
          disabled : true,
          value : fetchPeriod,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.message.MessageQueueGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },

        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              //          [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 70,
          width : 140,
          editable : false,
          disabled : true,
          value : maxFetchSize,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.message.MessageQueueGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : [
        // id
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.4
        },

        // schedulerObjectName
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : false,
          groupable : false,
          flex : 1.1
        },

        // channel profile
        {
          itemId : 'channelProfileColumn',
          dataIndex : 'channelProfileName',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textChannelProfileColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipChannelProfileColumn' ),
          sortable : false,
          groupable : false,
          flex : 1.3,
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;

            var messageChannel = record.get( 'channelProfileChannel' );

            if ( value )
            {
              tooltipText = QDesk.util.EnumUtils.getMessageChannelName( messageChannel );

              if ( messageChannel == QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT )
                iconClass = 'messageChannelCollaborationHipChat';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK )
                iconClass = 'messageChannelCollaborationSlack';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.EMAIL )
                iconClass = 'messageChannelEmail';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_AIM )
                iconClass = 'messageChannelImAim';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_GTALK )
                iconClass = 'messageChannelImGtalk';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_ICQ )
                iconClass = 'messageChannelImIcq';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_XMPP )
                iconClass = 'messageChannelImXmpp';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_YAHOO )
                iconClass = 'messageChannelImYahoo';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.INCIDENT_MGMT_JIRA )
                iconClass = 'messageChannelIncidentMgmtJira';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.INCIDENT_MGMT_OPSGENIE )
                iconClass = 'messageChannelIncidentMgmtOpsGenie';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL )
                iconClass = 'messageChannelSmsClickatellCentral';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM )
                iconClass = 'messageChannelSmsClickatellPlatform';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK )
                iconClass = 'messageChannelSmsClockwork';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_NEXMO )
                iconClass = 'messageChannelSmsNexmo';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL )
                iconClass = 'messageChannelSmsSmsGlobal';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_TWILIO )
                iconClass = 'messageChannelSmsTwilio';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP )
                iconClass = 'messageChannelSnmpV1Trap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP )
                iconClass = 'messageChannelSnmpV2cTrap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP )
                iconClass = 'messageChannelSnmpV3Trap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.WEB_SERVICE )
                iconClass = 'messageChannelWebService';
            }
            else
            {
              value = QDesk.Msg.get( 'QDesk.notAvailable' );

              iconClass = 'messageChannelUnknown';
              var messageChannelProfileId = record.get( 'channelProfileId' );
              tooltipText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipChannelProfileNotFound',
                      messageChannelProfileId );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>' +
                '<span style="padding-left: 5px;">' + Ext.String.htmlEncode( value ) + '</span>';
          }
        },

        // status
        {
          itemId : 'statusColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textStatusColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textStatusColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipStatusColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;
            var status = record.get( 'status' );

            if ( status == QDesk.model.message.MessageStatus.PENDING )
              iconClass = 'messageStatusPending';
            else if ( status == QDesk.model.message.MessageStatus.PROCESSED )
              iconClass = 'messageStatusProcessed';
            else if ( status == QDesk.model.message.MessageStatus.FAILED_TEMP )
              iconClass = 'messageStatusFailedTemp';
            else if ( status == QDesk.model.message.MessageStatus.FAILED_PERM )
              iconClass = 'messageStatusFailedPerm';
            else
              Ext.Error.raise( 'Unexpected message status value: ' + value );

            tooltipText = QDesk.util.EnumUtils.getMessageStatusName( status );

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>';
          }
        },

        // correlationId
        {
          itemId : 'correlationIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textCorrelationIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipCorrelationIdColumn' ),
          dataIndex : 'correlationId',
          sortable : false,
          groupable : false,
          flex : 0.7
        },

        // creator
        {
          itemId : 'creatorColumn',
          dataIndex : 'creatorName',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textCreatorColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipCreatorColumn' ),
          sortable : false,
          groupable : false,
          flex : 1,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass;

            var creatorType = record.get( 'creatorType' );
            var tooltipText = QDesk.util.EnumUtils.getMessageCreatorTypeName( creatorType );

            if ( creatorType == QDesk.model.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE )
            {
              iconClass = 'messageCreatorTypeQuartzExecNotifRule';
            }
            else
            {
              iconClass = 'messageCreatorTypeUnknown';
            }

            // creator name not available
            if ( !value )
            {
              var creatorId = record.get( 'creatorId' );
              value = QDesk.Msg.get( 'QDesk.notAvailable' );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>' +
                '<span style="padding-left: 5px;">' + value + '</span>';
          }
        },

        // createdAt
        {
          itemId : 'createdAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textCreatedAtColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipCreatedAtColumn' ),
          dataIndex : 'createdAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        // createdBy
        {
          itemId : 'createdByColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textCreatedByColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipCreatedByColumn' ),
          dataIndex : 'createdBy',
          sortable : false,
          groupable : false,
          flex : 1
        },

        // postponedTill
        {
          itemId : 'postponedTillColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textPostponedTillColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipPostponedTillColumn' ),
          dataIndex : 'postponedTill',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        // processingStartedAt
        {
          itemId : 'processingStartedAtColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textProcessingStartedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipProcessingStartedAtColumn' ),
          dataIndex : 'processingStartedAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        // processingFinishedAt
        {
          itemId : 'processingFinishedAtColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textProcessingFinishedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipProcessingFinishedAtColumn' ),
          dataIndex : 'processingFinishedAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        // processedBy
        {
          itemId : 'processedByColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textProcessedByColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipProcessedByColumn' ),
          dataIndex : 'processedBy',
          sortable : false,
          groupable : false,
          flex : 1
        },

        // errorCount
        {
          xtype : 'numbercolumn',
          itemId : 'errorCountColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textErrorCountColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipErrorCountColumn' ),
          dataIndex : 'errorCount',
          sortable : false,
          groupable : false,
          flex : 0.7,
          renderer : function ( value, metaData, record )
          {
            var maxErrorCount = record.get( 'maxErrorCount' );
            if ( !maxErrorCount )
              maxErrorCount = '?';

            return value + ' / ' + maxErrorCount;
          }
        },

        // lastErrorAt
        {
          itemId : 'lastErrorAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textLastErrorAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipLastErrorAtColumn' ),
          dataIndex : 'lastErrorAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        // lastErrorDesc
        {
          xtype : 'actioncolumn',
          itemId : 'lastErrorDescColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textLastErrorDescColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.textLastErrorDescColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipLastErrorDescColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          items : [
            {
              icon : 'images/icon/scheduler/message-error-16x16.png',
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var lastErrorDesc = record.get( 'lastErrorDesc' );
                this.fireEvent( 'requestViewError', lastErrorDesc );
              },
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.message.MessageQueueGrid.tooltipLastErrorDescColumn' ),
              getClass : function ( value, metaData, record )
              {
                // hide the icon in case the log data is not available
                var lastErrorDesc = record.get( 'lastErrorDesc' );
                return lastErrorDesc ? 'clickable' : 'x-hidden';
              }
            }
          ]
        }
      ],

      store : store,

      qdesk : {
        connectionCtx : config.qdesk.connectionCtx,
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );

    // rows have different CSS style that reflect the status
    me.getView().getRowClass = function ( record )
    {
      var status = record.get( 'status' );

      if ( status == QDesk.model.message.MessageStatus.PENDING )
        return 'statusPending';
      else if ( status == QDesk.model.message.MessageStatus.PROCESSED )
        return 'statusProcessed';
      else if ( status == QDesk.model.message.MessageStatus.FAILED_TEMP )
        return 'statusFailedTemp';
      else if ( status == QDesk.model.message.MessageStatus.FAILED_PERM )
        return 'statusFailedPerm';
    };
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var extra = {};

    var fetchPeriod = me.down( '#fetchPeriod' ).getValue();
    if ( fetchPeriod != null )
    {
      var fetchPeriodLength = QDesk.util.CommonUtils.getDiffInDays( fetchPeriod.from, fetchPeriod.to );
      extra[ me.qdesk.gridId + '.fetchPeriodLength' ] = fetchPeriodLength;
    }

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();
    extra[ me.qdesk.gridId + '.maxFetchSize' ] = maxFetchSize;

    return extra;
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    // A little hack to work around a problem with the datetime period feidl being marked
    // as invalid when the grid is shown for the first time. What is causing the issue???
    me.on( 'afterrender', function ()
    {
      me.down( '#fetchPeriod' ).isValid();
    } );

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var emptyText = null;

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          if ( me.filtersValid() )
          {
            emptyText =
                QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.emptyTextNoMatchingData' );
          }
          else
          {
            emptyText =
                QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.emptyTextInvalidFilters' );
          }
        }
        else
        {
          // records not empty => show the records, not the empty text
        }
      }
      else
      {
        emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueGrid.emptyTextErrorLoadingData' );
      }

      if ( !Ext.isEmpty( emptyText ) )
      {
        me.empty( emptyText );
      }
    } );
  },


  /**
   * Returns true if the filters in the grid's toolbar are valid, false otherwise.
   * @return true if filters are valid, false otherwise.
   */
  filtersValid : function ()
  {
    var me = this;

    if ( !me.rendered )
      return true;

    var fetchPeriodPicker = me.down( '#fetchPeriod' );
    var valid = fetchPeriodPicker.isValid();

    // here we can add other filters

    return valid;
  },


  /**
   * Empties the grid and shows the specified emptyMsg (if not null).
   * @param emptyText an empty message to show.
   */
  empty : function ( emptyText )
  {
    var me = this;

    var store = me.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
    store.getProxy().extraParams = {};
    store.loadRecords( [] );

    if ( !Ext.isEmpty( emptyText ) )
    {
      var view = me.getView();
      view.emptyText = emptyText;
      view.refresh();
    }
  }
} );
