/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.message.MessageQueueWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.common.message.MessageQueueGrid'
  ],

  alias : 'widget.MessageQueueWindow',

  cls : 'messageQueueWindow',
  iconCls : 'windowIcon',

  scrollable : true,

  modal : true,
  resizable : true,
  maximizable : true,

  width : 800,
  height : 500,

  minWidth : 800,
  minHeight : 500,

  layout : 'fit',

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.message.MessageQueueWindow.title' ),
      items : [
        {
          xtype : 'MessageQueueGrid',
          itemId : 'messageQueueGrid',
          qdesk : {
            connectionCtx : config.qdesk.connectionCtx,
            gridId : 'messageQueueGrid'
          }
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'closeButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } );
  }
} );