/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage',

    'QDesk.model.common.ExpressionLanguage',
    'QDesk.model.common.ExpressionType',
    'QDesk.model.common.Weekday',

    'QDesk.model.message.channel.MessageChannel',

    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.scheduler.ExecNotificationRuleConditionType',
    'QDesk.model.scheduler.NotificationSeverity',

    'QDesk.model.security.Permission',

    'QDesk.store.enums.ExecStatusStore',
    'QDesk.store.enums.ExpressionLanguageStore',
    'QDesk.store.enums.NotificationSeverityStore',

    'QDesk.store.enums.scheduler.ExecNotificationRuleConditionTypeStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.work.scheduler.common.notif.exec.ExecNotificationRuleMessageMacrosWindow',

    'QDesk.view.widget.WMessageChannelProfileCombo',
    'QDesk.view.widget.expression.WExpressionEditor'
  ],

  alias : 'widget.EditExecNotificationRuleForm',

  defaultButton : 'actionButton',

  fieldDefaults : {
    labelWidth : 80
  },

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /*
     * Mode of the form (see EditExecNotificationRuleFormMode).
     */
    mode : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    var connectionCtx = config.qdesk.connectionCtx;
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    var items = [];

    items.push(
        {
          xtype : 'hiddenfield',
          itemId : 'notificationRuleId',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'schedulerObjectName',
          name : 'schedulerObjectName'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'messageChannelProfileChannel',
          name : 'messageChannelProfileChannel'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'createdAt',
          name : 'createdAt'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'updatedAt',
          name : 'updatedAt'
        }
    );

    items.push(
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetLabelJob' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetTooltipJob' ),
          padding : '0 10 0 10',
          hidden : true,
          defaults : {
            labelWidth : 55,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'jobName',
              name : 'jobName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelJobName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtJobName' ),
              allowBlank : true,
              readOnly : true,
              maxLength : 200,
              listeners : {
                /**
                 * Makes the Job fieldset visible if the job name is available.
                 */
                change : function ( value )
                {
                  var me = this;
                  if ( !Ext.isEmpty( value ) )
                  {
                    me.up( 'fieldset' ).setVisible( true );
                  }
                }
              }
            },
            {
              xtype : 'textfield',
              itemId : 'jobGroupName',
              name : 'jobGroupName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelJobGroupName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtJobGroupName' ),
              allowBlank : true,
              readOnly : true,
              maxLength : 200,
              listeners : {
                /**
                 * Makes the Job fieldset visible if the job group name is available.
                 */
                change : function ( value )
                {
                  var me = this;
                  if ( !Ext.isEmpty( value ) )
                  {
                    me.up( 'fieldset' ).setVisible( true );
                  }
                }
              }
            }
          ]
        }
    );

    items.push(
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetLabelTrigger' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetTooltipTrigger' ),
          padding : '0 10 0 10',
          hidden : true,
          defaults : {
            labelWidth : 55,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'triggerName',
              name : 'triggerName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelTriggerName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtTriggerName' ),
              allowBlank : true,
              readOnly : true,
              maxLength : 200,
              listeners : {
                /**
                 * Makes the Trigger fieldset visible if the trigger name is available.
                 */
                change : function ( value )
                {
                  var me = this;
                  if ( !Ext.isEmpty( value ) )
                  {
                    me.up( 'fieldset' ).setVisible( true );
                  }
                }
              }
            },
            {
              xtype : 'textfield',
              itemId : 'triggerGroupName',
              name : 'triggerGroupName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelTriggerGroupName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtTriggerGroupName' ),
              allowBlank : true,
              readOnly : true,
              maxLength : 200,
              listeners : {
                /**
                 * Makes the Trigger fieldset visible if the trigger group name is available.
                 */
                change : function ( value )
                {
                  var me = this;
                  if ( !Ext.isEmpty( value ) )
                  {
                    me.up( 'fieldset' ).setVisible( true );
                  }
                }
              }
            }
          ]
        }
    );

    /* enabledOnWeekdaysPanel */
    var enabledOnWeekdaysPanel =
        {
          xtype : 'panel',
          border : false,
          height : 50,
          layout : {
            type : 'hbox',
            pack : 'start',
            align : 'stretch'
          },
          items : [
            {
              xtype : 'checkboxgroup',
              style : {
                paddingLeft : '10px',
                backgroundColor : '#E6F7E6'
              },
              flex : 2,

              vertical : true,
              columns : 3,
              items : [
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.MONDAY ),
                  name : 'enabledOnMonday',
                  itemId : 'enabledOnMonday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.MONDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.TUESDAY ),
                  name : 'enabledOnTuesday',
                  itemId : 'enabledOnTuesday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.TUESDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.WEDNESDAY ),
                  name : 'enabledOnWednesday',
                  itemId : 'enabledOnWednesday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.WEDNESDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.THURSDAY ),
                  name : 'enabledOnThursday',
                  itemId : 'enabledOnThursday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.THURSDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.FRIDAY ),
                  name : 'enabledOnFriday',
                  itemId : 'enabledOnFriday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.FRIDAY
                  }
                }
              ]
            },
            {
              xtype : 'checkboxgroup',
              style : {
                paddingLeft : '10px',
                backgroundColor : '#F7E6E6'
              },
              flex : 1,
              vertical : true,
              columns : 1,
              items : [
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.SATURDAY ),
                  name : 'enabledOnSaturday',
                  itemId : 'enabledOnSaturday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.SATURDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.SUNDAY ),
                  name : 'enabledOnSunday',
                  itemId : 'enabledOnSunday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.SUNDAY
                  }
                }
              ]
            }
          ]
        };

    items.push(
        {
          xtype : 'textfield',
          itemId : 'name',
          name : 'name',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelName' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipName' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtName' ),
          value : QDesk.Msg.get( 'QDesk.unnamed' ),
          allowBlank : false,
          maxLength : 128
        },
        {
          xtype : 'textareafield',
          itemId : 'description',
          name : 'description',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelDescription' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipDescription' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtDescription' ),
          labelAlign : 'top',
          allowBlank : true,
          maxLength : 1024
        },
        {
          xtype : 'iconcombo',
          itemId : 'severity',
          name : 'severity',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelSeverity' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipSeverity' ),
          allowBlank : false,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.NotificationSeverityStore',
          valueField : 'value',
          displayField : 'valueLabel',
          iconClsField : 'value'
        },

        {
          xtype : 'checkboxfield',
          itemId : 'enabled',
          name : 'enabled',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelEnabled' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipEnabled' ),
          allowBlank : true
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetLabelEnabledOnWeekdays' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetTooltipEnabledOnWeekdays' ),
          itemId : 'enabledOnWeekdaysFieldSet',
          defaults : {
            labelWidth : 100,
            anchor : '100%'
          },
          padding : '0 10px 7px 10px',
          items : [
            enabledOnWeekdaysPanel
          ]
        }
    );

    /* enabledFromToPanel */
    items.push(
        {
          xtype : 'panel',
          border : false,
          layout : 'column',
          style : {
            marginBottom : '5px'
          },
          items : [
            {
              xtype : 'timefield',
              itemId : 'enabledFromTime',
              name : 'enabledFromTime',
              labelWidth : 95,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelEnabledFromTime' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipEnabledFromTime' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtEnabledFromTime' ),
              formatText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldFormatTxtEnabledFromTime' ),
              format : QDesk.util.constant.DateFormat.dayTimeFormat,
              increment : 30
            },
            {
              xtype : 'timefield',
              itemId : 'enabledToTime',
              name : 'enabledToTime',
              labelWidth : 82,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelEnabledToTime' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipEnabledToTime' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtEnabledToTime' ),
              formatText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldFormatTxtEnabledToTime' ),
              format : QDesk.util.constant.DateFormat.dayTimeFormat,
              increment : 30,
              margin : '0 0 0 15'
            }
          ]
        }
    );

    items.push(
        {
          xtype : 'tabpanel',
          bodyPadding : 10,
          items : [
            {
              xtype : 'panel',
              border : false,
              iconCls : 'execNotificationConditionTabIcon',
              title : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabLabelTriggerCondition' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabTooltipTriggerCondition' ),
              layout : 'anchor',
              //height: 265,
              defaults : {
                labelWidth : 120,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'conditionType',
                  name : 'conditionType',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelConditionType' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionType' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.scheduler.ExecNotificationRuleConditionTypeStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },

                /* Condition exec status */
                {
                  xtype : 'combo',
                  itemId : 'conditionExecStatus',
                  name : 'conditionExecStatus',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelConditionExecStatus' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionExecStatus' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExecStatusStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },

                /* Condition max duration */
                {
                  xtype : 'numberfield',
                  itemId : 'conditionMaxDuration',
                  name : 'conditionMaxDuration',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMaxDuration' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMaxDuration' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMaxDuration' ),
                  allowBlank : false,
                  minValue : 1
                },

                /* Condition expression */
                {
                  xtype : 'expression.WExpressionEditor',
                  itemId : 'conditionExpression',
                  name : 'conditionExpression',
                  allowBlank : false,
                  value : {
                    language : QDesk.model.common.ExpressionLanguage.JAVASCRIPT,
                    text : null
                  },
                  expressionType : QDesk.model.common.ExpressionType.QUARTZ_EXEC_NOTIF_RULE_CONDITION,
                  textEmptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtConditionExpressionText' ),
                  textTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipConditionExpressionText' ),
                  objectsStore : me._createObjectsTreeStore( schedulerInfo.getSchedulerType() ),
                  dataTypesStore : me._createDataTypesTreeStore( schedulerInfo.getSchedulerType() ),
                  extraValidateParamFn : function ()
                  {
                    return {
                      connectionId : connectionCtx.getConnectionId()
                    };
                  }
                }
              ]
            },
            {
              xtype : 'panel',
              itemId : 'messageTab',
              border : false,
              iconCls : 'execNotificationMessageTabIcon',
              title : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabLabelMessage' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.tabTooltipMessage' ),
              layout : 'anchor',
              defaults : {
                labelWidth : 105,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'panel',
                  itemId : 'messageChannelProfilePanel',
                  border : false,
                  layout : 'column',
                  style : {
                    marginBottom : '5px'
                  },
                  defaults : {
                    labelWidth : 105
                  },
                  items : [
                    {
                      xtype : 'WMessageChannelProfileCombo',
                      itemId : 'messageChannelProfileCombo',
                      name : 'messageChannelProfileId',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageChannelProfile' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageChannelProfile' ),
                      allowBlank : false,
                      columnWidth : 1
                    },
                    {
                      xtype : 'button',
                      itemId : 'messageChannelProfilesButton',
                      style : {
                        marginLeft : '3px'
                      },
                      tooltip : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipMessageChannelProfiles' ),
                      iconCls : 'messageChannelProfiles'
                    }
                  ]
                },

                {
                  xtype : 'textfield',
                  itemId : 'messageToAddr',
                  name : 'messageToAddr',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageToAddr' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageToAddr' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMessageToAddr' ),
                  allowBlank : false,
                  maxLength : 256,
                  hidden : true
                },

                {
                  xtype : 'panel',
                  itemId : 'messageSubjectTemplatePanel',
                  border : false,
                  hidden : true, /* subject panel is normally hidden and it is made visible only when EMAIL/WEB_SERVICE channel is selected */
                  layout : 'column',
                  defaults : {
                    labelWidth : 105
                  },
                  items : [
                    {
                      xtype : 'textfield',
                      itemId : 'messageSubjectTemplate',
                      name : 'messageSubjectTemplate',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageSubjectTemplate' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageSubjectTemplate' ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMessageSubjectTemplate' ),
                      allowBlank : false,
                      maxLength : 256,
                      columnWidth : 1
                    },
                    {
                      xtype : 'button',
                      itemId : 'messageSubjectTemplateMacros',
                      style : {
                        marginLeft : '3px'
                      },
                      tooltip : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipMessageSubjectTemplateMacros' ),
                      iconCls : 'messageSubjectTemplateMacros'
                    }
                  ]
                },

                {
                  xtype : 'panel',
                  itemId : 'messageBodyTemplatePanel',
                  border : false,
                  layout : 'column',
                  style : {
                    marginBottom : '5px'
                  },
                  items : [
                    {
                      xtype : 'textareafield',
                      itemId : 'messageBodyTemplate',
                      name : 'messageBodyTemplate',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldLabelMessageBodyTemplate' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldTooltipMessageBodyTemplate' ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldEmptyTxtMessageBodyTemplate' ),
                      labelAlign : 'top',
                      allowBlank : false,
                      columnWidth : 1,
                      height : 200
                    },
                    {
                      xtype : 'button',
                      itemId : 'messageBodyTemplateMacros',
                      style : {
                        marginLeft : '3px',
                        marginTop : '22px'
                      },
                      tooltip : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonTooltipMessageBodyTemplateMacros' ),
                      iconCls : 'messageBodyTemplateMacros'
                    }
                  ]
                },


                {
                  xtype : 'fieldset',
                  title : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetLabelMessageAttachments' ),
                  tooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.fieldsetTooltipMessageAttachments' ),
                  itemId : 'messageAttachmentsFieldSet',
                  defaults : {
                    labelWidth : 100,
                    anchor : '100%'
                  },
                  items : [
                    {
                      xtype : 'checkboxgroup',
                      //vertical : true,
                      columns : 4,
                      items : [
                        {
                          name : 'messageAttachLog',
                          itemId : 'messageAttachLog',
                          boxLabel : QDesk.Msg.get(
                              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachLog' )
                        },
                        {
                          name : 'messageAttachError',
                          itemId : 'messageAttachError',
                          boxLabel : QDesk.Msg.get(
                              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachError' )
                        },
                        {
                          name : 'messageAttachResult',
                          itemId : 'messageAttachResult',
                          boxLabel : QDesk.Msg.get(
                              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachResult' )
                        },
                        {
                          name : 'messageAttachUserData',
                          itemId : 'messageAttachUserData',
                          boxLabel : QDesk.Msg.get(
                              'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.boxLabelMessageAttachUserData' )
                        }
                      ]
                    }
                  ]
                },

                {
                  xtype : 'panel',
                  border : false,
                  height : 25,
                  layout : {
                    type : 'hbox',
                    pack : 'start',
                    align : 'stretch'
                  },
                  defaults : {
                    labelWidth : 80,
                    anchor : '100%'
                  },
                  items : [
                    {
                      xtype : 'button',
                      itemId : 'sendTestMessage',
                      text : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.buttonLabelSendTestMessage' ),
                      anchor : '20%'
                    },
                    {
                      xtype : 'displayfield',
                      value : QDesk.Msg.get(
                          'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm.noteSendTestMessage' ),
                      style : {
                        marginLeft : '5px'
                      }
                    }
                  ]
                }
              ]
            }
          ]
        }
    );

    Ext.apply( config, {
      items : items,

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true,
          qdRequiresAnyPermissions : me._getRequiredPermissions( config.qdesk.mode )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    /* Defaults */
    me.down( '#severity' ).setValue( QDesk.model.scheduler.NotificationSeverity.INFO );
    me.down( '#enabled' ).setValue( true );
    me.down( '#enabledOnMonday' ).setValue( true );
    me.down( '#enabledOnTuesday' ).setValue( true );
    me.down( '#enabledOnWednesday' ).setValue( true );
    me.down( '#enabledOnThursday' ).setValue( true );
    me.down( '#enabledOnFriday' ).setValue( true );
    me.down( '#enabledOnSaturday' ).setValue( true );
    me.down( '#enabledOnSunday' ).setValue( true );

    me.down( '#conditionType' ).setValue( QDesk.model.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS );
    me.down( '#conditionExecStatus' ).setValue( QDesk.model.scheduler.ExecStatus.SUCCESS );
  },


  /**
   * Invoked from the load-success handler to set special form field values (e.g. the enabledOnWeekdays
   * checkboxes etc.).
   * @param jsonData JSON data.
   */
  loadSpecialFields : function ( jsonData )
  {
    var me = this;

    // set enabledOnWeekdays checkboxes
    Ext.each( Ext.ComponentQuery.query( '#enabledOnWeekdaysFieldSet checkbox', me ),
        function ( checkbox )
        {
          checkbox.setValue( Ext.Array.contains( jsonData.enabledOnWeekday, checkbox.qdesk.value ) );
        } );

    // set enabledFromTime
    if ( jsonData.enabledFromTime )
    {
      var enabledFromDate = QDesk.util.FormatUtils.parseDayTime( jsonData.enabledFromTime );
      me.down( '#enabledFromTime' ).setValue( enabledFromDate );
    }

    // set enabledToTime fields
    if ( jsonData.enabledToTime )
    {
      var enabledToDate = QDesk.util.FormatUtils.parseDayTime( jsonData.enabledToTime );
      me.down( '#enabledToTime' ).setValue( enabledToDate );
    }
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // notification condition type change
    me.down( '#conditionType' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#conditionExecStatus' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS );
        field.setDisabled( newValue != QDesk.model.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS );
        field.validate();
      } );

      Ext.each( me.down( '#conditionMaxDuration' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.ExecNotificationRuleConditionType.MAX_DURATION );
        field.setDisabled( newValue != QDesk.model.scheduler.ExecNotificationRuleConditionType.MAX_DURATION );
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpression' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.ExecNotificationRuleConditionType.EXPRESSION );
        field.setDisabled( newValue != QDesk.model.scheduler.ExecNotificationRuleConditionType.EXPRESSION );
        field.validate();
      } );
    } );

    // message channel profile change
    me.down( '#messageChannelProfileCombo' ).on( 'change', function ( combo, newValue, oldValue )
    {
      var messageChannel = combo.getMessageChannel();

      me.down( '#messageChannelProfileChannel' ).setValue( messageChannel );

      // show/hide message recipients
      if ( QDesk.util.MessageChannelProfileUtils.messageChannelSupportsRecipients( messageChannel ) )
      {
        // required recipient(s)
        var field = me.down( '#messageToAddr' );
        field.setVisible( true );
        field.setDisabled( false );
        field.validate();
      }
      else
      {
        var field = me.down( '#messageToAddr' );
        field.setVisible( false );
        field.setDisabled( true );
        field.validate();
      }

      // show/hide message subjects
      if ( QDesk.util.MessageChannelProfileUtils.messageChannelSupportsSubject( messageChannel ) )
      {
        // required message subject
        var panel = me.down( '#messageSubjectTemplatePanel' );
        panel.setVisible( true );

        var field = panel.down( '#messageSubjectTemplate' );
        field.setDisabled( false );
        field.validate();
      }
      else
      {
        var panel = me.down( '#messageSubjectTemplatePanel' );
        panel.setVisible( false );

        var field = panel.down( '#messageSubjectTemplate' );
        field.setDisabled( true );
        field.validate();
      }

      // show/hide message attachments
      me.down( '#messageAttachmentsFieldSet' ).setVisible(
          QDesk.util.MessageChannelProfileUtils.messageChannelSupportsAttachments( messageChannel ) );

      // revalidates the form
      me.getForm().isValid();
    }, me );

    // message subject macros
    me.down( '#messageSubjectTemplateMacros' ).on( 'click',
        function ( button )
        {
          var window = Ext.widget( 'ExecNotificationRuleMessageMacrosWindow', {
            qdesk : {
              schedulerType : schedulerInfo.getSchedulerType()
            }
          } );

          window.on( 'macroSelected',
              function ( value )
              {
                var messageSubjectField = me.down( '#messageSubjectTemplate' );
                QDesk.util.FormUtils.insertTextAtCursor( messageSubjectField, value );
              } );

          window.show();
        }, me );


    // message body macros
    me.down( '#messageBodyTemplateMacros' ).on( 'click',
        function ( button )
        {
          var window = Ext.widget( 'ExecNotificationRuleMessageMacrosWindow', {
            qdesk : {
              schedulerType : schedulerInfo.getSchedulerType()
            }
          } );

          window.on( 'macroSelected',
              function ( value )
              {
                var messageBodyTemplate = me.down( '#messageBodyTemplate' );
                QDesk.util.FormUtils.insertTextAtCursor( messageBodyTemplate, value );
              } );

          window.show();
        }, me );

    // manage message channel profiles
    me.down( '#messageChannelProfilesButton' ).on( 'click', function ( button )
    {
      Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles', {} ).execute();

      // upon message channel profiles window close reload the message channel profile
      // combo tree (profiles might have been added/removed)
      var window = Ext.ComponentQuery.query( 'MessageChannelProfilesWindow' )[ 0 ];
      window.on( 'close', function ()
      {
        var treeStore = me.down( '#messageChannelProfileCombo' ).getStore();
        var rootNode = treeStore.getRootNode();
        rootNode.removeAll( false );
        treeStore.load( { node : rootNode } );
      }, me );
    }, me );

    /*
     * Send Test Message button functionality - enabled only when all allowBlank=false fields
     * on the messageTab have a value.
     */
    var messageTab = me.down( '#messageTab' );
    Ext.each( Ext.ComponentQuery.query( 'field', messageTab ), function ( field )
    {
      field.on( 'change', function ()
      {
        me._maybeEnableSendTestMessageButton();
      }, me );
    }, me );

    me.down( '#sendTestMessage' ).on( 'click', function ( button )
    {
      me._sendTestMessage();
    }, me );

    me._maybeEnableSendTestMessageButton();
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _getRequiredPermissions : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.ADD:
        return [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ];

      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.EDIT:
        return [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ];

      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.CLONE:
        return [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ];

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  /**
   * Checks the value of all required (allowBlank=false) fields on the message tab
   * and if all the required fields have a value, then the Send Test Message button
   * is enabled.
   */
  _maybeEnableSendTestMessageButton : function ()
  {
    var me = this;

    var messageTab = me.down( '#messageTab' );
    var disableButton = false;

    Ext.each( Ext.ComponentQuery.query( 'field', messageTab ), function ( field )
    {
      if ( !field.allowBlank && Ext.isEmpty( field.getValue() ) )
      {
        disableButton = true;
      }
    }, me );

    messageTab.down( '#sendTestMessage' ).setDisabled( disableButton );
  },


  /**
   * Sends a test message.
   */
  _sendTestMessage : function ()
  {
    var me = this;

    var messageChannelProfileCombo = me.down( '#messageChannelProfileCombo' );
    var channelProfileId = messageChannelProfileCombo.getValue();
    var channel = messageChannelProfileCombo.getMessageChannel();

    var recipients = me.down( '#messageToAddr' ).getValue();
    if ( !QDesk.util.MessageChannelProfileUtils.messageChannelSupportsRecipients( channel ) )
    {
      recipients = null;
    }

    var subjectTemplate = me.down( '#messageSubjectTemplate' ).getValue();
    if ( !QDesk.util.MessageChannelProfileUtils.messageChannelSupportsSubject( channel ) )
    {
      subjectTemplate = null;
    }

    var bodyTemplate = me.down( '#messageBodyTemplate' ).getValue();

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.SendTestMessage', {
      channelProfileId : channelProfileId,
      channel : channel,
      recipients : recipients,
      subjectTemplate : subjectTemplate,
      bodyTemplate : bodyTemplate
    } ).execute();
  },


  _createObjectsTreeStore : function ( schedulerType )
  {
    var store = null;

    switch ( schedulerType )
    {
      case QDesk.model.scheduler.SchedulerType.QUARTZ:
        store = Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingObjectsTreeStore' );
        break;

      default:
        Ext.Error.raise( 'Unrecognized scheduler type: ' + schedulerType );
        break;
    }

    if ( store == null )
      Ext.Error.raise( 'Objects store for scheduler type: ' + schedulerType + ' not found.' );

    return store;
  },


  _createDataTypesTreeStore : function ( schedulerType )
  {
    var store = null;

    switch ( schedulerType )
    {
      case QDesk.model.scheduler.SchedulerType.QUARTZ:
        store = Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingDataTypesTreeStore' );
        break;

      default:
        Ext.Error.raise( 'Unrecognized scheduler type: ' + schedulerType );
        break;
    }

    if ( store == null )
      Ext.Error.raise( 'Data types store for scheduler type: ' + schedulerType + ' not found.' );

    return store;
  }
} );
