/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.notif.exec.ExecNotificationRuleMessageMacrosTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg'
  ],

  alias : 'widget.ExecNotificationRuleMessageMacrosTreePanel',

  cls : 'execNotificationRuleMessageMacrosTreePanel',

  scrollable : true,

  collapsible : false,
  useArrows : true,
  rootVisible : false,
  multiSelect : false,
  singleExpand : false,

  displayField : 'macroName',

  qdesk : {
    /*
     * Scheduler type (see QDesk.model.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    var macrosStore = me._createMessageMacrosTreeStore( config.qdesk.schedulerType );

    Ext.apply( config, {
      store : macrosStore,
      columns : [
        {
          xtype : 'treecolumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.ExecNotificationRuleMessageMacrosTreePanel.textMacroName' ),
          flex : 2.6,
          dataIndex : 'macroName',
          sortable : false,
          renderer : function ( value, metaData, record )
          {
            var leaf = record.get( 'leaf' );

            if ( leaf )
              metaData.style = 'text-decoration: underline';

            var i = value.lastIndexOf( '.' );
            return i == -1 ? value : value.substring( i + 1 );
          }
        },
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.common.notif.exec.ExecNotificationRuleMessageMacrosTreePanel.textDescription' ),
          flex : 8,
          dataIndex : 'description',
          sortable : false
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  _createMessageMacrosTreeStore : function ( schedulerType )
  {
    var store = null;

    switch ( schedulerType )
    {
      case QDesk.model.scheduler.SchedulerType.QUARTZ:
        store = Ext.StoreManager.lookup( 'scheduler.quartz.QuartzExecNotificationRuleMessageMacrosTreeStore' );
        break;

      default:
        Ext.Error.raise( 'Unrecognized scheduler type: ' + schedulerType );
        break;
    }

    if ( store == null )
      Ext.Error.raise( 'Store for MessageMacrosWindow not found.' );

    return store;
  }
} );
