/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.SelectJobForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.work.scheduler.quartz.SelectJobGrid'
  ],

  alias : 'widget.QuartzSelectJobForm',

  defaultButton : 'actionButton',

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /**
     * Optional group name of the job to preselect in the grid when it is first loaded.
     */
    preselectedJobGroupName : undefined,

    /**
     * Optional name of the job to preselect in the grid when it is first loaded.
     */
    preselectedJobName : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    Ext.apply( config, {
      items : [
        {
          xtype : 'QuartzSelectJobGrid',
          itemId : 'selectJobGrid',
          height : 400,

          qdesk : {
            connectionId : config.qdesk.connectionCtx.getConnectionId(),
            preselectedJobGroupName : config.qdesk.preselectedJobGroupName,
            preselectedJobName : config.qdesk.preselectedJobName
          }
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSelect' )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    // load job grid upon render
    me.on( 'afterrender', function ()
    {
      me.down( '#selectJobGrid' ).refresh();
    } );
  }
} );
