/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.SelectTriggerWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.quartz.SelectTriggerForm'
  ],

  alias : 'widget.QuartzSelectTriggerWindow',

  cls : 'quartzSelectTriggerWindow',
  iconCls : 'windowIcon',

  modal : true,
  resizable : false,

  width : 800,

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /**
     * Required title text for the window.
     */
    title : undefined,

    /**
     * Optional group name of the trigger to preselect in the grid when it is first loaded.
     */
    preselectedTriggerGroupName : undefined,

    /**
     * Optional name of the trigger to preselect in the grid when it is first loaded.
     */
    preselectedTriggerName : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.title )
      Ext.Error.raise( 'Required config parameter missing: qdesk.title' );

    Ext.apply( config, {
      title : config.qdesk.title,
      items : [
        {
          xtype : 'QuartzSelectTriggerForm',
          itemId : 'form',
          bodyPadding : 0,
          border : 0,
          qdesk : {
            connectionCtx : config.qdesk.connectionCtx,
            preselectedTriggerGroupName : config.qdesk.preselectedTriggerGroupName,
            preselectedTriggerName : config.qdesk.preselectedTriggerName
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } );
  }
} );
