/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.common.ExpressionLanguage',
    'QDesk.model.common.ExpressionType',
    'QDesk.model.scheduler.JobChainTargetExecType',
    'QDesk.model.scheduler.JobChainTargetType',

    'QDesk.store.enums.scheduler.JobChainTargetExecTypeStore',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',

    'QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode',
    'QDesk.view.work.scheduler.quartz.SelectJobGrid',

    'QDesk.view.widget.WConnectionCombo',
    'QDesk.view.widget.expression.WExpressionEditor'
  ],

  alias : 'widget.QuartzEditJobChainTargetForm',

  defaultButton : 'actionButton',

  fieldDefaults : {
    labelWidth : 80
  },

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /*
     * Mode of the form (see EditJobChainTargetFormMode).
     */
    mode : undefined,

    /*
     * Job chain target instance if EditJobChainTargetFormMode = EDIT
     */
    target : undefined,

    originalConnectionType : undefined,

    /**
     * Internally used variable with the originally selected connection ID (local or remote)
     */
    originalConnectionId : undefined,

    currentConnectionType : undefined,

    /**
     * Internally used variable with the currently selected connection ID (local or remote)
     */
    currentConnectionId : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( config.qdesk.mode == QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.EDIT &&
        !config.qdesk.target )
    {
      Ext.Error.raise( 'Required config parameter missing: qdesk.target' );
    }

    var connectionCtx = config.qdesk.connectionCtx;

    // default target connection is the LOCAL connection
    var targetConnectionId = connectionCtx.getConnectionId();

    // if a remote target is defined, use the target's REMOTE connection
    if ( config.qdesk.target
        && config.qdesk.target.type == QDesk.model.scheduler.JobChainTargetType.REMOTE
        && config.qdesk.target.remoteConnectionId )
    {
      targetConnectionId = config.qdesk.target.remoteConnectionId;
    }

    Ext.apply( config, {
          items : [
            {
              xtype : 'hiddenfield',
              itemId : 'jobChainTargetId',
              name : 'id'
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldsetLabelConnection' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                /*
                 * Local Connection
                 */
                {
                  xtype : 'radiofield',
                  itemId : 'localConnectionRadio',
                  name : 'type',
                  inputValue : QDesk.model.scheduler.JobChainTargetType.LOCAL,
                  boxLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.boxLabelLocalConnection' ),
                  boxTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.boxTooltipLocalConnection' )
                },

                /*
                 * Remote Connection
                 */
                {
                  xtype : 'radiofield',
                  itemId : 'remoteConnectionRadio',
                  name : 'type',
                  inputValue : QDesk.model.scheduler.JobChainTargetType.REMOTE,
                  boxLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.boxLabelRemoteConnection' ),
                  boxTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.boxTooltipRemoteConnection' )
                },
                {
                  xtype : 'fieldcontainer',
                  itemId : 'remoteConnectionFields',
                  padding : '0 0 0 10',
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 90
                  },
                  items : [
                    {
                      xtype : 'WConnectionCombo',
                      itemId : 'remoteConnectionCombo',
                      name : 'remoteConnectionId',
                      allowBlank : false,
                      columnWidth : 1
                    }
                  ]
                }
              ]
            },

            {
              xtype : 'checkboxfield',
              itemId : 'enabled',
              name : 'enabled',
              value : true,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.boxLabelEnabled' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.boxTooltipEnabled' ),
              allowBlank : true
            },

            {
              xtype : 'QuartzSelectJobGrid',
              itemId : 'selectJobGrid',
              height : 200,
              qdesk : {
                connectionId : targetConnectionId
              }
            },

            {
              xtype : 'fieldset',
              title : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldsetLabelExecType' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%',
                labelWidth : 120
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'execType',
                  name : 'execType',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldLabelExecType' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecType' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.scheduler.JobChainTargetExecTypeStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },


                /* Delay */
                {
                  xtype : 'numberfield',
                  itemId : 'execTypeDelay',
                  name : 'execTypeDelay',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldLabelExecTypeDelay' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecTypeDelay' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldEmptyTxtExecTypeDelay' ),
                  allowBlank : false,
                  minValue : 1
                },

                /* Date expression */
                {
                  xtype : 'expression.WExpressionEditor',
                  itemId : 'execTypeDateExpression',
                  name : 'execTypeDateExpression',
                  allowBlank : false,
                  maxLength : 1024,
                  value : {
                    language : QDesk.model.common.ExpressionLanguage.JAVASCRIPT,
                    text : null
                  },
                  expressionType : QDesk.model.common.ExpressionType.QUARTZ_JOB_CHAIN_TARGET_EXEC_TYPE_DATE,
                  textEmptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldEmptyTxtExecTypeDateExpressionText' ),
                  textTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.fieldTooltipExecTypeDateExpressionText' ),
                  objectsStore : me._createObjectsTreeStore(),
                  dataTypesStore : me._createDataTypesTreeStore(),
                  extraValidateParamFn : function ()
                  {
                    return {
                      connectionId : connectionCtx.getConnectionId()
                    };
                  }
                }
              ]
            }
          ],

          buttons : [
            {
              itemId : 'cancelButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
            },
            {
              itemId : 'actionButton',
              reference : 'actionButton',
              text : me._getActionButtonText( config.qdesk.mode ),
              disabled : true
              //formBind : true  // cannot use as we also need to consider if there is a job selected in the job grid...
            }
          ]
        }
    );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    if ( me.qdesk.target )
    {
      // populate the form with data (edit mode)
      me.loadRecord( me.qdesk.target );
    }
    else
    {
      // defaults for the add mode
      me.down( '#localConnectionRadio' ).setValue( true );
      me.down( '#remoteConnectionRadio' ).setValue( false );
      me.down( '#execType' ).setValue( QDesk.model.scheduler.JobChainTargetExecType.IMMEDIATE );
    }
  },


  isValid : function ()
  {
    var me = this;
    var valid = me.callParent();

    // check a job is selected in the grid
    var selectJobGrid = me.down( '#selectJobGrid' );
    var jobIsSelected = !Ext.isEmpty( selectJobGrid.getSelection() );

    return valid && jobIsSelected;
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    /*
     * Upon form validation change event, we enable/disable the action button. Apart
     * from the validity of the form, we also check if a job is selected in the grid.
     */
    me.on( 'validitychange', function ( valid )
    {
      me._maybeDisableActionButton();
    } );

    var selectJobGrid = me.down( '#selectJobGrid' );

    // local / remote connection change
    me.down( '#localConnectionRadio' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#remoteConnectionCombo' ), function ( field )
      {
        field.setDisabled( newValue );
        field.allowBlank = newValue;
        field.validate();
      } );

      if ( newValue )
      {
        me.qdesk.currentConnectionType = QDesk.model.scheduler.JobChainTargetType.LOCAL;
        me.qdesk.currentConnectionId = me.qdesk.connectionCtx.getConnectionId();

        if ( me.qdesk.originalConnectionId == null )
        {
          me.qdesk.originalConnectionType = me.qdesk.currentConnectionType;
          me.qdesk.originalConnectionId = me.qdesk.currentConnectionId;
        }

        // reload the selectJobGrid using the local connection
        selectJobGrid.changeConnection( me.qdesk.currentConnectionId );
      }
    } );

    me.down( '#remoteConnectionRadio' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#remoteConnectionCombo' ), function ( field )
      {
        field.setDisabled( !newValue );
        field.allowBlank = !newValue;
        field.validate();
      } );

      if ( newValue )
      {
        // reload the selectJobGrid using the currently selected remote connection
        var remoteConnectionCombo = me.down( '#remoteConnectionCombo' );

        me.qdesk.currentConnectionType = QDesk.model.scheduler.JobChainTargetType.REMOTE;
        me.qdesk.currentConnectionId = remoteConnectionCombo.getValue();

        if ( me.qdesk.originalConnectionId == null )
        {
          me.qdesk.originalConnectionType = me.qdesk.currentConnectionType;
          me.qdesk.originalConnectionId = me.qdesk.currentConnectionId;
        }

        selectJobGrid.changeConnection( me.qdesk.currentConnectionId );
      }
    } );

    me.down( '#remoteConnectionCombo' ).on( 'change', function ( combo, newValue, oldValue )
    {
      me.qdesk.currentConnectionType = QDesk.model.scheduler.JobChainTargetType.REMOTE;
      me.qdesk.currentConnectionId = newValue;

      if ( me.qdesk.originalConnectionId == null )
      {
        me.qdesk.originalConnectionType = me.qdesk.currentConnectionType;
        me.qdesk.originalConnectionId = me.qdesk.currentConnectionId;
      }

      selectJobGrid.changeConnection( me.qdesk.currentConnectionId );
    } );

    // target execution type change
    me.down( '#execType' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#execTypeDelay' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.JobChainTargetExecType.DELAYED );
        field.setDisabled( newValue != QDesk.model.scheduler.JobChainTargetExecType.DELAYED );
        field.validate();
      } );

      Ext.each( me.down( '#execTypeDateExpression' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.JobChainTargetExecType.DATE_EXPRESSION );
        field.setDisabled( newValue != QDesk.model.scheduler.JobChainTargetExecType.DATE_EXPRESSION );
        field.validate();
      } );
    } );

    selectJobGrid.on( 'selectionchange', function ( grid, selected )
    {
      me._maybeDisableActionButton();
    } );

    var selectJobGridStore = selectJobGrid.getStore();
    selectJobGridStore.on( 'load',
        function ( store, records, successful )
        {
          // alert( 'me.qdesk.currentConnectionType=' + me.qdesk.currentConnectionType +
          //     ' me.qdesk.currentConnectionId=' + me.qdesk.currentConnectionId + '\n' +
          //     'me.qdesk.originalConnectionType=' + me.qdesk.originalConnectionType +
          //     ' me.qdesk.originalConnectionId=' + me.qdesk.originalConnectionId );

          if ( successful )
          {
            if ( Ext.isEmpty( records ) )
            {
              // if no records found, but the target is specified, we report it as a warning
              if ( me.qdesk.target )
              {
                var jobName = me.qdesk.target.get( 'jobName' );
                var jobGroupName = me.qdesk.target.get( 'jobGroupName' );

                // record not found
                QDesk.util.UxUtils.showWarningDialog(
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.targetJobNotFound',
                        Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ) );
              }
            }
            else
            {
              /*
               * Non-empty list of records loaded. If it is the first successful load of the store with the original
               * connection ID, we select the target if we find it in the list of loaded records.
               */
              if ( me.qdesk.target
                  && me.qdesk.currentConnectionType == me.qdesk.originalConnectionType
                  && me.qdesk.currentConnectionId == me.qdesk.originalConnectionId )
              {
                var jobName = me.qdesk.target.get( 'jobName' );
                var jobGroupName = me.qdesk.target.get( 'jobGroupName' );

                var storeIdx = store.findBy(
                    function ( record, id )
                    {
                      return record.get( 'name' ) == jobName && record.get( 'group' ).name == jobGroupName;
                    } );

                if ( storeIdx > -1 )
                {
                  var record = store.getAt( storeIdx );

                  /*
                   * It seems the grid needs some time to finish rendering records before the scroll 100% works.
                   */
                  Ext.defer(
                      function ()
                      {
                        //selectJobGrid.setSelection( record );
                        selectJobGrid.getSelectionModel().select( [ record ] );
                        selectJobGrid.getView().focusRow( record );  // scroll to the record
                      }, QDesk.util.constant.Const.DELAY_GRID_SELECT );
                }
                else
                {
                  // record not found
                  QDesk.util.UxUtils.showWarningDialog(
                      QDesk.Msg.get(
                          'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetForm.targetJobNotFound',
                          Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ) );
                }
              }
            }
          }
        } );

    me.down( '#actionButton' ).on( 'click', function ()
    {
      if ( me.isValid() )
      {
        var fieldValues = me.getForm().getFieldValues();

        // set the jobName and jobGroupName (from selected JobDetail model instance)
        var selectedJob = selectJobGrid.getSelection();
        Ext.apply( fieldValues, {
          jobGroupName : selectedJob[ 0 ].get( 'group' ).name,
          jobName : selectedJob[ 0 ].get( 'name' )
        } );

        // set the remote connection name
        if ( fieldValues.type == QDesk.model.scheduler.JobChainTargetType.REMOTE )
        {
          var remoteConnectionCombo = me.down( '#remoteConnectionCombo' );
          Ext.apply( fieldValues, {
            remoteConnectionName : remoteConnectionCombo.getSelectedRecord().get( 'name' )
          } );
        }

        var jobChainTarget = QDesk.util.quartz.QuartzJobChainUtils.createJobChainTarget( fieldValues );

        if ( me.qdesk.mode == QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.ADD )
        {
          me.fireEvent( 'jobChainTargetAdded', jobChainTarget );
        }
        else if ( me.qdesk.mode == QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.EDIT )
        {
          me.fireEvent( 'jobChainTargetUpdated', jobChainTarget, me.qdesk.target );
        }
      }
    } );
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _maybeDisableActionButton : function ()
  {
    var me = this;

    // disable action button if the form has validation errors or no job is selected
    me.down( '#actionButton' ).setDisabled( !me.isValid() );
  },


  _createObjectsTreeStore : function ()
  {
    return Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingObjectsTreeStore' );
  },


  _createDataTypesTreeStore : function ()
  {
    return Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingDataTypesTreeStore' );
  }
} );
