/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',

    'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm'
  ],

  alias : 'widget.QuartzExportExecHistoryWindow',

  cls : 'quartzExportExecHistoryWindow',
  iconCls : 'windowIcon',

  scrollable : true,

  modal : true,
  resizable : false,

  width : 575,

  /* QDesk specific data */
  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryWindow.windowTitle' ),
      items : [
        {
          xtype : 'QuartzExportExecHistoryForm',
          itemId : 'form',
          qdesk : config.qdesk
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    var formPanel = me.down( '#form' );

    me.on( 'afterrender', function ()
    {
      var selectedColumnsField = formPanel.down( '#selectedColumns' );
      QDesk.util.UxUtils.focus( selectedColumnsField, false );
    } );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } );
  }
} );