/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.FormatUtils',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.work.scheduler.common.AgentConfigurationGrid',
    'QDesk.view.work.scheduler.common.ClassNamesGrid',
    'QDesk.view.work.scheduler.quartz.ListenersGrid'
  ],

  alias : 'widget.QuartzSchedulerInfoForm',

  border : 0,

  cls : 'quartzSchedulerInfoForm',
  bodyCls : 'quartzSchedulerInfoFormBody',

  frame : false,

  scrollable : true,

  fieldDefaults : {
    labelWidth : 150
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;

    var items = [];

    items.push(
        {
          xtype : 'displayfield',
          itemId : 'status',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelStatus' )
        },
        {
          xtype : 'displayfield',
          itemId : 'name',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelName' )
        },
        {
          xtype : 'displayfield',
          itemId : 'instanceId',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelInstanceId' )
        },
        {
          xtype : 'displayfield',
          itemId : 'version',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelVersion' )
        },
        {
          xtype : 'displayfield',
          itemId : 'threadPoolClassName',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelThreadPoolClassName' )
        },
        {
          xtype : 'displayfield',
          itemId : 'threadPoolSize',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelThreadPoolSize' ),
          renderer : function ( value )
          {
            if ( value <= 0 )
            {
              return QDesk.Msg.get( 'QDesk.notAvailable' );
            }
            else
            {
              return value;
            }
          }
        },
        {
          xtype : 'displayfield',
          itemId : 'jobStoreClassName',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelJobStoreClassName' )
        }
    );

    if ( licensedProductEdition === QDesk.model.license.ProductEdition.LITE )
    {
      // do nothing
    }
    else
    {
      items.push(
          // *** show only if public API is ready
          {
            xtype : 'displayfield',
            itemId : 'dateInitialized',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelDateInitialized' ),
            hidden : true,
            renderer : function ( value, field )
            {
              if ( value )
              {
                var dateInitialized = QDesk.util.FormatUtils.parseTimestamp( value.rawValue );
                var uptime = new Date().getTime() - dateInitialized.getTime();

                return QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.valueFormatDateInitialized',
                    value.targetFmtValue,
                    QDesk.util.FormatUtils.formatDuration( uptime ) );
              }
            }
          },
          {
            xtype : 'displayfield',
            itemId : 'dateStarted',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelDateStarted' ),
            hidden : true,
            renderer : function ( value, field )
            {
              if ( value )
              {
                var dateStarted = QDesk.util.FormatUtils.parseTimestamp( value.rawValue );
                var uptime = new Date().getTime() - dateStarted.getTime();

                return QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.valueFormatDateStarted',
                    value.targetFmtValue,
                    QDesk.util.FormatUtils.formatDuration( uptime ) );
              }
            }
          },
          {
            xtype : 'displayfield',
            itemId : 'datePaused',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelDatePaused' ),
            hidden : true,
            renderer : function ( value, field )
            {
              if ( value )
              {
                var datePaused = QDesk.util.FormatUtils.parseTimestamp( value.rawValue );
                var downtime = new Date().getTime() - datePaused.getTime();

                return QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.valueFormatDatePaused',
                    value.targetFmtValue,
                    QDesk.util.FormatUtils.formatDuration( downtime ) );
              }
            }
          },
          {
            xtype : 'displayfield',
            itemId : 'dateStopped',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelDateStopped' ),
            hidden : true,
            renderer : function ( value, field )
            {
              if ( value )
              {
                var dateStopped = QDesk.util.FormatUtils.parseTimestamp( value.rawValue );
                var downtime = new Date().getTime() - dateStopped.getTime();

                return QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.valueFormatDateStopped',
                    value.targetFmtValue,
                    QDesk.util.FormatUtils.formatDuration( downtime ) );
              }
            }
          },


          {
            xtype : 'displayfield',
            itemId : 'jobCount',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelJobCount' ),
            hidden : true,
            renderer : function ( value, field )
            {
              var me = this;
              var tooltip = Ext.String.htmlEncode(
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.tooltipJobCountValue' ) );

              var html = '<a href="#" data-qtip="' + tooltip + '">' + value + '</a>';

              Ext.defer( function ()
              {
                var a = field.getEl().down( 'a' );
                if ( a )
                {
                  a.on( 'click', function ( event )
                  {
                    me.fireEvent( 'valueClick', me );
                    event.stopPropagation();
                    event.preventDefault();
                  } );
                }
              }, 100 );

              return html;
            }
          },
          {
            xtype : 'displayfield',
            itemId : 'triggerCount',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelTriggerCount' ),
            hidden : true,
            renderer : function ( value, field )
            {
              var me = this;
              var tooltip = Ext.String.htmlEncode(
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.tooltipTriggerCountValue' ) );

              var html = '<a href="#" data-qtip="' + tooltip + '">' + value + '</a>';

              Ext.defer( function ()
              {
                var a = field.getEl().down( 'a' );
                if ( a )
                {
                  a.on( 'click', function ( event )
                  {
                    me.fireEvent( 'valueClick', me );
                    event.stopPropagation();
                    event.preventDefault();
                  } );
                }
              }, 100 );

              return html;
            }
          }
          // ***************************
      );
    }

    items.push(
        {
          xtype : 'displayfield',
          itemId : 'monitoringUrl',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelMonitoringUrl' ),
          renderer : function ( value, field )
          {
            if ( Ext.isEmpty( value ) )
              return value;

            var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

            var tooltip;
            if ( licensedFeatures.PollMonitoring )
            {
              tooltip =
                  Ext.String.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.tooltipMonitoringUrl' ) );
              return '<a href="' + value + '" class="newWindow" target="_blank" data-qtip="' +
                  tooltip + '">' + value + '</a>';
            }
            else
            {
              tooltip =
                  Ext.String.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.tooltipMonitoringUrlNotLicensed' ) );
              return '<a href="' + value +
                  '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="'
                  + tooltip + '">' + value + '</a>';
            }
          }
        }
    );

    if ( licensedProductEdition == QDesk.model.license.ProductEdition.LITE )
    {
      // do nothing
    }
    else
    {
      items.push(
          {
            xtype : 'textareafield',
            itemId : 'classLoaderInfo',
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldsetLabelClassLoaderInfo' ),
            fieldTooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldsetTooltipClassLoaderInfo' ),
            labelAlign : 'top',
            hidden : true,
            height : 150,
            readOnly : true,
            readOnlyCls : '', /* prevent lower opacity of read-only fields */
            style : {
              marginBottom : '15px'
            }
          },

          {
            xtype : 'fieldset',
            itemId : 'publicApiInfo',
            title : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldsetLabelPublicApiInfo' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldsetTooltipPublicApiInfo' ),
            hidden : true,
            layout : 'anchor',
            defaults : {
              anchor : '100%'
            },
            style : {
              marginBottom : '15px'
            },
            items : [
              {
                xtype : 'displayfield',
                itemId : 'publicApiDetected',
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelPublicApiDetected' ),
                hidden : true
              },

              {
                xtype : 'displayfield',
                itemId : 'publicApiVersion',
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelPublicApiVersion' ),
                hidden : true,
                renderer : function ( value )
                {
                  if ( Ext.isEmpty( value ) )
                    return value;

                  return QDesk.util.FormatUtils.formatVersion( value );
                }
              },
              {
                xtype : 'displayfield',
                itemId : 'publicApiCompatibleVersion',
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelPublicApiCompatibleVersion' ),
                hidden : true
              },

              {
                xtype : 'displayfield',
                itemId : 'publicApiLoadedCorrectly',
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelPublicApiLoadedCorrectly' ),
                hidden : true
              },

              {
                xtype : 'textareafield',
                itemId : 'publicApiClassLoaderInfo',
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldLabelPublicApiClassLoaderInfo' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldTooltipPublicApiClassLoaderInfo' ),
                labelAlign : 'top',
                hidden : true,
                height : 150,
                readOnly : true,
                readOnlyCls : '' /* prevent lower opacity of read-only fields */
              }
            ]
          },

          {
            xtype : 'ClassNamesGrid',
            itemId : 'availableJobClasses',
            title : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldsetLabelAvailableJobClasses' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.fieldsetTooltipAvailableJobClasses' ),
            height : 100,
            hidden : true,
            style : {
              marginBottom : '15px'
            },
            qdesk : {
              visibleNameColumn : false
            }
          },

          {
            xtype : 'QuartzListenersGrid',
            itemId : 'jobListeners',
            title : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.gridTitleJobListeners' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.gridTooltipJobListeners' ),
            height : 100,
            hidden : true,
            style : {
              marginBottom : '15px'
            },
            qdesk : {
              visibleNameColumn : true
            }
          },
          {
            xtype : 'QuartzListenersGrid',
            itemId : 'triggerListeners',
            title : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.gridTitleTriggerListeners' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.gridTooltipTriggerListeners' ),
            height : 100,
            hidden : true,
            style : {
              marginBottom : '15px'
            },
            qdesk : {
              visibleNameColumn : true
            }
          },
          {
            xtype : 'QuartzListenersGrid',
            itemId : 'schedulerListeners',
            title : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.gridTitleSchedulerListeners' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.info.SchedulerInfoForm.gridTooltipSchedulerListeners' ),
            height : 100,
            hidden : true,
            qdesk : {
              visibleNameColumn : false
            }
          }
      );
    }

    Ext.apply( config, {
      items : items
    } );

    me.callParent( [ config ] );
  }
} );
