/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.Msg',

    'QDesk.model.license.ProductEdition',
    'QDesk.model.scheduler.ExecType',
    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzJobGroupStore',
    'QDesk.store.scheduler.quartz.QuartzExecutingJobStore',

    'QDesk.util.quartz.QuartzExecutingJobUtils',
    'QDesk.util.quartz.QuartzJobChainUtils',

    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzExecutingJobsGrid',

  cls : 'quartzExecutingJobsGrid',

  border : false,
  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.quartz.QuartzExecutingJobUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    var publicApiReady = schedulerInfo.isPublicApiReady();

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecutingJobStore' );

    store.getProxy().extraParams = {
      connectionId : connectionCtx.getConnectionId()
    };

    Ext.apply( config, {
      store : store,

      qdesk : {
        connectionCtx : connectionCtx,
        gridId : 'quartz.executingJobsGrid'
      }
    } );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'interruptJobButton',
          scale : 'small',
          iconCls : 'quartzInterruptJob',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipInterruptJobButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_INTERRUPT ],
          qdNotAuthorizedIconCls : 'quartzInterruptJobNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'viewThreadInfoButton',
          scale : 'small',
          iconCls : 'viewThreadInfo',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipViewThreadInfoButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_THREAD_INFO_VIEW ],
          qdNotAuthorizedIconCls : 'viewThreadInfoNAUTH'
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByJobGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.fieldLabelFilterByJobGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.emptyTextFilterByJobGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true,
          tpl : Ext.create( 'Ext.XTemplate',
              '<ul class="x-list-plain"><tpl for=".">',
              '<li role="option" class="x-boundlist-item">{name:htmlEncode}</li>',
              '</tpl></ul>' )
        },
        //'-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          store : store,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 35,
          width : 400,
          disabled : true,
          hidden : true  /* Index searches are currently not available for the currently executing jobs */
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : [
        // job related columns
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.name );
          }
        },
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          sortable : true,
          groupable : true,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.group.name );
          }
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textJobClassColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobClassColumn' ),
          dataIndex : 'jobDetail.jobClass',
          sortable : true,
          groupable : true,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.jobClass );
          }
        },

        // trigger related columns
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'trigger.name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return Ext.String.htmlEncode( trigger.name );
          }
        },
        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'trigger.group.name',
          sortable : true,
          groupable : true,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return Ext.String.htmlEncode( trigger.group.name );
          }
        },

        {
          itemId : 'execTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textExecTypeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeColumn' ),
          dataIndex : 'execType',
          sortable : true,
          groupable : false,
          flex : 0.9,
          renderer : function ( value, metaData, record )
          {
            var text = QDesk.util.EnumUtils.getExecTypeName( value );

            var iconClass, tooltipText;

            if ( value == QDesk.model.scheduler.ExecType.NORMAL )
            {
              iconClass = 'execTypeNormal';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeNormal' );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_SYSTEM )
            {
              iconClass = 'execTypeQDSystem';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDSystem' );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_MANUAL )
            {
              iconClass = 'execTypeQDManual';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDManual' );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_JOB_CHAIN )
            {
              var jobDataMap = record.get( 'jobDataMap' );
              var jobChainDetails = QDesk.util.quartz.QuartzJobChainUtils.extractJobChainDetails( jobDataMap );

              var jobChainId = jobChainDetails[ 'QD_JOB_CHAIN_ID' ];
              var jobChainName = jobChainDetails[ 'QD_JOB_CHAIN_NAME' ];

              var jobChainSrcJobGroupName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_JOB_GROUP_NAME' ];
              var jobChainSrcJobName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_JOB_NAME' ];

              var jobChainSrcTriggerGroupName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_TRIGGER_GROUP_NAME' ];
              var jobChainSrcTriggerName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_TRIGGER_NAME' ];

              var jobChainSrcExecHistoryId = jobChainDetails[ 'QD_JOB_CHAIN_SRC_EXEC_HISTORY_ID' ];

              var jobChainFlowId = jobChainDetails[ 'QD_JOB_CHAIN_FLOW_ID' ];

              iconClass = 'execTypeQDJobChain';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDJobChain',
                  jobChainId,
                  jobChainName,
                  jobChainSrcJobGroupName,
                  jobChainSrcJobName,
                  jobChainSrcTriggerGroupName,
                  jobChainSrcTriggerName,
                  jobChainSrcExecHistoryId,
                  jobChainFlowId
              );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_WEB_SERVICE )
            {
              iconClass = 'execTypeQDWebService';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDWebService' );
            }
            else
            {
              Ext.Error.raise( 'Unexpected exec type value: ' + value );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + Ext.String.htmlEncode( tooltipText ) + '"/>' +
                '<span style="padding-left: 5px;">' + text + '</span>';
          }
        },

        // effective job data map
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var executingJobRecord = grid.getStore().getAt( rowIndex );
                var jobDataMap = executingJobRecord.get( 'jobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap );
              }
            }
          ]
        },

        {
          xtype : 'actioncolumn',
          itemId : 'logColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textLogColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textLogColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipLogColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/scheduler/execution-log-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipLogColumnIcon' ),

              getClass : function ( value, metaData, record )
              {
//                var jobExecutionId = record.get( 'jobExecutionId' );
//                var disabled = Ext.isEmpty( jobExecutionId ) || !jvmAgentReady;
                var disabled = !jvmAgentReady;
                return disabled ? 'nonClickable' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                this.fireEvent( 'requestViewLog', record );
              },
              isActionDisabled : function ( view, rowIndex, colIndex, item, record )
              {
//                var jobExecutionId = record.get( 'jobExecutionId' );
//                return Ext.isEmpty( jobExecutionId ) || !jvmAgentReady;
                return !jvmAgentReady;
              }
            }
          ]
        },

        // non-jobDetail/non-trigger related columns
        {
          itemId : 'fireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textFireTimeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipFireTimeColumn' ),
          dataIndex : 'fireTime',
          sortable : true,
          groupable : true,
          flex : 1.1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },
        {
          itemId : 'scheduledFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textScheduledFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipScheduledFireTimeColumn' ),
          dataIndex : 'scheduledFireTime',
          sortable : true,
          groupable : true,
          flex : 1.1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },
        {
          itemId : 'previousFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textPreviousFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipPreviousFireTimeColumn' ),
          dataIndex : 'previousFireTime',
          sortable : true,
          groupable : true,
          flex : 1.1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },
        {
          itemId : 'nextFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textNextFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipNextFireTimeColumn' ),
          dataIndex : 'nextFireTime',
          sortable : true,
          groupable : true,
          flex : 1.1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'recoveringColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textRecoveringColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipRecoveringColumn' ),
          dataIndex : 'recovering',
          sortable : false,
          groupable : false,
          width : 80
        },
        {
          itemId : 'refireCountColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textRefireCountColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipRefireCountColumn' ),
          dataIndex : 'refireCount',
          sortable : false,
          groupable : false,
          flex : 0.75,
          renderer : function ( value, metaData, record )
          {
            return Ext.String.htmlEncode( record.get( 'refireCount' ) );
          }
        },

        {
          itemId : 'fireInstanceIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.textFireInstanceId' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipFireInstanceId' ),
          dataIndex : 'fireInstanceId',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            return Ext.String.htmlEncode( record.get( 'fireInstanceId' ) );
          }
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionCtx.getConnectionId()
    };

    me.down( '#filterByJobGroupCombo' ).setStore( jobGroupStore );

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Invoked to create a context menu for the specified executing job in the ExecutingJobsGrid row.
   * @param executingJob an executing job.
   * @private
   */
  _createRowCtxMenu : function ( executingJob )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // interrupt job
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.jobCtxMenuInterrupt' ),
          iconCls : 'quartzInterruptJob',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_INTERRUPT ],
          qdNotAuthorizedIconCls : 'quartzInterruptJobNAUTH',
          handler : function ()
          {
            me.fireEvent( 'interruptJob', executingJob );
          }
        },
        // view thread info
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsGrid.jobCtxMenuViewThreadInfo' ),
          iconCls : 'viewThreadInfo',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_THREAD_INFO_VIEW ],
          qdNotAuthorizedIconCls : 'viewThreadInfoNAUTH',
          handler : function ()
          {
            me.fireEvent( 'viewThreadInfo', executingJob );
          }
        }
      ]
    } );
  }
} );
