/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.triggers.JobTriggersCtxView', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.scheduler.quartz.jobs.triggers.JobTriggersGrid'
  ],

  alias : 'widget.QuartzJobTriggersCtxView',

  iconCls : 'quartzJobTriggersCtxViewIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    // var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    // var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var items = [];
    if ( licensedFeatures.MaxTriggersPerJob != QDesk.util.license.LicensedProductFeatures.UNLIMITED_INT )
    {
      items.push( {
        xtype : 'WarningMessagePanel',
        style : {
          margin : '5px'
        },
        qdesk : {
          msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.maxTriggersPerJob',
              QDesk.util.license.LicensedProduct.productName, licensedFeatures.MaxTriggersPerJob )
        }
      } );
    }

    items.push( {
      xtype : 'QuartzJobTriggersGrid',
      itemId : 'jobTriggersGrid',
      flex : 1,
      qdesk : config.qdesk
    } );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.triggers.JobTriggersCtxView.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.triggers.JobTriggersCtxView.tooltip' ),
      items : [
        {
          xtype : 'panel',
          border : false,
          layout : 'fit',
          flex : 1,
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'vbox',
                align : 'stretch'
              },
              items : items
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );