/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus',
    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzMisfiredTriggersStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.quartz.QuartzEnumUtils',

    'QDesk.view.widget.WMaxTodayDateField',
    'QDesk.view.widget.WSearchField',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzMisfiredTriggersGrid',

  cls : 'quartzMisfiredTriggersGrid',

  border : false,
  scrollable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    if ( !config.qdesk.fetchPeriod )
      Ext.Error.raise( 'Required config parameter missing: qdesk.fetchPeriod' );

    if ( !config.qdesk.maxFetchSize )
      Ext.Error.raise( 'Required config parameter missing: qdesk.maxFetchSize' );

    var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    // var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzMisfiredTriggersStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'WDateTimePeriod',
          itemId : 'fetchPeriod',
          allowBlank : false,
          disabled : true,
          value : config.qdesk.fetchPeriod,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              // [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 72,
          width : 172,
          editable : false,
          disabled : true,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextFieldFilterQueryText' ),
          //store : misfiredTriggersStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          hidden : true, // TODO: full-text index for misfired triggers is not available yet
          disabled : true,
          listeners : {
            search : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' ) );
                }
              }
            },
            clear : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],


      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          itemId : 'detectedAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textDetectedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipDetectedAtColumn' ),
          dataIndex : 'detectedAt',
          sortable : true,
          groupable : false,
          flex : 1.7,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        {
          itemId : 'missedFireTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textMissedFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipMissedFireTimeColumn' ),
          dataIndex : 'missedFireTime',
          sortable : true,
          groupable : false,
          flex : 1.7,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },


        {
          itemId : 'refireStatusColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textRefireStatusColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefireStatusColumn',
              QDesk.util.quartz.QuartzEnumUtils.getMisfiredTriggerRefireStatusName(
                  QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.NOT_FIRED ),
              QDesk.util.quartz.QuartzEnumUtils.getMisfiredTriggerRefireStatusName(
                  QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_EXECUTING ),
              QDesk.util.quartz.QuartzEnumUtils.getMisfiredTriggerRefireStatusName(
                  QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_SUCCESS ),
              QDesk.util.quartz.QuartzEnumUtils.getMisfiredTriggerRefireStatusName(
                  QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_VETO ),
              QDesk.util.quartz.QuartzEnumUtils.getMisfiredTriggerRefireStatusName(
                  QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_ERROR ) ),
          dataIndex : 'refireStatus',
          sortable : false,
          groupable : false,
          flex : 1,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var refireEhId = record.get( 'refireEhId' );
            var refireEhStartedAt = record.get( 'refireEhStartedAt' );
            var refireEhFinishedAt = record.get( 'refireEhFinishedAt' );
            //var refireEhExecStatus = record.get( 'refireEhExecStatus' );
            var refireEhExecType = record.get( 'refireEhExecType' );
            var refireEhTriggerGroupName = record.get( 'refireEhTriggerGroupName' );
            var refireEhTriggerName = record.get( 'refireEhTriggerName' );

            var text = QDesk.util.quartz.QuartzEnumUtils.getMisfiredTriggerRefireStatusName( value );

            var iconClass, tooltipText;

            if ( value === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.NOT_FIRED )
            {
              iconClass = 'quartzRefireStatusNotFired';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefireStatusNotFired' );
            }
            else if ( value === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_EXECUTING )
            {
              iconClass = 'quartzRefireStatusFiredExecuting';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefireStatusFiredExecuting',
                  refireEhStartedAt == null ? null : refireEhStartedAt.targetFmtValue,
                  //QDesk.util.EnumUtils.getExecTypeName( refireEhExecType ),
                  me._getExecTypeIconAndText( refireEhExecType ),
                  refireEhTriggerGroupName,
                  refireEhTriggerName );
            }
            else if ( value === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_SUCCESS )
            {
              iconClass = 'quartzRefireStatusFiredSuccess';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefireStatusFiredSuccess',
                  refireEhId,
                  refireEhStartedAt == null ? null : refireEhStartedAt.targetFmtValue,
                  refireEhFinishedAt == null ? null : refireEhFinishedAt.targetFmtValue,
                  //QDesk.util.EnumUtils.getExecTypeName( refireEhExecType ),
                  me._getExecTypeIconAndText( refireEhExecType ),
                  refireEhTriggerGroupName,
                  refireEhTriggerName );
            }
            else if ( value === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_VETO )
            {
              iconClass = 'quartzRefireStatusFiredVeto';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefireStatusFiredVeto',
                  refireEhId,
                  refireEhStartedAt == null ? null : refireEhStartedAt.targetFmtValue,
                  refireEhFinishedAt == null ? null : refireEhFinishedAt.targetFmtValue,
                  //QDesk.util.EnumUtils.getExecTypeName( refireEhExecType ),
                  me._getExecTypeIconAndText( refireEhExecType ) );
            }
            else if ( value === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_ERROR )
            {
              iconClass = 'quartzRefireStatusFiredError';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipRefireStatusFiredError',
                  refireEhId,
                  refireEhStartedAt == null ? null : refireEhStartedAt.targetFmtValue,
                  refireEhFinishedAt == null ? null : refireEhFinishedAt.targetFmtValue,
                  //QDesk.util.EnumUtils.getExecTypeName( refireEhExecType ),
                  me._getExecTypeIconAndText( refireEhExecType ),
                  refireEhTriggerGroupName,
                  refireEhTriggerName );
            }
            else
            {
              Ext.Error.raise( 'Unexpected refire status value: ' + value );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + Ext.String.htmlEncode( tooltipText ) + '"/>' +
                '<span style="padding-left: 5px;">' + text + '</span>';
          }
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerNameColumn' ),
          dataIndex : 'schedulerName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerInstanceIdColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerInstanceIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerInstanceIdColumn' ),
          dataIndex : 'schedulerInstanceId',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerVersionColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textSchedulerVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipSchedulerVersionColumn' ),
          dataIndex : 'schedulerVersion',
          renderer : function ( value )
          {
            return value.major + '.' + value.minor + '.' + value.maintenance;
          },
          sortable : true,
          groupable : false,
          flex : 1.1
        },
        /* End of scheduler related columns */

        /* Trigger related columns */
        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'triggerGroupName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'triggerName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          xtype : 'templatecolumn',
          itemId : 'triggerTypeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerTypeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerTypeColumn' ),
          dataIndex : 'trigger.type',
          tpl : '{[Ext.isEmpty( values.triggerType ) ? QDesk.Msg.get("QDesk.unknown") : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( values.triggerType )]}',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        {
          itemId : 'triggerMisfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textTriggerMisfireInstructionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipTriggerMisfireInstructionColumn' ),
          dataIndex : 'triggerMisfireInstruction',
          sortable : false,
          groupable : false,
          /* Need custom renderer to display misfire instruction name */
          renderer : function ( value, metaData, record )
          {
            var triggerType = record.get( 'triggerType' );

            if ( triggerType )
            {
              var misfireInstruction =
                  QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionFromNumber( triggerType,
                      value );

              return QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionName( triggerType,
                  misfireInstruction );
            }
            else
            {
              // if we do not know the trigger type, then we are not able to determine the misfire instruction name;
              // so we display the numeric constant instead
              return QDesk.Msg.get( 'QDesk.unknown' ) + ' (' + value.toString() + ')';
            }
          },
          flex : 1
        },
        /* End of trigger related columns */

        /* Job related columns */
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobGroupName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        /* End of job related columns */

        /* Host related data */
        {
          itemId : 'hostNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textHostNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipHostNameColumn' ),
          dataIndex : 'hostName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        /* JVM related data */
        {
          xtype : 'numbercolumn',
          itemId : 'jvmPidColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJvmPidColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJvmPidColumn' ),
          dataIndex : 'jvmPid',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          itemId : 'jvmNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJvmNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJvmNameColumn' ),
          dataIndex : 'jvmName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmVendorColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJvmVendorColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJvmVendorColumn' ),
          dataIndex : 'jvmVendor',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJvmVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJvmVersionColumn' ),
          dataIndex : 'jvmVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmRuntimeVersionColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textJvmRuntimeVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipJvmRuntimeVersionColumn' ),
          dataIndex : 'jvmRuntimeVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        /* OS related data */
        {
          itemId : 'osNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textOsNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipOsNameColumn' ),
          dataIndex : 'osName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'osVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textOsVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipOsVersionColumn' ),
          dataIndex : 'osVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'osArchColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.textOsArchColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.tooltipOsArchColumn' ),
          dataIndex : 'osArch',
          sortable : true,
          groupable : false,
          flex : 0.5
        }
      ],

      store : store,

      qdesk : {
        connectionCtx : connectionCtx,
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );

    var maxFetchSize = QDesk.ApplicationConfig.get( me.qdesk.gridId + '.maxFetchSize' );
    me.down( '#maxFetchSizeSelect' ).setValue( maxFetchSize );

    // rows have different CSS style that reflect the refire status
    me.getView().getRowClass = function ( record )
    {
      var refireStatus = record.get( 'refireStatus' );
      //var refireExecHistoryExecStatus = record.get( 'refireExecHistoryExecStatus' );

      if ( refireStatus === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.NOT_FIRED )
      {
        return 'refireStatusNotFired';
      }
      else if ( refireStatus === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_EXECUTING )
      {
        return 'refireStatusFiredExecuting';
      }
      else if ( refireStatus === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_SUCCESS )
      {
        return 'refireStatusFiredSuccess';
      }
      else if ( refireStatus === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_VETO )
      {
        return 'refireStatusFiredVeto';
      }
      else if ( refireStatus === QDesk.model.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.FIRED_ERROR )
      {
        return 'refireStatusFiredError';
      }
      else return '';
    };
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var extra = {};

    var fetchPeriod = me.down( '#fetchPeriod' ).getValue();
    if ( fetchPeriod != null )
    {
      var fetchPeriodLength = QDesk.util.CommonUtils.getDiffInDays( fetchPeriod.from, fetchPeriod.to );
      extra[ me.qdesk.gridId + '.fetchPeriodLength' ] = fetchPeriodLength;
    }

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();
    extra[ me.qdesk.gridId + '.maxFetchSize' ] = maxFetchSize;

    return extra;
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();


    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var emptyText = null;

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          var jobsTab = me.up( '#jobsTab' );
          var triggersTab = me.up( '#triggersTab' );

          if ( jobsTab )
          {
            var selectedJobs = jobsTab.down( '#jobsGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedJobs ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoJobSelected' );
            }
            else
            {
              if ( me.filtersValid() )
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoMatchingDataForJob' );
              }
              else
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' );
              }
            }
          }
          else if ( triggersTab )
          {
            var selectedTriggers = triggersTab.down( '#triggersGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedTriggers ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoTriggerSelected' );
            }
            else
            {
              if ( me.filtersValid() )
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoMatchingDataForTrigger' );
              }
              else
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' );
              }
            }
          }
          else
          {
            if ( me.filtersValid() )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextNoMatchingData' );
            }
            else
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextInvalidFilters' );
            }
          }
        }
        else
        {
          // records not empty => show the records, not the empty text
        }
      }
      else
      {
        emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.misfire.MisfiredTriggersGrid.emptyTextErrorLoadingData' );
      }

      if ( !Ext.isEmpty( emptyText ) )
      {
        me.empty( emptyText );
      }
    } );
  },


  _getExecTypeIconAndText : function ( execType )
  {
    var iconClass;
    switch ( execType )
    {
      case QDesk.model.scheduler.ExecType.NORMAL:
        iconClass = 'execTypeNormalIcon';
        break;

      case QDesk.model.scheduler.ExecType.QD_SYSTEM:
        iconClass = 'execTypeQDSystemIcon';
        break;

      case QDesk.model.scheduler.ExecType.QD_MANUAL:
        iconClass = 'execTypeQDManualIcon';
        break;

      case QDesk.model.scheduler.ExecType.QD_JOB_CHAIN:
        iconClass = 'execTypeQDJobChainIcon';
        break;

      case QDesk.model.scheduler.ExecType.QD_WEB_SERVICE:
        iconClass = 'execTypeQDWebServiceIcon';
        break;

      default:
        Ext.Error.raise( 'Unexpected exec type value: ' + execType );
    }
    return '<span class="nobr ' + iconClass + '" style="padding: 0 2px 2px 21px;">' +
        QDesk.util.EnumUtils.getExecTypeName( execType ) + '</span>';
  },


  /**
   * Returns true if the filters in the grid's toolbar are valid, false otherwise.
   * @return true if filters are valid, false otherwise.
   */
  filtersValid : function ()
  {
    var me = this;

    if ( !me.rendered )
      return true;

    var fetchPeriodPicker = me.down( '#fetchPeriod' );
    var valid = fetchPeriodPicker.isValid();

    // here we can add other filters

    return valid;
  },


  /**
   * Empties the grid and shows the specified emptyMsg (if not null).
   * @param emptyText an empty message to show.
   */
  empty : function ( emptyText )
  {
    var me = this;

    var store = me.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
    store.getProxy().extraParams = {};
    store.loadRecords( [] );

    if ( !Ext.isEmpty( emptyText ) )
    {
      var view = me.getView();
      view.emptyText = emptyText;
      view.refresh();
    }
  }
} );
