/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.NotificationSeverity',
    'QDesk.model.message.channel.MessageChannel',
    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzExecNotificationRuleStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGridController',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzExecNotificationRulesGrid',

  cls : 'quartzExecNotificationRulesGrid',

  border : false,
  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.quartz.QuartzExecNotificationRuleUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    // var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      controller : Ext.create(
          'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGridController', {
            qdesk : {
              connectionCtx : connectionCtx
            }
          } )
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.3
        }
    );

    if ( config.qdesk.gridId == 'quartz.jobExecNotificationRulesViewGrid' )
    {
      columns.push(
          {
            itemId : 'jobColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textJobColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipJobColumn' ),
            sortable : true,
            groupable : false,
            flex : 1,
            renderer : function ( value, metaData, record )
            {
              var jobGroupName = record.get( 'jobGroupName' );
              var jobName = record.get( 'jobName' );
              return Ext.String.htmlEncode( jobGroupName + ' / ' + jobName );
            }
          }
      );
    }

    if ( config.qdesk.gridId == 'quartz.triggerExecNotificationRulesViewGrid' )
    {
      columns.push(
          {
            itemId : 'triggerColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textTriggerColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipTriggerColumn' ),
            sortable : true,
            groupable : false,
            flex : 1,
            renderer : function ( value, metaData, record )
            {
              var triggerGroupName = record.get( 'triggerGroupName' );
              var triggerName = record.get( 'triggerName' );
              return Ext.String.htmlEncode( triggerGroupName + ' / ' + triggerName );
            }
          }
      );
    }

    columns.push(
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'severityColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSeverityColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSeverityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSeverityColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;
            var severity = record.get( 'severity' );

            if ( severity == QDesk.model.scheduler.NotificationSeverity.INFO )
              iconClass = 'notificationSeverityInfo nonClickable';
            else if ( severity == QDesk.model.scheduler.NotificationSeverity.WARNING )
              iconClass = 'notificationSeverityWarning nonClickable';
            else if ( severity == QDesk.model.scheduler.NotificationSeverity.ERROR )
              iconClass = 'notificationSeverityError nonClickable';
            else if ( severity == QDesk.model.scheduler.NotificationSeverity.FATAL )
              iconClass = 'notificationSeverityFatal nonClickable';
            else
              Ext.Error.raise( 'Unexpected severity value: ' + value );

            tooltipText = QDesk.util.EnumUtils.getNotificationSeverityName( severity );

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>';
          }
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'enabledColumn',
          align : 'center',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledColumn' ),
          dataIndex : 'enabled',
          sortable : false,
          groupable : false,
          width : 100
        },

        {
          itemId : 'enabledOnWeekdaysColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledOnWeekdaysColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledOnWeekdaysColumn' ),
          dataIndex : 'enabledOnWeekday',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enums value */
          renderer : function ( value, metaData, record )
          {
            var result = '';
            Ext.each( value, function ( weekday, index, weekdays )
            {
              result += QDesk.util.EnumUtils.getWeekdayShortName( weekday );
              if ( index < weekdays.length - 1 )
                result += ', ';
            } );

            return result;
          },
          flex : 1
        },

        {
          itemId : 'enabledFromTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledFromTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledFromTimeColumn' ),
          dataIndex : 'enabledFromTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'enabledToTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textEnabledToTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEnabledToTimeColumn' ),
          dataIndex : 'enabledToTime',
          sortable : false,
          groupable : false,
          flex : 0.5
        },

        {
          itemId : 'conditionTypeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textConditionTypeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipConditionTypeColumn' ),
          dataIndex : 'conditionType',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enum value */
          renderer : function ( value, metaData, record )
          {
            switch ( value )
            {
              case QDesk.model.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS:
                var conditionExecStatus = record.get( 'conditionExecStatus' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExecStatusName( conditionExecStatus ) + ']';
                break;

              case QDesk.model.scheduler.ExecNotificationRuleConditionType.MAX_DURATION:
                var conditionMaxDuration = record.get( 'conditionMaxDuration' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    conditionMaxDuration + 'ms]';
                break;

              case QDesk.model.scheduler.ExecNotificationRuleConditionType.EXPRESSION:
                var conditionExpression = record.get( 'conditionExpression' );
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value ) + ' [' +
                    QDesk.util.EnumUtils.getExpressionLanguageName( conditionExpression.language ) + ']';
                break;

              default:
                return QDesk.util.EnumUtils.getExecNotificationRuleConditionTypeName( value );
            }
          },
          flex : 0.7
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'messageChannelProfileColumn',
          dataIndex : 'messageChannelProfileName',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessageChannelProfileColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessageChannelProfileColumn' ),
          sortable : false,
          groupable : false,
          flex : 1,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;

            if ( value )
            {
              var messageChannel = record.get( 'messageChannelProfileChannel' );
              tooltipText = QDesk.util.EnumUtils.getMessageChannelName( messageChannel );

              if ( messageChannel == QDesk.model.message.channel.MessageChannel.COLLABORATION_HIPCHAT )
                iconClass = 'messageChannelCollaborationHipChat';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.COLLABORATION_SLACK )
                iconClass = 'messageChannelCollaborationSlack';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.EMAIL )
                iconClass = 'messageChannelEmail';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_AIM )
                iconClass = 'messageChannelImAim';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_GTALK )
                iconClass = 'messageChannelImGtalk';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_ICQ )
                iconClass = 'messageChannelImIcq';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_XMPP )
                iconClass = 'messageChannelImXmpp';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.IM_YAHOO )
                iconClass = 'messageChannelImYahoo';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_CENTRAL )
                iconClass = 'messageChannelSmsClickatellCentral';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLICKATELL_PLATFORM )
                iconClass = 'messageChannelSmsClickatellPlatform';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_CLOCKWORK )
                iconClass = 'messageChannelSmsClockwork';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_NEXMO )
                iconClass = 'messageChannelSmsNexmo';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_SMSGLOBAL )
                iconClass = 'messageChannelSmsSmsGlobal';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SMS_TWILIO )
                iconClass = 'messageChannelSmsTwilio';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV1_TRAP )
                iconClass = 'messageChannelSnmpV1Trap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV2C_TRAP )
                iconClass = 'messageChannelSnmpV2cTrap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP )
                iconClass = 'messageChannelSnmpV3Trap';
              else if ( messageChannel == QDesk.model.message.channel.MessageChannel.WEB_SERVICE )
                iconClass = 'messageChannelWebService';
            }
            else
            {
              value = QDesk.Msg.get( 'QDesk.notAvailable' );
              iconClass = 'messageChannelUnknown';
              var messageChannelProfileId = record.get( 'messageChannelProfileId' );
              tooltipText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessageChannelProfileNotFound',
                      messageChannelProfileId );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>' +
                '<span style="padding-left: 5px;">' + Ext.String.htmlEncode( value ) + '</span>';
          }
        },

        {
          itemId : 'createdAtColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textCreatedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipCreatedAtColumn' ),
          dataIndex : 'createdAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },
        {
          itemId : 'updatedAtColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textUpdatedAtColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipUpdatedAtColumn' ),
          dataIndex : 'updatedAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        {
          xtype : 'actioncolumn',
          itemId : 'messagesColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessagesColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.textMessagesColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessagesColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/message-queue-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipMessagesColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var notificationRule = grid.getStore().getAt( rowIndex );
                me.fireEvent( 'viewRuleMessages', notificationRule );
              }
            }
          ]
        }
    );

    ///

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecNotificationRuleStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                me.fireEvent( 'refresh' );
              }
            }
          }
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addButton',
          scale : 'small',
          iconCls : 'addExecNotifRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipAddExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ],
          qdNotAuthorizedIconCls : 'addExecNotifRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                me.fireEvent( 'addRule' );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'deleteButton',
          scale : 'small',
          iconCls : 'deleteExecNotifRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipDeleteExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_DELETE ],
          qdNotAuthorizedIconCls : 'deleteExecNotifRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var notificationRule = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'deleteRule', notificationRule );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'editButton',
          scale : 'small',
          iconCls : 'editExecNotifRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipEditExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'editExecNotifRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var notificationRule = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'editRule', notificationRule );
              }
            }
          }
        },
        {
          xtype : 'button',
          itemId : 'cloneButton',
          scale : 'small',
          iconCls : 'cloneExecNotifRule',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipCloneExecNotificationRule' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ],
          qdNotAuthorizedIconCls : 'cloneExecNotifRuleNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var notificationRule = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'cloneRule', notificationRule );
              }
            }
          }
        },
        '-',
        {
          xtype : 'button',
          itemId : 'viewAuditTrailButton',
          scale : 'small',
          iconCls : 'viewExecNotifRuleAuditTrail',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipViewExecNotificationRuleAuditTrail' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
          qdNotAuthorizedIconCls : 'viewExecNotifRuleAuditTrailNAUTH',
          listeners : {
            click : {
              fn : function ( button )
              {
                var notificationRule = me.getSelectionModel().getSelection()[ 0 ];
                me.fireEvent( 'viewRuleAuditTrail', notificationRule );
              }
            }
          }
        },
        '-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextFieldFilterQueryText' ),
          store : store,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : columns,

      store : store,

      qdesk : {
        connectionCtx : connectionCtx,
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          var emptyText = null;

          /*
           * Global views.
           */
          if ( me.qdesk.gridId == 'quartz.schedulerExecNotificationRulesViewGrid' )
          {
            emptyText = QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForScheduler' );

          }
          else if ( me.qdesk.gridId == 'quartz.jobExecNotificationRulesViewGrid' )
          {
            emptyText = QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForJobs' );

          }
          else if ( me.qdesk.gridId == 'quartz.triggerExecNotificationRulesViewGrid' )
          {
            emptyText = QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForTriggers' );

          }

          /*
           * Contextual views.
           */
          else if ( me.qdesk.gridId == 'quartz.jobExecNotificationRulesGrid' )
          {
            var jobsTab = me.up( '#jobsTab' );
            if ( jobsTab )
            {
              var selectedJobs = jobsTab.down( '#jobsGrid' ).getSelectionModel().getSelection();
              if ( Ext.isEmpty( selectedJobs ) )
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoJobSelected' );
              }
              else
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForJob' );
              }
            }
          }
          else if ( me.qdesk.gridId == 'quartz.triggerExecNotificationRulesGrid' )
          {
            var triggersTab = me.up( '#triggersTab' );
            if ( triggersTab )
            {
              var selectedTriggers = triggersTab.down( '#triggersGrid' ).getSelectionModel().getSelection();
              if ( Ext.isEmpty( selectedTriggers ) )
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoTriggerSelected' );
              }
              else
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextNoMatchingDataForTrigger' );
              }
            }
          }
          else
          {
            Ext.Error.raise( 'Unrecognized qdesk.gridId config value: ' + me.qdesk.gridId );
          }

          view.emptyText = emptyText;

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Invoked to create a context menu for the specified execution notification rule.
   * @param execNotifRule an execution notification rule.
   * @private
   */
  _createRowCtxMenu : function ( execNotifRule )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // enable rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuEnable' ),
          iconCls : 'enableExecNotificationRule',
          disabled : execNotifRule.get( 'enabled' ),
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'enableExecNotificationRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'enableRule', execNotifRule );
          }
        },
        // disable rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuDisable' ),
          iconCls : 'disableExecNotificationRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'disableExecNotificationRuleNAUTH',
          disabled : !execNotifRule.get( 'enabled' ),
          handler : function ()
          {
            me.fireEvent( 'disableRule', execNotifRule );
          }
        },
        '-',
        // delete rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuDelete' ),
          iconCls : 'deleteExecNotifRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_DELETE ],
          qdNotAuthorizedIconCls : 'deleteExecNotifRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteRule', execNotifRule );
          }
        },
        // edit rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuEdit' ),
          iconCls : 'editExecNotifRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_UPDATE ],
          qdNotAuthorizedIconCls : 'editExecNotifRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'editRule', execNotifRule );
          }
        },
        // clone rule
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuClone' ),
          iconCls : 'cloneExecNotifRule',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_NOTIF_RULE_ADD ],
          qdNotAuthorizedIconCls : 'cloneExecNotifRuleNAUTH',
          handler : function ()
          {
            me.fireEvent( 'cloneRule', execNotifRule );
          }
        },
        '-',
        // view audit trail
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid.ruleCtxMenuViewAuditTrail' ),
          iconCls : 'viewExecNotifRuleAuditTrail',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
          qdNotAuthorizedIconCls : 'viewExecNotifRuleAuditTrailNAUTH',
          handler : function ()
          {
            me.fireEvent( 'viewRuleAuditTrail', execNotifRule );
          }
        }
      ]
    } );
  }
} );