/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.FormatUtils',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGrid',
    'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesViewController'
  ],

  alias : 'widget.QuartzTriggerExecNotificationRulesView',

  border : false,

  iconCls : 'quartzTriggerExecNotificationsViewIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    var publicApiReady = schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      controller : Ext.create( 'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesViewController',
          {
            qdesk : {
              connectionCtx : connectionCtx
            }
          } )
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications )
    {
      if ( jvmAgentReady )
      {
        var items = [];

        if ( !publicApiReady )
        {
          items.push(
              {
                xtype : 'WarningMessagePanel',
                style : {
                  margin : '5px'
                },
                qdesk : {
                  msg : me.getPublicApiWarningMessage( connectionCtx )
                }
              }
          );
        }

        items.push(
            {
              xtype : 'QuartzExecNotificationRulesGrid',
              itemId : 'execNotificationRulesGrid',
              flex : 1,
              qdesk : {
                connectionCtx : connectionCtx,
                gridId : 'quartz.triggerExecNotificationRulesViewGrid'
              }
            }
        );

        /*
         * Intentionally over-nested a bit so that we use identical layout to other tabs that
         * have a license restrictions related message panel above the grid.
         */
        Ext.apply( config, {
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : 'fit',
              flex : 1,
              items : [
                {
                  xtype : 'panel',
                  border : false,
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : items
                }
              ]
            }
          ]
        } );
      }
      else
      {
        Ext.apply( config, {
          items : [
            {
              xtype : 'WarningMessagePanel',
              style : {
                margin : '5px'
              },
              qdesk : {
                msg : me.getJvmAgentWarningMessage( connectionCtx )
              }
            }
          ]
        } );
      }
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView.tooltip' )
    } );

    me.callParent( [ config ] );
  },

  /**
   * Overridden method from the base class that returns the warning message shown at the top of this view.
   * @param connectionCtx a {@link QDesk.ConnectionContext} instance.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView.publicApiNotLoadedCorrectlyMsg' );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView.publicApiIncompatibleVersionMsg',
            QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get(
          'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView.publicApiNotDetectedMsg' );
    }
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView.title' );
    if ( !licensedFeatures.ExecutionNotifications )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );