Ext.define( 'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesViewController', {
  extend : 'Ext.app.ViewController',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SelectTrigger',

    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',

    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid',
    'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail',

    'QDesk.model.message.MessageCreatorType',

    'QDesk.util.license.LicensedProductFeatures'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    me.callParent( [ config ] );
  },


  init : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    // // TriggerExecNotificationRulesView
    var view = me.getView();

    view.on( 'activate',
        function ()
        {
          if ( !view.isInitialized() )
          {
            me._bindEventHandlers( view );
            view.setInitialized( true );

            Ext.create(
                'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
                  connectionCtx : connectionCtx
                } ).execute();
          }
        } );

    me.callParent();
  },


  /**
   * Binds events of the specified view to their respective handlers.
   */
  _bindEventHandlers : function ( view )
  {
    var me = this;
    me._bindEventHandlersNotificationRulesGrid( view );
  },


  /**
   * Binds events of the embedded ExecNotificationRulesGrid to their respective handlers.
   */
  _bindEventHandlersNotificationRulesGrid : function ( view )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var connectionCtx = me.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    if ( licensedFeatures.ExecutionNotifications && jvmAgentReady )
    {
      var notificationRulesGrid = view.down( '#execNotificationRulesGrid' );

      // refresh button
      notificationRulesGrid.on( 'refresh',
          function ()
          {
            me._refreshRules();
          } );

      notificationRulesGrid.on( 'addRule',
          function ()
          {
            me._addRule();
          } );

      notificationRulesGrid.on( 'enableRule',
          function ( notificationRule )
          {
            me._enableRule( notificationRule );
          } );

      notificationRulesGrid.on( 'disableRule',
          function ( notificationRule )
          {
            me._disableRule( notificationRule );
          } );

      notificationRulesGrid.on( 'deleteRule',
          function ( notificationRule )
          {
            me._deleteRule( notificationRule );
          } );

      notificationRulesGrid.on( 'editRule',
          function ( notificationRule )
          {
            me._editRule( notificationRule );
          } );

      notificationRulesGrid.on( 'cloneRule',
          function ( notificationRule )
          {
            me._cloneRule( notificationRule );
          } );

      notificationRulesGrid.on( 'viewRuleAuditTrail',
          function ( notificationRule )
          {
            me._viewRuleAuditTrail( notificationRule );
          } );

      notificationRulesGrid.on( 'viewRuleMessages',
          function ( notificationRule )
          {
            me._viewRuleMessages( notificationRule );
          } );
    }
  },


  _refreshRules : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
      connectionCtx : connectionCtx
    } ).execute();
  },


  _addRule : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.SelectTrigger', {
      connectionCtx : connectionCtx,
      title : QDesk.Msg.get(
          'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesViewController.selectTriggerWindowTitle' ),
      /*
       * Upon trigger selection, open the Add job chain window.
       */
      success : function ( trigger )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
          connectionCtx : connectionCtx,
          triggerGroupName : trigger.get( 'group' ).name,
          triggerName : trigger.get( 'name' ),
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid',
                {
                  connectionCtx : connectionCtx
                } ).execute();
          }
        } ).execute();
      }
    } ).execute();
  },


  _enableRule : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
      connectionCtx : connectionCtx,
      notificationRule : notificationRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _disableRule : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule', {
      connectionCtx : connectionCtx,
      notificationRule : notificationRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _deleteRule : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
      connectionCtx : connectionCtx,
      notificationRule : notificationRule,
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _editRule : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
      connectionCtx : connectionCtx,
      notificationRuleId : notificationRule.get( 'id' ),
      /*
       * Upon success refresh the notifications tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _cloneRule : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.SelectTrigger', {
      connectionCtx : connectionCtx,
      title : QDesk.Msg.get(
          'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesViewController.selectTriggerWindowTitle' ),
      preselectedTriggerGroupName : notificationRule.get( 'triggerGroupName' ),
      preselectedTriggerName : notificationRule.get( 'triggerName' ),
      /*
       * Upon trigger selection, open the Add job chain window.
       */
      success : function ( trigger )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
          connectionCtx : connectionCtx,
          notificationRuleId : notificationRule.get( 'id' ),
          triggerGroupName : trigger.get( 'group' ).name,
          triggerName : trigger.get( 'name' ),
          /*
           * Upon success refresh the notifications tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid',
                {
                  connectionCtx : connectionCtx
                } ).execute();
          }
        } ).execute();
      }
    } ).execute();
  },


  _viewRuleAuditTrail : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail', {
      connectionCtx : connectionCtx,
      notificationRuleId : notificationRule.get( 'id' )
    } ).execute();
  },


  _viewRuleMessages : function ( notificationRule )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
      connectionCtx : connectionCtx,
      creatorId : notificationRule.get( 'id' ),
      creatorType : QDesk.model.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
    } ).execute();
  }
} );