/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm', {
  extend : 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.store.enums.scheduler.quartz.QuartzCronTriggerMisfireInstructionStore',

    'QDesk.util.constant.DateFormat'
  ],

  alias : 'widget.QuartzEditCronTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    me.callParent( [ config ] );
  },


  /**
   * @private
   * @override
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    if ( trigger.get( 'timeZone' ) != null )
    {
      me.getForm().setValues( {
        timeZone : trigger.get( 'timeZone' ).id
      } );
    }
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    var jvmAgentInfo = qdesk.connectionCtx.getJvmAgentInfo();
    //var schedulerInfo = qdesk.connectionCtx.getSchedulerInfo();

    var timeZoneStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.common.TimeZone'
    } );

    timeZoneStore.loadData( jvmAgentInfo.getAvailableTimeZones() );

    return [
      {
        xtype : 'textfield',
        itemId : 'cronExpression',
        name : 'cronExpression',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldLabelCronExpression' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldTooltipCronExpression' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldEmptyTxtCronExpression' ),
        allowBlank : false
      },

      {
        xtype : 'displayfield',
        itemId : 'cronExpressionPrepareLink',
        fieldLabel : '',
        hideEmptyLabel : false,
        margin : '-5 0 5 0',
        renderer : function ()
        {
          return '<a href="http://www.cronmaker.com/" class="newWindow" rel="noopener noreferrer" target="cronMaker">'
              + QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldTextCronExpressionPrepareLink' )
              + '</a>';
        }
      },

      {
        xtype : 'combo',
        itemId : 'timeZone',
        name : 'timeZone',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldLabelTimeZone' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldTooltipTimeZone' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm.fieldEmptyTxtTimeZone' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : timeZoneStore,
        valueField : 'id',
        displayField : 'displayName',
        value : jvmAgentInfo.getTimeZone().id
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.QuartzCronTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.model.scheduler.quartz.QuartzCronTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );
  }
} );

