/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm', {
  extend : 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.quartz.QuartzRepeatIntervalUnit',

    'QDesk.store.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstructionStore',
    'QDesk.store.enums.scheduler.quartz.QuartzDateIntervalTriggerRepeatIntervalUnitStore',

    'QDesk.util.CommonUtils',
    'QDesk.util.constant.DateFormat'
  ],

  alias : 'widget.QuartzEditDateIntervalTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    me.callParent( [ config ] );
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    //var jvmAgentInfo = qdesk.connectionCtx.getJvmAgentInfo();
    var schedulerInfo = qdesk.connectionCtx.getSchedulerInfo();

    var repeatIntervalUnitStore = Ext.create(
        'QDesk.store.enums.scheduler.quartz.QuartzDateIntervalTriggerRepeatIntervalUnitStore', {
          qdesk : {
            schedulerVersion : schedulerInfo.getSchedulerVersion()
          }
        } );

    return [
      {
        xtype : 'combo',
        itemId : 'repeatIntervalUnit',
        name : 'repeatIntervalUnit',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldLabelRepeatIntervalUnit' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldTooltipRepeatIntervalUnit' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldEmptyTxtRepeatIntervalUnit' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : repeatIntervalUnitStore,
        valueField : 'value',
        displayField : 'valueLabel'
      },
      {
        xtype : 'numberfield',
        itemId : 'repeatInterval',
        name : 'repeatInterval',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldLabelRepeatInterval' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldTooltipRepeatInterval' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm.fieldEmptyTxtRepeatInterval' ),
        minValue : 0,
        allowBlank : false
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.model.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );

    me.down( '#repeatIntervalUnit' ).setValue( QDesk.model.scheduler.quartz.QuartzRepeatIntervalUnit.DAY );
    me.down( '#repeatInterval' ).setValue( 1 );
  }
} );
