/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.groups.GroupsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.security.groups.GroupStore',

    'QDesk.util.UxUtils',

    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.GroupsGrid',

  cls : 'groupsGrid',

  viewConfig : {
    stripeRows : true
  },

  scrollable : true,

  border : false,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var store = Ext.create( 'QDesk.store.security.groups.GroupStore' );

    Ext.apply( config, {
      store : store,
      qdesk : {
        gridId : 'groupsGrid'
      },

      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addGroupButton',
          scale : 'small',
          iconCls : 'addGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipAddGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.GROUP_ADD ],
          qdNotAuthorizedIconCls : 'addGroupNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'deleteGroupButton',
          scale : 'small',
          iconCls : 'deleteGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipDeleteGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.GROUP_DELETE ],
          qdNotAuthorizedIconCls : 'deleteGroupNAUTH'
        },
        '-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          emptyText : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.emptyTextFieldFilterQueryText' ),
          store : store,
          paramName : 'filterQuery',
          width : 150,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'systemColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.textSystemColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipSystemColumn' ),
          dataIndex : 'system',
          sortable : false,
          groupable : false,
          width : 65,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;

            if ( value )
            {
              iconClass = 'tick';
              tooltipText = QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.tooltipSystem' );

              // img src must be present in order to get rid of the image border that is shown in Chrome
              return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                  ' nonClickable" data-qtip="' + Ext.String.htmlEncode( tooltipText ) + '"/>';
            }
            else
            {
              return '';
            }
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Refreshes the Groups grid.
   */
  refresh : function ()
  {
    var me = this;
    var store = me.getStore();

    var filterQueryText = me.down( '#filterQueryText' ).getValue();

    var extraStoreParams = {};
    if ( !Ext.isEmpty( filterQueryText ) )
    {
      Ext.apply( extraStoreParams, {
        filterQuery : filterQueryText
      } );
    }

    store.getProxy().extraParams = extraStoreParams;
    store.loadPage( 1 );
  },


  /**
   * Invoked to create a context menu for the specified group stub.
   * @param groupStub a group stub.
   * @private
   */
  _createRowCtxMenu : function ( groupStub )
  {
    var me = this;

    var isSystemGroup = groupStub.get( 'system' );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // // lock group
        // {
        //   text : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.groupCtxMenuLock' ),
        //   iconCls : 'lockGroup',
        //   // system groups or groups with non-ACTIVE account status cannot be locked (manually from the GUI)
        //   disabled : isSystemGroup || accountStatus != QDesk.enums.security.GroupAccountStatus.ACTIVE,
        //   qdRequiresAnyPermissions : [ QDesk.model.security.Permission.GROUP_LOCK_UPDATE ],
        //   qdNotAuthorizedIconCls : 'lockGroupNAUTH',
        //   handler : function ()
        //   {
        //     me.fireEvent( 'lockGroup', groupStub );
        //   }
        // },
        // // unlock group
        // {
        //   text : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.groupCtxMenuUnlock' ),
        //   iconCls : 'unlockGroup',
        //   // system groups or groups with non-LOCKED account status cannot be unlocked
        //   disabled : isSystemGroup || accountStatus != QDesk.enums.security.GroupAccountStatus.LOCKED,
        //   qdRequiresAnyPermissions : [ QDesk.model.security.Permission.GROUP_LOCK_UPDATE ],
        //   qdNotAuthorizedIconCls : 'unlockGroupNAUTH',
        //   handler : function ()
        //   {
        //     me.fireEvent( 'unlockGroup', groupStub );
        //   }
        // },
        // '-',
        // delete group
        {
          text : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.groupCtxMenuDelete' ),
          iconCls : 'deleteGroup',
          // system groups cannot be deleted
          disabled : isSystemGroup,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.GROUP_DELETE ],
          qdNotAuthorizedIconCls : 'deleteGroupNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteGroup', groupStub );
          }
        }
      ]
    } );
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var store = me.getStore();

    // refresh button
    var refreshButton = me.down( '#refreshButton' );
    refreshButton.on( 'click',
        function ()
        {
          me.refresh();
        } );

    // add group button
    var addGroupButton = me.down( '#addGroupButton' );
    addGroupButton.on( 'click',
        function ()
        {
          // unselect a selected records in the grid if there is such a record
          me.getSelectionModel().deselectAll();

          me.fireEvent( 'addGroup' );
        } );

    // delete group button
    var deleteGroupButton = me.down( '#deleteGroupButton' );
    deleteGroupButton.on( 'click',
        function ()
        {
          var groupStub = me.getSelectionModel().getSelection()[ 0 ];
          me.fireEvent( 'deleteGroup', groupStub );
        } );

    var filterQueryText = me.down( '#filterQueryText' );

    store.on( 'beforeload', function ( store, operation )
    {
      refreshButton.disable();
      addGroupButton.disable();
      deleteGroupButton.disable();
      filterQueryText.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      me.getSelectionModel().deselectAll();
    } );

    store.on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      refreshButton.enable();

      if ( successful )
      {
        addGroupButton.enable();
        filterQueryText.enable();

        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        store.removeAll();

        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsGrid.emptyTextErrorLoadingData' );

        view.refresh();

        QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
      }
    } );

    me.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length == 0 )
          {
            deleteGroupButton.disable();

            me.fireEvent( 'groupUnselected' );
          }
          else
          {
            var groupStub = selectedRecords[ 0 ];

            // enable delete button if the group is NOT a system group
            if ( !groupStub.get( 'system' ) )
            {
              deleteGroupButton.enable();
            }

            me.fireEvent( 'groupSelected', groupStub );
          }
        } );

    // when the grid has been rendered, load the data
    me.on( 'afterrender', function ()
    {
      me.refresh();
    } )
  }
} );
