/*
 * Copyright (c) 2013-2022 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.top.TopContainer', {
  extend : 'Ext.container.Container',

  requires : [
    'QDesk.view.widget.WLocaleCombo',
    'QDesk.view.work.top.LicenseInfoPanel',
    'QDesk.view.work.top.TopButtonsContainer'
  ],

  alias : 'widget.TopContainer',

  cls : 'topContainer',

  layout : {
    type : 'border'
  },

  defaults : {
    border : false,
    bodyStyle : 'background: transparent;'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.product )
      Ext.Error.raise( 'Required config parameter missing: qdesk.product' );

    if ( !config.qdesk.license )
      Ext.Error.raise( 'Required config parameter missing: qdesk.license' );

    var productInfo = config.qdesk.product;
    var licenseInfo = config.qdesk.license;

    // we do not want the work panel to cache the qdesk data
    delete config.qdesk;

    Ext.apply( config, {
      items : [
        {
          xtype : 'container',
          region : 'west',
          bodyStyle : 'background: transparent;',

          layout : {
            type : 'hbox',
            pack : 'end',
            align : 'middle'
          },

          defaults : {
            border : false,
            bodyStyle : 'background: transparent;'
          },

          items : [
            {
              xtype : 'container',

              layout : {
                type : 'table',
                columns : 2
              },

              defaults : {
                border : false,
                bodyStyle : 'background: transparent;'
                //style : 'border: 1px solid red'
              },

              items : [
                {
                  xtype : 'container',
                  itemId : 'productLogo',
                  cls : 'productLogo',
                  width : 200,
                  minWidth : 200,
                  height : 40
                },
                {
                  xtype : 'container',
                  itemId : 'customizableTopLeftLogo',
                  cls : 'customizableTopLeftLogo',
                  width : 300,
                  minWidth : 300,
                  height : 40
                }
              ]
            }
          ]
        },


        {
          xtype : 'container',
          region : 'center',

          layout : 'ux.center',

          defaults : {
            border : false,
            bodyStyle : 'background: transparent;'
            //style : 'border: 1px solid red'
          },

          items : [
            {
              xtype : 'container',
              itemId : 'customizableTopCenterLogo',
              cls : 'customizableTopCenterLogo',
              width : 300,
              minWidth : 300,
              height : 40
            }
          ]
        },


        {
          xtype : 'container',
          region : 'east',

          layout : {
            type : 'hbox',
            pack : 'end',
            align : 'middle'
          },

          defaults : {
            border : false,
            bodyStyle : 'background: transparent;'
            //style : 'border: 1px solid red'
          },

          items : [
            {
              xtype : 'container',

              layout : {
                type : 'table',
                columns : 3
              },

              defaults : {
                border : false,
                bodyStyle : 'background: transparent;'
                // style : 'border: 1px solid red'
              },

              items : [
                // {
                //   xtype : 'WLocaleCombo',
                //   itemId : 'switchLocaleCombo',
                //   hidden : true,  // hide locale selection combo (currently we only support the English locale anyway)
                //   width : 100,
                //   margin : '0 10 0 0'
                // },

                {
                  xtype : 'container',
                  itemId : 'customizableTopRightLogo',
                  cls : 'customizableTopRightLogo',
                  width : 300,
                  minWidth : 300,
                  height : 40
                },

                {
                  xtype : 'LicenseInfoPanel',
                  itemId : 'licenseInfoPanel',
                  qdesk : {
                    product : productInfo,
                    license : licenseInfo
                  }
                },

                {
                  xtype : 'TopButtonsContainer',
                  itemId : 'topButtonsContainer'
                }
              ]
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.on( 'resize', function ( container, width, height )
    {
      me._maybeHideLogos( width );
    }, me );

    me.on( 'afterrender', function ( container )
    {
      me._maybeHideLogos( me.getEl().getWidth( false ) );
    }, me );

    me.callParent();
  },


  _maybeHideLogos : function ( containerWidth )
  {
    var me = this;

    var logoCnt = me.down( '#productLogo' );
    var customizableTopLeftLogoCnt = me.down( '#customizableTopLeftLogo' );
    var customizableTopCenterLogoCnt = me.down( '#customizableTopCenterLogo' );
    var customizableTopRightLogoCnt = me.down( '#customizableTopRightLogo' );

    //QDesk.Log.info( 'Top container width: ' + containerWidth );

    if ( containerWidth < 1590 )
      customizableTopRightLogoCnt.hide();

    if ( containerWidth < 1290 )
      customizableTopCenterLogoCnt.hide();

    if ( containerWidth < 970 )
      customizableTopLeftLogoCnt.hide();

    if ( containerWidth < 640 )
      logoCnt.hide();

    if ( containerWidth >= 1590 )
      customizableTopRightLogoCnt.show();

    if ( containerWidth >= 1290 )
      customizableTopCenterLogoCnt.show();

    if ( containerWidth >= 970 )
      customizableTopLeftLogoCnt.show();

    if ( containerWidth >= 640 )
      logoCnt.show();
  }
} );
